@@ -1,4 +1,4 @@
-#!/bin/bash
+#! /bin/sh
 # -----------------------------------------------------------------------------
 # jruby.sh - Start Script for the JRuby interpreter
 #
@@ -11,13 +11,12 @@
 #
 # -----------------------------------------------------------------------------
 
-cygwin=false
-
-# ----- Identify OS we are running under --------------------------------------
-case "`uname`" in
-  CYGWIN*) cygwin=true;;
-  Darwin) darwin=true;;
-esac
+# ********************************** NOTE *************************************
+# This script is provided as a convenience for users on systems that do not
+# have the "bash" shell available. It does not support all the flags the bash
+# and native executables do, but should be complete enough for most users.
+# Improvements are welcome.
+# *****************************************************************************
 
 # ----- Verify and Set Required Environment Variables -------------------------
 
@@ -53,14 +52,14 @@ fi
 
 JRUBY_OPTS_SPECIAL="--ng" # space-separated list of special flags
 unset JRUBY_OPTS_TEMP
-function process_special_opts {
+process_special_opts() {
     case $1 in
         --ng) nailgun_client=true;;
         *) break;;
     esac
 }
-for opt in ${JRUBY_OPTS[@]}; do
-    for special in ${JRUBY_OPTS_SPECIAL[@]}; do
+for opt in ${JRUBY_OPTS}; do
+    for special in ${JRUBY_OPTS_SPECIAL}; do
         if [ $opt != $special ]; then
             JRUBY_OPTS_TEMP="${JRUBY_OPTS_TEMP} $opt"
         else
@@ -74,18 +73,17 @@ for opt in ${JRUBY_OPTS[@]}; do
 done
 JRUBY_OPTS=${JRUBY_OPTS_TEMP}
 
-if [ -z "$JAVA_HOME" ] ; then
-  JAVA_CMD='java'
-else
-  if $cygwin; then
-    JAVA_HOME=`cygpath -u "$JAVA_HOME"`
+if [ -z "$JAVACMD" ] ; then
+  if [ -z "$JAVA_HOME" ] ; then
+    JAVACMD='java'
+  else
+    JAVACMD="$JAVA_HOME/bin/java"
   fi
-  JAVA_CMD="$JAVA_HOME/bin/java"
 fi
 
 # If you're seeing odd exceptions, you may have a bad JVM install.
 # Uncomment this and report the version to the JRuby team along with error.
-#$JAVA_CMD -version
+#$JAVACMD -version
 
 JRUBY_SHELL=/bin/sh
 
@@ -109,10 +107,6 @@ for j in "$JRUBY_HOME"/lib/jruby.jar "$JRUBY_HOME"/lib/jruby-complete.jar; do
     JRUBY_ALREADY_ADDED=true
 done
 
-if $cygwin; then
-    JRUBY_CP=`cygpath -p -w "$JRUBY_CP"`
-fi
-
 # ----- Set Up The System Classpath -------------------------------------------
 
 if [ "$JRUBY_PARENT_CLASSPATH" != "" ]; then
@@ -134,15 +128,8 @@ else
         fi
     done
 
-    if $cygwin; then
-        CP=`cygpath -p -w "$CP"`
-    fi
 fi
 
-if $cygwin; then
-    # switch delimiter only after building Unix style classpaths
-    CP_DELIMITER=";"
-fi
 
 # ----- Execute The Requested Command -----------------------------------------
 
@@ -151,16 +138,20 @@ if [ -z "$JAVA_MEM" ] ; then
 fi
 
 if [ -z "$JAVA_STACK" ] ; then
-  JAVA_STACK=-Xss1024k
+  JAVA_STACK=-Xss2048k
 fi
 
-JAVA_VM=-client
+if [ -z "$JAVA_VM" ]; then
+  JAVA_VM=-client
+fi
 JAVA_ENCODING=""
 
-declare -a java_args
-declare -a ruby_args
+#declare -a java_args
+#declare -a ruby_args
 
-java_class=org.jruby.Main
+JAVA_CLASS_JRUBY_MAIN=org.jruby.Main
+java_class=$JAVA_CLASS_JRUBY_MAIN
+JAVA_CLASS_NGSERVER=com.martiansoftware.nailgun.NGServer
 
 # Split out any -J argument for passing to the JVM.
 # Scanning for args is aborted by '--'.
@@ -175,11 +166,11 @@ do
         elif [ "${val:0:4}" = "-Xss" ]; then
             JAVA_STACK=$val
         elif [ "${val}" = "" ]; then
-            $JAVA_CMD -help
+            $JAVACMD -help
             echo "(Prepend -J in front of these options when using 'jruby' command)" 
             exit
         elif [ "${val}" = "-X" ]; then
-            $JAVA_CMD -X
+            $JAVACMD -X
             echo "(Prepend -J in front of these options when using 'jruby' command)" 
             exit
         elif [ "${val}" = "-classpath" ]; then
@@ -196,32 +187,47 @@ do
             elif [ "${val:0:16}" = "-Dfile.encoding=" ]; then
                 JAVA_ENCODING=$val
             fi
-            java_args=("${java_args[@]}" "${1:2}")
+            java_args="${java_args} ${1:2}"
         fi
         ;;
+     # Pass -X... and -X? search options through
+     -X*\.\.\.|-X*\?)
+        ruby_args="${ruby_args} $1" ;;
+     -X+T)
+      echo "error: -X+T isn't supported in the shell launcher"
+      exit 1
+      ;;
+     -Xclassic)
+      ;;
+     # Match -Xa.b.c=d to translate to -Da.b.c=d as a java option
+     -X*)
+     val=${1:2}
+     if expr "$val" : '.*[.]' > /dev/null; then
+       java_args="${java_args} -Djruby.${val}"
+     else
+       ruby_args="${ruby_args} -X${val}"
+     fi
+     ;;
      # Match switches that take an argument
-     -C|-e|-I|-S) ruby_args=("${ruby_args[@]}" "$1" "$2"); shift ;;
+     -C|-e|-I|-S) ruby_args="${ruby_args} $1 $2"; shift ;;
      # Match same switches with argument stuck together
-     -e*|-I*|-S*) ruby_args=("${ruby_args[@]}" "$1" ) ;;
+     -e*|-I*|-S*) ruby_args="${ruby_args} $1" ;;
      # Run with JMX management enabled
      --manage)
-        java_args=("${java_args[@]}" "-Dcom.sun.management.jmxremote")
-        java_args=("${java_args[@]}" "-Djruby.management.enabled=true") ;;
+        java_args="${java_args} -Dcom.sun.management.jmxremote"
+        java_args="${java_args} -Djruby.management.enabled=true" ;;
      # Don't launch a GUI window, no matter what
      --headless)
-        java_args=("${java_args[@]}" "-Djava.awt.headless=true") ;;
+        java_args="${java_args} -Djava.awt.headless=true" ;;
      # Run under JDB
      --jdb)
         if [ -z "$JAVA_HOME" ] ; then
-          JAVA_CMD='jdb'
+          JAVACMD='jdb'
         else
-          if $cygwin; then
-            JAVA_HOME=`cygpath -u "$JAVA_HOME"`
-          fi
-          JAVA_CMD="$JAVA_HOME/bin/jdb"
+          JAVACMD="$JAVA_HOME/bin/jdb"
         fi 
-        java_args=("${java_args[@]}" "-sourcepath" "$JRUBY_HOME/lib/ruby/1.8:.")
-        JRUBY_OPTS=("${JRUBY_OPTS[@]}" "-X+C") ;;
+        java_args="${java_args} -sourcepath $JRUBY_HOME/lib/ruby/1.9:."
+        JRUBY_OPTS="${JRUBY_OPTS} -X+C" ;;
      --client)
         JAVA_VM=-client ;;
      --server)
@@ -229,18 +235,20 @@ do
      --noclient)         # JRUBY-4296
         unset JAVA_VM ;; # For IBM JVM, neither '-client' nor '-server' is applicable
      --sample)
-        java_args=("${java_args[@]}" "-Xprof") ;;
+        java_args="${java_args} -Xprof" ;;
      --ng-server)
         # Start up as Nailgun server
-        java_class=com.martiansoftware.nailgun.NGServer
+        java_class=$JAVA_CLASS_NGSERVER
         VERIFY_JRUBY=true ;;
+     --no-bootclasspath)
+        NO_BOOTCLASSPATH=true ;;
      --ng)
         # Use native Nailgun client to toss commands to server
         process_special_opts "--ng" ;;
      # Abort processing on the double dash
      --) break ;;
      # Other opts go to ruby
-     -*) ruby_args=("${ruby_args[@]}" "$1") ;;
+     -*) ruby_args="${ruby_args} $1" ;;
      # Abort processing on first non-opt arg
      *) break ;;
     esac
@@ -248,51 +256,20 @@ do
 done
 
 # Force file.encoding to UTF-8 when on Mac, since Apple JDK defaults to MacRoman (JRUBY-3576)
-if [[ $darwin && -z "$JAVA_ENCODING" ]]; then
-  java_args=("${java_args[@]}" "-Dfile.encoding=UTF-8")
+if [[ -z "$JAVA_ENCODING" ]]; then
+  java_args="${java_args} -Dfile.encoding=UTF-8"
 fi
 
-# Add a property to report memory max
-JAVA_OPTS="$JAVA_OPTS $JAVA_VM -Djruby.memory.max=${JAVA_MEM:4} -Djruby.stack.max=${JAVA_STACK:4}"
-
 # Append the rest of the arguments
-ruby_args=("${ruby_args[@]}" "$@")
+ruby_args="${ruby_args} $@"
 
 # Put the ruby_args back into the position arguments $1, $2 etc
-set -- "${ruby_args[@]}"
+set -- "${ruby_args}"
 
-JAVA_OPTS="$JAVA_OPTS $JAVA_MEM $JAVA_STACK"
+JAVA_OPTS="$JAVA_OPTS $JAVA_VM $JAVA_MEM $JAVA_STACK"
 
-JFFI_BOOT=""
-if [ -d $JRUBY_HOME/lib/native/ ]; then
-  for d in $JRUBY_HOME/lib/native/*`uname -s`; do
-    if [ -z "$JFFI_BOOT" ]; then
-      JFFI_BOOT="$d"
-    else
-      JFFI_BOOT="$JFFI_BOOT:$d"
-    fi
-  done
-fi
-JFFI_OPTS="-Djffi.boot.library.path=$JFFI_BOOT"
+JFFI_OPTS="-Djffi.boot.library.path=$JRUBY_HOME/lib/jni"
 
-if $cygwin; then
-  JRUBY_HOME=`cygpath --mixed "$JRUBY_HOME"`
-  JRUBY_SHELL=`cygpath --mixed "$JRUBY_SHELL"`
-  
-  if [[ ( "${1:0:1}" = "/" ) && ( ( -f "$1" ) || ( -d "$1" )) ]]; then
-    win_arg=`cygpath -w "$1"`
-    shift
-    win_args=("$win_arg" "$@")
-    set -- "${win_args[@]}"
-  fi
-
-  # fix JLine to use UnixTerminal
-  stty -icanon min 1 -echo > /dev/null 2>&1
-  if [ $? = 0 ]; then
-    JAVA_OPTS="$JAVA_OPTS -Djline.terminal=jline.UnixTerminal"
-  fi
-
-fi
 
 if [ "$nailgun_client" != "" ]; then
   if [ -f $JRUBY_HOME/tool/nailgun/ng ]; then
@@ -302,16 +279,22 @@ if [ "$nailgun_client" != "" ]; then
     exit 1
   fi
 else
-if [ "$VERIFY_JRUBY" != "" ]; then
+if [ "$NO_BOOTCLASSPATH" != "" || "$VERIFY_JRUBY" != "" ]; then
   if [ "$PROFILE_ARGS" != "" ]; then
       echo "Running with instrumented profiler"
   fi
 
-  "$JAVA_CMD" $PROFILE_ARGS $JAVA_OPTS "$JFFI_OPTS" "${java_args[@]}" -classpath "$JRUBY_CP$CP_DELIMITER$CP$CP_DELIMITER$CLASSPATH" \
+  if [ $java_class = $JAVA_CLASS_NGSERVER -a -n "${JRUBY_OPTS}" ]; then
+    echo "warning: starting a nailgun server; discarding JRUBY_OPTS: ${JRUBY_OPTS}"
+    JRUBY_OPTS=''
+  fi
+
+
+  "$JAVACMD" $PROFILE_ARGS $JAVA_OPTS "$JFFI_OPTS" ${java_args} -classpath "$JRUBY_CP$CP_DELIMITER$CP$CP_DELIMITER$CLASSPATH" \
     "-Djruby.home=$JRUBY_HOME" \
     "-Djruby.lib=$JRUBY_HOME/lib" -Djruby.script=jruby \
     "-Djruby.shell=$JRUBY_SHELL" \
-    $java_class $JRUBY_OPTS "$@"
+    $java_class $JRUBY_OPTS $@
 
   # Record the exit status immediately, or it will be overridden.
   JRUBY_STATUS=$?
@@ -322,33 +305,14 @@ if [ "$VERIFY_JRUBY" != "" ]; then
       rm profile.txt
   fi
 
-  if $cygwin; then
-    stty icanon echo > /dev/null 2>&1
-  fi
 
   exit $JRUBY_STATUS
 else
-  if $cygwin; then
-    # exec doed not work correctly with cygwin bash
-    "$JAVA_CMD" $JAVA_OPTS "$JFFI_OPTS" "${java_args[@]}" -Xbootclasspath/a:"$JRUBY_CP" -classpath "$CP$CP_DELIMITER$CLASSPATH" \
-      "-Djruby.home=$JRUBY_HOME" \
-      "-Djruby.lib=$JRUBY_HOME/lib" -Djruby.script=jruby \
-      "-Djruby.shell=$JRUBY_SHELL" \
-      $java_class $JRUBY_OPTS "$@"
-
-    # Record the exit status immediately, or it will be overridden.
-    JRUBY_STATUS=$?
-
-    stty icanon echo > /dev/null 2>&1
-
-    exit $JRUBY_STATUS
-  else
-    exec "$JAVA_CMD" $JAVA_OPTS "$JFFI_OPTS" "${java_args[@]}" -Xbootclasspath/a:"$JRUBY_CP" -classpath "$CP$CP_DELIMITER$CLASSPATH" \
-      "-Djruby.home=$JRUBY_HOME" \
-      "-Djruby.lib=$JRUBY_HOME/lib" -Djruby.script=jruby \
-      "-Djruby.shell=$JRUBY_SHELL" \
-      $java_class $JRUBY_OPTS "$@"
-  fi
+  exec $JAVACMD $JAVA_OPTS $JFFI_OPTS ${java_args} -Xbootclasspath/a:$JRUBY_CP -classpath $CP$CP_DELIMITER$CLASSPATH \
+      -Djruby.home=$JRUBY_HOME \
+      -Djruby.lib=$JRUBY_HOME/lib -Djruby.script=jruby \
+      -Djruby.shell=$JRUBY_SHELL \
+      $java_class $JRUBY_OPTS $@
 fi
 fi
 
