@@ -17,7 +17,7 @@
  * Copyright (C) 2004 Stefan Matthias Aust <sma@3plus4.de>
  * Copyright (C) 2007 Ola Bini <ola@ologix.com>
  * Copyright (C) 2008-2009 Joseph LaFata <joe@quibb.org>
- * 
+ *
  * Alternatively, the contents of this file may be used under the terms of
  * either of the GNU General Public License Version 2 or later (the "GPL"),
  * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
@@ -34,60 +34,142 @@ package org.jruby;
 
 import java.io.File;
 import java.io.IOException;
-import static org.jruby.RubyEnumerator.enumeratorize;
 
-import org.jruby.anno.JRubyMethod;
-import org.jruby.exceptions.RaiseException;
 import jnr.posix.FileStat;
 import jnr.posix.util.Platform;
+
+import static org.jruby.RubyEnumerator.enumeratorize;
+import static org.jruby.runtime.Visibility.PRIVATE;
+
+import org.jruby.anno.FrameField;
+import org.jruby.anno.JRubyMethod;
+import org.jruby.exceptions.RaiseException;
+import org.jruby.internal.runtime.GlobalVariable;
 import org.jruby.runtime.Block;
 import org.jruby.runtime.IAccessor;
+import org.jruby.runtime.ObjectAllocator;
 import org.jruby.runtime.ThreadContext;
 import org.jruby.runtime.builtin.IRubyObject;
 import org.jruby.util.ByteList;
-import static org.jruby.CompatVersion.*;
-import org.jruby.internal.runtime.GlobalVariable;
+import org.jruby.util.TypeConverter;
+
+public class RubyArgsFile extends RubyObject {
+
+    public RubyArgsFile(Ruby runtime, RubyClass metaClass) {
+        super(runtime, metaClass);
+    }
+
+    public static void initArgsFile(final Ruby runtime) {
+        RubyClass argfClass = runtime.defineClass("ARGFClass", runtime.getObject(), ARGF_ALLOCATOR);
+        argfClass.includeModule(runtime.getEnumerable());
+
+        argfClass.defineAnnotatedMethods(RubyArgsFile.class);
+
+        IRubyObject argsFile = argfClass.newInstance(runtime.getCurrentContext(), new IRubyObject[] { null }, (Block) null);
+
+        runtime.setArgsFile(argsFile);
+        runtime.getGlobalVariables().defineReadonly("$<", new ArgsFileAccessor(runtime), GlobalVariable.Scope.GLOBAL);
+        runtime.defineGlobalConstant("ARGF", argsFile);
+        runtime.defineReadonlyVariable("$FILENAME", runtime.newString("-"), GlobalVariable.Scope.GLOBAL);
+    }
+
+
+    private static class ArgsFileAccessor implements IAccessor {
 
-public class RubyArgsFile {
-    private static final class ArgsFileData {
         private final Ruby runtime;
-        public ArgsFileData(Ruby runtime) {
+        ArgsFileAccessor(Ruby runtime) { this.runtime = runtime; }
+
+        public IRubyObject getValue() {
+            return runtime.getArgsFile();
+        }
+
+        public IRubyObject setValue(IRubyObject newValue) {
+            throw new UnsupportedOperationException("Not supported yet.");
+        }
+    }
+
+    private static final ObjectAllocator ARGF_ALLOCATOR = new ObjectAllocator() {
+        @Override
+        public IRubyObject allocate(Ruby runtime, RubyClass klass) {
+            return new RubyArgsFile(runtime, klass);
+        }
+    };
+
+    @JRubyMethod(name = "initialize", visibility = PRIVATE, rest = true)
+    public IRubyObject initialize(ThreadContext context, IRubyObject[] args) {
+        final Ruby runtime = context.runtime;
+        final RubyArray argv;
+        if (args.length == 1 && args[0] == null) {
+            argv = runtime.getObject().getConstant("ARGV").convertToArray();
+        } else {
+            argv = runtime.newArray(args);
+        }
+
+        // ARGF is intended to be a singleton from a Ruby perspective but it is still
+        // possible for someone to ARGF.class.new.  We do still want a global view of
+        // ARGF otherwise getline and rewind in IO would have to keep track of the n
+        // instances in play.  So all instances will share
+        if (runtime.getArgsFile() == null) {
+            dataWrapStruct(new ArgsFileData(runtime, argv));
+        } else {
+            ArgsFileData data = (ArgsFileData) runtime.getArgsFile().dataGetStruct();
+            dataWrapStruct(data);
+            data.setArgs(argv);
+        }
+        return this;
+    }
+
+    public static final class ArgsFileData {
+
+        private final Ruby runtime;
+        private RubyArray argv;
+        public IRubyObject currentFile;
+        private boolean inited = false;
+        public int next_p = 0;
+        public boolean binmode = false;
+
+        public ArgsFileData(Ruby runtime, RubyArray argv) {
             this.runtime = runtime;
+            this.argv = argv;
             this.currentFile = runtime.getNil();
         }
 
-        public IRubyObject currentFile;
-        public int currentLineNumber;
-        public int minLineNumber;
-        private boolean inited = false;
-        public int next_p = 0;
+        @Deprecated
+        public void setCurrentLineNumber(Ruby runtime, int linenumber) {
+            runtime.setCurrentLine(linenumber);
+        }
+
+        void setArgs(RubyArray argv) {
+            inited = false;
+            this.argv = argv;
+        }
 
         public boolean next_argv(ThreadContext context) {
-            RubyArray args = (RubyArray)runtime.getGlobalVariables().get("$*");
             if (!inited) {
-                if (args.getLength() > 0) {
-                    next_p = 1;
-                } else {
-                    next_p = -1;
-                }
+                next_p = argv.getLength() > 0 ? 1 : -1;
                 inited = true;
-                currentLineNumber = 0;
+                runtime.setCurrentLine(0);
+            } else {
+                if (argv.isNil()) {
+                    next_p = -1;
+                } else if (next_p == -1 && argv.getLength() > 0) {
+                    next_p = 1;
+                }
             }
 
+            IRubyObject $FILENAME = runtime.getGlobalVariables().get("$FILENAME");
+
             if (next_p == 1) {
-                next_p = 0;
-                if (args.getLength() > 0) {
-                    IRubyObject arg = args.shift(context);
-                    RubyString filename = (RubyString)((RubyObject)arg).to_s();
-                    ByteList filenameBytes = filename.getByteList();
-                    if (!filename.op_equal(context, (RubyString) runtime.getGlobalVariables().get("$FILENAME")).isTrue()) {
+                if (argv.getLength() > 0) {
+                    final RubyString filename = argv.shift(context).convertToString();
+                    if ( ! filename.op_equal(context, $FILENAME).isTrue() ) {
                         runtime.defineReadonlyVariable("$FILENAME", filename, GlobalVariable.Scope.GLOBAL);
                     }
 
-                    if (filenameBytes.length() == 1 && filenameBytes.get(0) == '-') {
+                    if (filenameEqlDash(filename)) {
                         currentFile = runtime.getGlobalVariables().get("$stdin");
                     } else {
-                        currentFile = RubyFile.open(context, runtime.getFile(), new IRubyObject[]{filename}, Block.NULL_BLOCK);
+                        currentFile = RubyFile.open(context, runtime.getFile(), new IRubyObject[]{ filename }, Block.NULL_BLOCK);
                         String extension = runtime.getInstanceConfig().getInPlaceBackupExtension();
                         if (extension != null) {
                             if (Platform.IS_WINDOWS) {
@@ -96,32 +178,34 @@ public class RubyArgsFile {
                                 inplaceEdit(context, filename.asJavaString(), extension);
                             }
                         }
-                        minLineNumber = currentLineNumber;
-                        currentFile.callMethod(context, "lineno=", context.runtime.newFixnum(currentLineNumber));
+                        if (binmode) ((RubyIO) currentFile).binmode();
                     }
+                    next_p = 0;
                 } else {
                     next_p = 1;
                     return false;
                 }
             } else if (next_p == -1) {
                 currentFile = runtime.getGlobalVariables().get("$stdin");
-                if(!runtime.getGlobalVariables().get("$FILENAME").asJavaString().equals("-")) {
+                if (!filenameEqlDash((RubyString) $FILENAME)) {
                     runtime.defineReadonlyVariable("$FILENAME", runtime.newString("-"), GlobalVariable.Scope.GLOBAL);
                 }
             }
-
             return true;
         }
 
+        private static boolean filenameEqlDash(final RubyString filename) {
+            final ByteList filenameBytes = filename.getByteList();
+            return filenameBytes.length() == 1 && filenameBytes.get(0) == '-';
+        }
+
+        public static ArgsFileData getArgsFileData(Ruby runtime) {
+            return (ArgsFileData) runtime.getArgsFile().dataGetStruct();
+        }
+
+        @Deprecated
         public static ArgsFileData getDataFrom(IRubyObject recv) {
-            ArgsFileData data = (ArgsFileData)recv.dataGetStruct();
-
-            if (data == null) {
-                data = new ArgsFileData(recv.getRuntime());
-                recv.dataWrapStruct(data);
-            }
-
-            return data;
+            return getArgsFileData(recv.getRuntime());
         }
 
         private void createNewFile(File file) {
@@ -135,7 +219,7 @@ public class RubyArgsFile {
         private void inplaceEditWindows(ThreadContext context, String filename, String extension) throws RaiseException {
             File file = new File(filename);
 
-            if (!extension.equals("")) {
+            if (extension.length() > 0) {
                 String backup = filename + extension;
                 File backupFile = new File(backup);
 
@@ -158,7 +242,7 @@ public class RubyArgsFile {
             File file = new File(filename);
             FileStat stat = runtime.getPosix().stat(filename);
 
-            if (!extension.equals("")) {
+            if (extension.length() > 0) {
                 file.renameTo(new File(filename + extension));
             } else {
                 file.delete();
@@ -171,148 +255,107 @@ public class RubyArgsFile {
             runtime.getGlobalVariables().set("$stdout", (RubyIO) RubyFile.open(context, runtime.getFile(),
                     new IRubyObject[]{runtime.newString(filename), runtime.newString("w")}, Block.NULL_BLOCK));
         }
-    }    
-    
-    public static void setCurrentLineNumber(IRubyObject recv, int newLineNumber) {
-        ArgsFileData data = ArgsFileData.getDataFrom(recv);
 
-        if (data != null) {
-            data.currentLineNumber = newLineNumber;
+        public boolean isCurrentFile(RubyIO io) {
+            return currentFile == io;
         }
     }
 
-    public static void initArgsFile(final Ruby runtime) {
-        RubyObject argsFile = new RubyObject(runtime, runtime.getObject());
+    @Deprecated
+    public static void setCurrentLineNumber(IRubyObject recv, int newLineNumber) {
+        recv.getRuntime().setCurrentLine(newLineNumber);
+    }
 
-        runtime.getEnumerable().extend_object(argsFile);
-
-        runtime.setArgsFile(argsFile);
-        runtime.getGlobalVariables().defineReadonly("$<", new IAccessor() {
-            @Override
-            public IRubyObject getValue() {
-                return runtime.getArgsFile();
-            }
-
-            @Override
-            public IRubyObject setValue(IRubyObject newValue) {
-                throw new UnsupportedOperationException("Not supported yet.");
-            }
-        }, GlobalVariable.Scope.GLOBAL);
-        runtime.defineGlobalConstant("ARGF", argsFile);
-        
-        RubyClass argfClass = argsFile.getMetaClass();
-        argfClass.defineAnnotatedMethods(RubyArgsFile.class);
-        runtime.defineReadonlyVariable("$FILENAME", runtime.newString("-"), GlobalVariable.Scope.GLOBAL);
+    @JRubyMethod(name = "argv")
+    public static IRubyObject argv(ThreadContext context, IRubyObject recv) {
+        return ArgsFileData.getArgsFileData(context.runtime).argv;
     }
 
     @JRubyMethod(name = {"fileno", "to_i"})
     public static IRubyObject fileno(ThreadContext context, IRubyObject recv) {
-        return ((RubyIO) getData(context, recv, "no stream").currentFile).fileno(context);
+        return getCurrentDataFile(context, "no stream").fileno(context);
     }
 
     @JRubyMethod(name = "to_io")
     public static IRubyObject to_io(ThreadContext context, IRubyObject recv) {
-        return getData(context, recv, "no stream").currentFile;
+        return getCurrentDataFile(context, "no stream");
     }
-    
-    @JRubyMethod(compat = RUBY1_9)
+
+    @JRubyMethod
     public static IRubyObject set_encoding(ThreadContext context, IRubyObject recv, IRubyObject encoding) {
-        return ((RubyIO) getData(context, recv, "no stream to set encoding").currentFile).set_encoding(context, encoding);
+        return getCurrentDataFile(context, "no stream to set encoding").set_encoding(context, encoding);
     }
-    
-    @JRubyMethod(compat = RUBY1_9)
+
+    @JRubyMethod
     public static IRubyObject set_encoding(ThreadContext context, IRubyObject recv, IRubyObject encoding, IRubyObject internalEncoding) {
-        return ((RubyIO) getData(context, recv, "no stream to set encoding").currentFile).set_encoding(context, encoding, internalEncoding);
+        return getCurrentDataFile(context, "no stream to set encoding").set_encoding(context, encoding, internalEncoding);
     }
-    
-    @JRubyMethod(compat = RUBY1_9)
+
+    @JRubyMethod
     public static IRubyObject set_encoding(ThreadContext context, IRubyObject recv, IRubyObject encoding, IRubyObject internalEncoding, IRubyObject options) {
-        return ((RubyIO) getData(context, recv, "no stream to set encoding").currentFile).set_encoding(context, encoding, internalEncoding, options);
+        return getCurrentDataFile(context, "no stream to set encoding").set_encoding(context, encoding, internalEncoding, options);
     }
-    
-    @JRubyMethod(compat = RUBY1_9)
+
+    @JRubyMethod
     public static IRubyObject internal_encoding(ThreadContext context, IRubyObject recv) {
-        return ((RubyIO) getData(context, recv, "no stream to set encoding").currentFile).internal_encoding(context);
+        return getCurrentDataFile(context, "no stream to set encoding").internal_encoding(context);
     }
-    
-    @JRubyMethod(compat = RUBY1_9)
+
+    @JRubyMethod
     public static IRubyObject external_encoding(ThreadContext context, IRubyObject recv) {
-        return ((RubyIO) getData(context, recv, "no stream to set encoding").currentFile).external_encoding(context);
-    }    
+        return getCurrentDataFile(context, "no stream to set encoding").external_encoding(context);
+    }
 
+    // MRI: argf_getline
     private static IRubyObject argf_getline(ThreadContext context, IRubyObject recv, IRubyObject[] args) {
-        ArgsFileData data = ArgsFileData.getDataFrom(recv);
+        ArgsFileData data = ArgsFileData.getArgsFileData(context.runtime);
 
-        boolean retry = true;
-        IRubyObject line = null;
-        while(retry) {
-            retry = false;
-            if (!data.next_argv(context)) return context.runtime.getNil();
+        while (true) {
+            if (!data.next_argv(context)) return context.nil;
 
-            line = data.currentFile.callMethod(context, "gets", args);
+            IRubyObject line = data.currentFile.callMethod(context, "gets", args);
 
             if (line.isNil() && data.next_p != -1) {
                 argf_close(context, data.currentFile);
                 data.next_p = 1;
-                retry = true;
+                continue;
             }
-        }
-        
-        if (!line.isNil()) {
-            context.runtime.setCurrentLine(data.currentLineNumber);
-        }
 
-        return line;
+            return line;
+        }
     }
 
     // ARGF methods
 
     /** Read a line.
-     * 
+     *
      */
-    @JRubyMethod(name = "gets", optional = 1)
+    @JRubyMethod(name = "gets", optional = 1, writes = FrameField.LASTLINE)
     public static IRubyObject gets(ThreadContext context, IRubyObject recv, IRubyObject[] args) {
-        ArgsFileData data = ArgsFileData.getDataFrom(recv);
-
-        if (!data.next_argv(context)) return context.runtime.getNil();
-
-        IRubyObject line;
-        if (!(data.currentFile instanceof RubyIO)) {
-            line = data.currentFile.callMethod(context, "gets", args);
-        } else {
-            line = argf_getline(context, recv, args);
-        }
-
-        context.setLastLine(line);
-        context.runtime.getGlobalVariables().set("$_", line);
-        
-        return line;
+        return context.setLastLine(argf_getline(context, recv, args));
     }
-    
+
     /** Read a line.
-     * 
+     *
      */
     @JRubyMethod(name = "readline", optional = 1)
     public static IRubyObject readline(ThreadContext context, IRubyObject recv, IRubyObject[] args) {
         IRubyObject line = gets(context, recv, args);
 
         if (line.isNil()) throw context.runtime.newEOFError();
-        
+
         return line;
     }
 
     @JRubyMethod(optional = 1)
     public static IRubyObject readlines(ThreadContext context, IRubyObject recv, IRubyObject[] args) {
-        ArgsFileData data = ArgsFileData.getDataFrom(recv);
         Ruby runtime = context.runtime;
-        
-        if (!data.next_argv(context)) {
-            return runtime.is1_9() ? runtime.newEmptyArray() : runtime.getNil();
-        }
-        if (!(data.currentFile instanceof RubyIO)) {
-            return data.currentFile.callMethod(context, "readlines", args);
-        }
-        
+        ArgsFileData data = ArgsFileData.getArgsFileData(runtime);
+
+        if (!data.next_argv(context)) return runtime.newEmptyArray();
+
+        if (!(data.currentFile instanceof RubyIO)) return data.currentFile.callMethod(context, "readlines", args);
+
         RubyArray ary = runtime.newArray();
         IRubyObject line;
         while(!(line = argf_getline(context, recv, args)).isNil()) {
@@ -323,17 +366,12 @@ public class RubyArgsFile {
 
     @JRubyMethod(optional = 1)
     public static IRubyObject to_a(ThreadContext context, IRubyObject recv, IRubyObject[] args) {
-        ArgsFileData data = ArgsFileData.getDataFrom(recv);
         Ruby runtime = context.runtime;
-        
-        if (!data.next_argv(context)) {
-            return runtime.is1_9() ? runtime.newEmptyArray() : runtime.getNil();
-        }
+        ArgsFileData data = ArgsFileData.getArgsFileData(runtime);
+
+        if (!data.next_argv(context)) return runtime.newEmptyArray();
+        if (!(data.currentFile instanceof RubyIO)) return data.currentFile.callMethod(context, "to_a", args);
 
-        if (!(data.currentFile instanceof RubyIO)) {
-            return data.currentFile.callMethod(context, "to_a", args);
-        }
-        
         RubyArray ary = runtime.newArray();
         IRubyObject line;
         while(!(line = argf_getline(context, recv, args)).isNil()) {
@@ -341,7 +379,7 @@ public class RubyArgsFile {
         }
         return ary;
     }
-    
+
     public static IRubyObject each_byte(ThreadContext context, IRubyObject recv, Block block) {
         IRubyObject bt;
 
@@ -373,7 +411,7 @@ public class RubyArgsFile {
     }
 
     public static IRubyObject each_charCommon(ThreadContext context, IRubyObject recv, Block block) {
-        ArgsFileData data = ArgsFileData.getDataFrom(recv);
+        ArgsFileData data = ArgsFileData.getArgsFileData(context.runtime);
         Ruby runtime = context.runtime;
         IRubyObject ch;
         while(!(ch = getc(context, recv)).isNil()) {
@@ -409,46 +447,44 @@ public class RubyArgsFile {
     /** Invoke a block for each line.
      *
      */
-    @JRubyMethod(name = {"each_line", "each"}, optional = 1)
     public static IRubyObject each_line(ThreadContext context, IRubyObject recv, IRubyObject[] args, Block block) {
         if (!block.isGiven()) return RubyEnumerator.enumeratorize(context.runtime, recv, "each_line");
-        ArgsFileData data = ArgsFileData.getDataFrom(recv);
-        if (!data.next_argv(context)) {
-            return context.runtime.getNil();
-        }
+        ArgsFileData data = ArgsFileData.getArgsFileData(context.runtime);
+
+        if (!data.next_argv(context)) return context.runtime.getNil();
 
         if (!(data.currentFile instanceof RubyIO)) {
             if (!data.next_argv(context)) return recv;
 
-            data.currentFile.callMethod(context, "each", new IRubyObject[0], block);
+            data.currentFile.callMethod(context, "each", NULL_ARRAY, block);
             data.next_p = 1;
         }
         IRubyObject str;
         while(!(str = argf_getline(context, recv, args)).isNil()) {
         	block.yield(context, str);
         }
-        
+
         return recv;
     }
-    
-    @JRubyMethod(compat = RUBY1_9, optional = 1)
+
+    @JRubyMethod(optional = 1)
     public static IRubyObject lines(ThreadContext context, IRubyObject recv, IRubyObject[] args, Block block) {
         return each_line(context, recv, args, block);
     }
 
-    @JRubyMethod(name = "each_line", optional = 1, compat = RUBY1_9)
+    @JRubyMethod(name = "each_line", optional = 1)
     public static IRubyObject each_line19(final ThreadContext context, IRubyObject recv, IRubyObject[] args, final Block block) {
         return block.isGiven() ? each_line(context, recv, args, block) : enumeratorize(context.runtime, recv, "each_line", args);
     }
 
-    @JRubyMethod(name = "each", optional = 1, compat = RUBY1_9)
+    @JRubyMethod(name = "each", optional = 1)
     public static IRubyObject each19(final ThreadContext context, IRubyObject recv, IRubyObject[] args, final Block block) {
         return block.isGiven() ? each_line(context, recv, args, block) : enumeratorize(context.runtime, recv, "each", args);
     }
 
     @JRubyMethod(name = "file")
     public static IRubyObject file(ThreadContext context, IRubyObject recv) {
-        ArgsFileData data = ArgsFileData.getDataFrom(recv);
+        ArgsFileData data = ArgsFileData.getArgsFileData(context.runtime);
 
         data.next_argv(context);
 
@@ -457,10 +493,11 @@ public class RubyArgsFile {
 
     @JRubyMethod(name = "skip")
     public static IRubyObject skip(IRubyObject recv) {
-        ArgsFileData data = ArgsFileData.getDataFrom(recv);
+        Ruby runtime = recv.getRuntime();
+        ArgsFileData data = ArgsFileData.getArgsFileData(runtime);
 
-        if (data.next_p != -1) {
-            argf_close(recv.getRuntime().getCurrentContext(), data.currentFile);
+        if (data.inited && data.next_p == 0) {
+            argf_close(runtime.getCurrentContext(), data.currentFile);
             data.next_p = 1;
         }
 
@@ -469,7 +506,7 @@ public class RubyArgsFile {
 
     public static void argf_close(ThreadContext context, IRubyObject file) {
         if(file instanceof RubyIO) {
-            ((RubyIO)file).ioClose(context.runtime);
+            ((RubyIO)file).rbIoClose(context.runtime);
         } else {
             file.callMethod(context, "close");
         }
@@ -477,216 +514,208 @@ public class RubyArgsFile {
 
     @JRubyMethod(name = "close")
     public static IRubyObject close(ThreadContext context, IRubyObject recv) {
-        ArgsFileData data = ArgsFileData.getDataFrom(recv);
+        ArgsFileData data = ArgsFileData.getArgsFileData(context.runtime);
 
         data.next_argv(context);
-        if (isClosed(context, data.currentFile)) {
-            throw context.runtime.newIOError("closed stream");
-        }
-        
+
+        if (data.currentFile == context.runtime.getGlobalVariables().get("$stdin")) return recv;
+
         argf_close(context, data.currentFile);
 
         if (data.next_p != -1) data.next_p = 1;
 
-        data.currentLineNumber = 0;
+        context.runtime.setCurrentLine(0);
+
         return recv;
     }
 
     @JRubyMethod(name = "closed?")
     public static IRubyObject closed_p(ThreadContext context, IRubyObject recv) {
-        ArgsFileData data = ArgsFileData.getDataFrom(recv);
+        ArgsFileData data = ArgsFileData.getArgsFileData(context.runtime);
 
         data.next_argv(context);
 
         return RubyBoolean.newBoolean(context.runtime, isClosed(context, data.currentFile));
     }
-    
-    private static boolean isClosed(ThreadContext context, IRubyObject currentFile) {
-        boolean closed = false;
 
-        if (!(currentFile instanceof RubyIO)) {
-            closed = currentFile.callMethod(context, "closed?").isTrue();
-        } else {
-            closed = ((RubyIO)currentFile).closed_p(context).isTrue();
-        }
-        return closed;
+    private static boolean isClosed(ThreadContext context, IRubyObject currentFile) {
+        if (!(currentFile instanceof RubyIO)) return currentFile.callMethod(context, "closed?").isTrue();
+
+        return ((RubyIO)currentFile).closed_p(context).isTrue();
     }
 
     @JRubyMethod(name = "binmode")
     public static IRubyObject binmode(ThreadContext context, IRubyObject recv) {
-        ArgsFileData data = getData(context, recv, "no stream");
-        
-        ((RubyIO)data.currentFile).binmode();
+        ArgsFileData data = ArgsFileData.getArgsFileData(context.runtime);
+
+        data.binmode = true;
+        if (!data.currentFile.isNil()) ((RubyIO) data.currentFile).binmode();
+
         return recv;
     }
-    
-    @JRubyMethod(name = "binmode?", compat = CompatVersion.RUBY1_9)
+
+    @JRubyMethod(name = "binmode?")
     public static IRubyObject op_binmode(ThreadContext context, IRubyObject recv) {
-        ArgsFileData data = getData(context, recv, "no stream");
-        
-        return ((RubyIO)data.currentFile).op_binmode(context);
+        return getCurrentDataFile(context, "no stream").op_binmode(context);
     }
 
     @JRubyMethod(name = "lineno")
     public static IRubyObject lineno(ThreadContext context, IRubyObject recv) {
-        return recv.getRuntime().newFixnum(ArgsFileData.getDataFrom(recv).currentLineNumber);
+        return context.runtime.newFixnum(context.runtime.getCurrentLine());
     }
 
     @JRubyMethod(name = "lineno=")
     public static IRubyObject lineno_set(ThreadContext context, IRubyObject recv, IRubyObject line) {
-        ArgsFileData data = ArgsFileData.getDataFrom(recv);
-        data.currentLineNumber = RubyNumeric.fix2int(line);
-        context.runtime.setCurrentLine(data.currentLineNumber);
+        context.runtime.setCurrentLine(RubyNumeric.fix2int(line));
+
         return recv.getRuntime().getNil();
     }
 
     @JRubyMethod(name = "tell", alias = {"pos"})
     public static IRubyObject tell(ThreadContext context, IRubyObject recv) {
-        ArgsFileData data = ArgsFileData.getDataFrom(recv);
-        if(!data.next_argv(context)) {
-            throw context.runtime.newArgumentError("no stream to tell");
-        }
-
-        return ((RubyIO)data.currentFile).pos(context);
+        return getCurrentDataFile(context, "no stream to tell").pos(context);
     }
 
     @JRubyMethod(name = "rewind")
     public static IRubyObject rewind(ThreadContext context, IRubyObject recv) {
-        ArgsFileData data = getData(context, recv, "no stream to rewind");
-        
-        RubyFixnum retVal = ((RubyIO)data.currentFile).rewind(context);
-        ((RubyIO)data.currentFile).lineno_set(context, context.runtime.newFixnum(0));
-        
-        data.minLineNumber = 0;
-        data.currentLineNumber = 0;
+        RubyIO currentFile = getCurrentDataFile(context, "no stream to rewind");
+
+        RubyFixnum retVal = currentFile.rewind(context);
+        currentFile.lineno_set(context, context.runtime.newFixnum(0));
+
         return retVal;
     }
 
     @JRubyMethod(name = {"eof"})
     public static IRubyObject eof(ThreadContext context, IRubyObject recv) {
-        ArgsFileData data = ArgsFileData.getDataFrom(recv);
+        ArgsFileData data = ArgsFileData.getArgsFileData(context.runtime);
 
-        if(!data.inited) {
-            return context.runtime.getTrue();
+        if (!data.inited) return context.runtime.getTrue();
+
+        if (!(data.currentFile instanceof RubyIO)) {
+            return data.currentFile.callMethod(context, "eof");
+        } else {
+            return ((RubyIO) data.currentFile).eof_p(context);
         }
-        if(!(data.currentFile instanceof RubyIO)) return data.currentFile.callMethod(context, "eof");
-
-        return ((RubyIO) data.currentFile).eof_p(context);
     }
 
     @JRubyMethod(name = {"eof?"})
     public static IRubyObject eof_p(ThreadContext context, IRubyObject recv) {
-        ArgsFileData data = ArgsFileData.getDataFrom(recv);
+        ArgsFileData data = ArgsFileData.getArgsFileData(context.runtime);
 
-        if (!data.inited) {
-            return context.runtime.getTrue();
+        if (!data.inited) return context.runtime.getTrue();
+
+        if (!(data.currentFile instanceof RubyIO)) {
+            return data.currentFile.callMethod(context, "eof?");
+        } else {
+            return ((RubyIO) data.currentFile).eof_p(context);
         }
-        if (!(data.currentFile instanceof RubyIO)) return data.currentFile.callMethod(context, "eof?");
-
-        return ((RubyIO) data.currentFile).eof_p(context);
     }
 
     @JRubyMethod(name = "pos=", required = 1)
     public static IRubyObject set_pos(ThreadContext context, IRubyObject recv, IRubyObject offset) {
-        ArgsFileData data = getData(context, recv, "no stream to set position");
-
-        return ((RubyIO)data.currentFile).pos_set(context, offset);
+        return getCurrentDataFile(context, "no stream to set position").pos_set(context, offset);
     }
 
     @JRubyMethod(name = "seek", required = 1, optional = 1)
     public static IRubyObject seek(ThreadContext context, IRubyObject recv, IRubyObject[] args) {
-        ArgsFileData data = getData(context, recv, "no stream to seek");
-
-        return ((RubyIO)data.currentFile).seek(context, args);
+        return getCurrentDataFile(context, "no stream to seek").seek(context, args);
     }
 
     @JRubyMethod(name = "readchar")
     public static IRubyObject readchar(ThreadContext context, IRubyObject recv) {
         IRubyObject c = getc(context, recv);
-        
-        if (c.isNil()) {
-            throw context.runtime.newEOFError();
-        }
+
+        if (c.isNil()) throw context.runtime.newEOFError();
 
         return c;
     }
-    
-    @JRubyMethod(compat = RUBY1_9)
+
+    @JRubyMethod
     public static IRubyObject getbyte(ThreadContext context, IRubyObject recv) {
-        ArgsFileData data = ArgsFileData.getDataFrom(recv);
+        ArgsFileData data = ArgsFileData.getArgsFileData(context.runtime);
 
         while(true) {
             if (!data.next_argv(context)) return context.runtime.getNil();
-            
+
             IRubyObject bt;
             if (!(data.currentFile instanceof RubyFile)) {
                 bt = data.currentFile.callMethod(context, "getbyte");
             } else {
-                bt = ((RubyIO)data.currentFile).getbyte19(context);
+                bt = ((RubyIO)data.currentFile).getbyte(context);
             }
 
             if (!bt.isNil()) return bt;
 
+            argf_close(context, data.currentFile);
             data.next_p = 1;
         }
     }
-    
-    @JRubyMethod(compat = RUBY1_9, required = 1, optional = 1)
+
+    @JRubyMethod(required = 1, optional = 2)
     public static IRubyObject read_nonblock(ThreadContext context, IRubyObject recv, IRubyObject[] args) {
         return getPartial(context, recv, args, true);
     }
-    
-    @JRubyMethod(compat = RUBY1_9, required = 1, optional = 1)
+
+    @JRubyMethod(required = 1, optional = 1)
     public static IRubyObject readpartial(ThreadContext context, IRubyObject recv, IRubyObject[] args) {
         return getPartial(context, recv, args, false);
     }
-    
+
     private static IRubyObject getPartial(ThreadContext context, IRubyObject recv, IRubyObject[] args, boolean nonBlocking) {
-        ArgsFileData data = ArgsFileData.getDataFrom(recv);
-        
-        if (args.length > 1) args[1] = args[1].convertToString();
-        
-        if (!data.next_argv(context)) throw context.runtime.newEOFError();
-        
-        IRubyObject tmp;
-        if (!(data.currentFile instanceof RubyFile)) { // In MRI this is stdin && !FILE
-            tmp = data.currentFile.callMethod(context, "getpartial", args);
-        } else {
-            // In MRI io_getpartial.  Not split
-            if (nonBlocking) {
-                tmp = ((RubyIO) data.currentFile).read_nonblock(context, args); 
-            } else {
-                tmp = ((RubyIO) data.currentFile).readpartial(context, args);
+        final Ruby runtime = context.runtime;
+        boolean noException = false;
+        RubyString str = null;
+        if ( args.length > 1 ) {
+            IRubyObject opts = TypeConverter.checkHashType(runtime, args[args.length - 1]);
+            if ( ! opts.isNil() &&
+                runtime.getFalse() == ((RubyHash) opts).op_aref(context, runtime.newSymbol("exception")) ) {
+                noException = true;
+            }
+            if (args.length > 2 || opts.isNil()) {
+                if (!args[1].isNil()) {
+                    args[1] = args[1].convertToString();
+                    str = (RubyString) args[1];
+                }
             }
         }
-        
-        if (tmp.isNil()) {
-            if (data.next_p == -1) throw context.runtime.newEOFError();
-                
+
+        final ArgsFileData data = ArgsFileData.getArgsFileData(runtime);
+
+        if (!data.next_argv(context)) {
+            if (str != null) str.clear();
+            
+            return RubyIO.nonblockEOF(runtime, noException);
+        }
+
+        IRubyObject res = ((RubyIO) data.currentFile).getPartial(context, args, nonBlocking, noException);
+        if (res.isNil()) {
+            if (data.next_p == -1) return RubyIO.nonblockEOF(runtime, noException);
+
             argf_close(context, data.currentFile);
             data.next_p = 1;
-            // FIXME: Missing EOF error is no more argv element at this point
 
-            if (args.length > 1 && args[1].isNil()) return context.runtime.newString();
+            if (data.argv.isEmpty()) return RubyIO.nonblockEOF(runtime, noException);
 
-            return args[1];
+            if (args.length > 1 && args[1] instanceof RubyString )return args[1];
+            return RubyString.newEmptyString(runtime);
         }
-        
-        return tmp;
+
+        return res;
     }
-    
-    @JRubyMethod(compat = RUBY1_9)
+
+    @JRubyMethod
     public static IRubyObject readbyte(ThreadContext context, IRubyObject recv) {
         IRubyObject c = getbyte(context, recv);
-        
+
         if (c.isNil()) throw context.runtime.newEOFError();
-        
+
         return c;
     }
 
     @JRubyMethod(name = "getc")
     public static IRubyObject getc(ThreadContext context, IRubyObject recv) {
-        ArgsFileData data = ArgsFileData.getDataFrom(recv);
+        ArgsFileData data = ArgsFileData.getArgsFileData(context.runtime);
 
         while(true) {
             if (!data.next_argv(context)) return context.runtime.getNil();
@@ -699,7 +728,8 @@ public class RubyArgsFile {
             }
 
             if (!bt.isNil()) return bt;
-            
+
+            argf_close(context, data.currentFile);
             data.next_p = 1;
         }
     }
@@ -707,7 +737,7 @@ public class RubyArgsFile {
     @JRubyMethod(name = "read", optional = 2)
     public static IRubyObject read(ThreadContext context, IRubyObject recv, IRubyObject[] args) {
         Ruby runtime = context.runtime;
-        ArgsFileData data = ArgsFileData.getDataFrom(recv);
+        ArgsFileData data = ArgsFileData.getArgsFileData(context.runtime);
         IRubyObject tmp, str, length;
         long len = 0;
 
@@ -762,23 +792,21 @@ public class RubyArgsFile {
 
     @JRubyMethod(name = "filename", alias = {"path"})
     public static IRubyObject filename(ThreadContext context, IRubyObject recv) {
-        ArgsFileData data = ArgsFileData.getDataFrom(recv);
-        data.next_argv(context);
+        ArgsFileData.getArgsFileData(context.runtime).next_argv(context);
+
         return context.runtime.getGlobalVariables().get("$FILENAME");
     }
 
-    @JRubyMethod(name = "to_s") 
+    @JRubyMethod(name = "to_s", alias = "inspect")
     public static IRubyObject to_s(IRubyObject recv) {
         return recv.getRuntime().newString("ARGF");
     }
-    
-    private static ArgsFileData getData(ThreadContext context, IRubyObject recv, String errorMessage) {
-        ArgsFileData data = ArgsFileData.getDataFrom(recv);
-        
-        if (!data.next_argv(context)) {
-            throw context.runtime.newArgumentError(errorMessage);
-        }
 
-        return data;
+    private static RubyIO getCurrentDataFile(ThreadContext context, String errorMessage) {
+        ArgsFileData data = ArgsFileData.getArgsFileData(context.runtime);
+
+        if (!data.next_argv(context)) throw context.runtime.newArgumentError(errorMessage);
+
+        return (RubyIO) data.currentFile;
     }
-}
+}
\ No newline at end of file
