@@ -21,7 +21,7 @@
  * Copyright (C) 2004 Stefan Matthias Aust <sma@3plus4.de>
  * Copyright (C) 2005 David Corbin <dcorbin@users.sf.net>
  * Copyright (C) 2006 Michael Studman <codehaus@michaelstudman.com>
- * 
+ *
  * Alternatively, the contents of this file may be used under the terms of
  * either of the GNU General Public License Version 2 or later (the "GPL"),
  * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
@@ -62,30 +62,38 @@ import static org.jruby.runtime.Visibility.PRIVATE;
  */
 @JRubyClass(name="Exception")
 public class RubyException extends RubyObject {
+
     protected RubyException(Ruby runtime, RubyClass rubyClass) {
-        this(runtime, rubyClass, null);
+        super(runtime, rubyClass);
+        this.cause = RubyBasicObject.UNDEF;
     }
 
     public RubyException(Ruby runtime, RubyClass rubyClass, String message) {
         super(runtime, rubyClass);
-        
-        this.message = message == null ? runtime.getNil() : runtime.newString(message);
+
+        this.setMessage(message == null ? runtime.getNil() : runtime.newString(message));
+        this.cause = RubyBasicObject.UNDEF;
     }
 
     @JRubyMethod(optional = 2, visibility = PRIVATE)
     public IRubyObject initialize(IRubyObject[] args, Block block) {
-        if (args.length == 1) message = args[0];
+        if ( args.length == 1 ) setMessage(args[0]);
+        // cause filled in at RubyKernel#raise ... Exception.new does not fill-in cause!
         return this;
     }
 
     @JRubyMethod
     public IRubyObject backtrace() {
-        IRubyObject bt = getBacktrace();
-        return bt;
+        return getBacktrace();
     }
 
     @JRubyMethod(required = 1)
     public IRubyObject set_backtrace(IRubyObject obj) {
+        setBacktrace(obj);
+        return backtrace();
+    }
+
+    private void setBacktrace(IRubyObject obj) {
         if (obj.isNil()) {
             backtrace = null;
         } else if (isArrayOfStrings(obj)) {
@@ -93,17 +101,15 @@ public class RubyException extends RubyObject {
         } else if (obj instanceof RubyString) {
             backtrace = RubyArray.newArray(getRuntime(), obj);
         } else {
-            throw getRuntime().newTypeError("backtrace must be Array of String or a single String");
+            throw getRuntime().newTypeError("backtrace must be Array of String");
         }
-
-        return backtrace();
     }
-    
-    @JRubyMethod(compat = CompatVersion.RUBY2_0, omit = true)
+
+    @JRubyMethod(omit = true)
     public IRubyObject backtrace_locations(ThreadContext context) {
         Ruby runtime = context.runtime;
         RubyStackTraceElement[] elements = backtraceData.getBacktrace(runtime);
-        
+
         return RubyThread.Location.newLocationArray(runtime, elements);
     }
 
@@ -118,10 +124,8 @@ public class RubyException extends RubyObject {
             case 0 :
                 return this;
             case 1 :
-                if(args[0] == this) {
-                    return this;
-                }
-                RubyException ret = (RubyException)rbClone();
+                if (args[0] == this) return this;
+                RubyException ret = (RubyException) rbClone();
                 ret.initialize(args, Block.NULL_BLOCK); // This looks wrong, but it's the way MRI does it.
                 return ret;
             default :
@@ -129,28 +133,15 @@ public class RubyException extends RubyObject {
         }
     }
 
-    @JRubyMethod(name = "to_s", compat = CompatVersion.RUBY1_8)
+    @JRubyMethod(name = "to_s")
     public IRubyObject to_s(ThreadContext context) {
-        if (message.isNil()) {
-            return context.runtime.newString(getMetaClass().getRealClass().getName());
-        }
-        message.setTaint(isTaint());
-        return message;
+        final IRubyObject msg = getMessage();
+        if ( ! msg.isNil() ) return msg.asString();
+        return context.runtime.newString(getMetaClass().getRealClass().getName());
     }
 
-    @JRubyMethod(name = "to_s", compat = CompatVersion.RUBY1_9)
-    public IRubyObject to_s19(ThreadContext context) {
-        if (message.isNil()) {
-            return context.runtime.newString(getMetaClass().getRealClass().getName());
-        }
-        message.setTaint(isTaint());
-        return message.asString();
-    }
-
-    @JRubyMethod(name = "to_str", compat = CompatVersion.RUBY1_8)
-    public IRubyObject to_str(ThreadContext context) {
-        return callMethod(context, "to_s");
-    }
+    @Deprecated
+    public IRubyObject to_s19(ThreadContext context) { return to_s(context); }
 
     @JRubyMethod(name = "message")
     public IRubyObject message(ThreadContext context) {
@@ -162,28 +153,28 @@ public class RubyException extends RubyObject {
      *@return A RubyString containing the debug information.
      */
     @JRubyMethod(name = "inspect")
-    public IRubyObject inspect(ThreadContext context) {
+    public RubyString inspect(ThreadContext context) {
         // rb_class_name skips intermediate classes (JRUBY-6786)
-        RubyModule rubyClass = getMetaClass().getRealClass();
+        String rubyClass = getMetaClass().getRealClass().getName();
         RubyString exception = RubyString.objAsString(context, this);
 
-        if (exception.size() == 0) return getRuntime().newString(rubyClass.getName());
-        StringBuilder sb = new StringBuilder("#<");
-        sb.append(rubyClass.getName()).append(": ").append(exception.getByteList()).append('>');
-        return getRuntime().newString(sb.toString());
+        if (exception.isEmpty()) return context.runtime.newString(rubyClass);
+
+        return RubyString.newString(context.runtime,
+                new StringBuilder(2 + rubyClass.length() + 2 + exception.size() + 1).
+                    append("#<").append(rubyClass).append(": ").append(exception.getByteList()).append('>')
+        );
     }
 
-    @JRubyMethod(name = "==", compat = CompatVersion.RUBY1_9)
     @Override
-    public IRubyObject op_equal(ThreadContext context, IRubyObject other) {
+    @JRubyMethod(name = "==")
+    public RubyBoolean op_equal(ThreadContext context, IRubyObject other) {
         if (this == other) return context.runtime.getTrue();
 
         boolean equal = context.runtime.getException().isInstance(other) &&
-                        callMethod(context, "message").equals(other.callMethod(context, "message")) &&
-                        callMethod(context, "backtrace").equals(other.callMethod(context, "backtrace"));
-        if (context.runtime.is2_0()) {
-            equal = equal && getMetaClass().getRealClass() == other.getMetaClass().getRealClass();
-        }
+                getMetaClass().getRealClass() == other.getMetaClass().getRealClass() &&
+                callMethod(context, "message").equals(other.callMethod(context, "message")) &&
+                callMethod(context, "backtrace").equals(other.callMethod(context, "backtrace"));
         return context.runtime.newBoolean(equal);
     }
 
@@ -205,6 +196,24 @@ public class RubyException extends RubyObject {
         return ((RubyClass)recv).op_eqq(context, other);
     }
 
+    @JRubyMethod(name = "cause")
+    public IRubyObject cause(ThreadContext context) {
+        assert cause != null;
+
+        if (cause == RubyBasicObject.UNDEF) return context.nil;
+
+        return cause;
+    }
+
+    public void setCause(IRubyObject cause) {
+        this.cause = cause;
+    }
+
+    // NOTE: can not have IRubyObject as NativeException has getCause() returning Throwable
+    public Object getCause() {
+        return cause;
+    }
+
     public void setBacktraceData(BacktraceData backtraceData) {
         this.backtraceData = backtraceData;
     }
@@ -242,17 +251,17 @@ public class RubyException extends RubyObject {
     }
 
     public void forceBacktrace(IRubyObject backtrace) {
-        backtraceData = BacktraceData.EMPTY;
-        set_backtrace(backtrace);
+        backtraceData = (backtrace != null && backtrace.isNil()) ? null : BacktraceData.EMPTY;
+        setBacktrace(backtrace);
     }
-    
+
     public IRubyObject getBacktrace() {
         if (backtrace == null) {
             initBacktrace();
         }
         return backtrace;
     }
-    
+
     public void initBacktrace() {
         Ruby runtime = getRuntime();
         if (backtraceData == null) {
@@ -263,6 +272,7 @@ public class RubyException extends RubyObject {
     }
 
     @Override
+    @SuppressWarnings("deprecated")
     public void copySpecialInstanceVariables(IRubyObject clone) {
         RubyException exception = (RubyException)clone;
         exception.backtraceData = backtraceData;
@@ -287,36 +297,36 @@ public class RubyException extends RubyObject {
      * @param errorStream the PrintStream to which backtrace should be printed
      */
     public void printBacktrace(PrintStream errorStream, int skip) {
-        IRubyObject backtrace = callMethod(getRuntime().getCurrentContext(), "backtrace");
-        if (!backtrace.isNil() && backtrace instanceof RubyArray) {
-            IRubyObject[] elements = backtrace.convertToArray().toJavaArray();
-
+        IRubyObject trace = callMethod(getRuntime().getCurrentContext(), "backtrace");
+        if ( trace.isNil() ) return;
+        if ( trace instanceof RubyArray ) {
+            IRubyObject[] elements = ((RubyArray) trace).toJavaArrayMaybeUnsafe();
             for (int i = skip; i < elements.length; i++) {
                 IRubyObject stackTraceLine = elements[i];
                 if (stackTraceLine instanceof RubyString) {
-                    printStackTraceLine(errorStream, stackTraceLine);
+                    errorStream.println("\tfrom " + stackTraceLine);
+                }
+                else {
+                    errorStream.println("\t" + stackTraceLine);
                 }
             }
         }
     }
 
-    private void printStackTraceLine(PrintStream errorStream, IRubyObject stackTraceLine) {
-            errorStream.print("\tfrom " + stackTraceLine + '\n');
-    }
-	
     private boolean isArrayOfStrings(IRubyObject backtrace) {
-        if (!(backtrace instanceof RubyArray)) return false; 
-            
-        IRubyObject[] elements = ((RubyArray) backtrace).toJavaArray();
-        
-        for (int i = 0 ; i < elements.length ; i++) {
-            if (!(elements[i] instanceof RubyString)) return false;
+        if (!(backtrace instanceof RubyArray)) return false;
+
+        final RubyArray rTrace = ((RubyArray) backtrace);
+
+        for (int i = 0 ; i < rTrace.getLength() ; i++) {
+            if (!(rTrace.eltInternal(i) instanceof RubyString)) return false;
         }
-            
+
         return true;
     }
 
     public static ObjectAllocator EXCEPTION_ALLOCATOR = new ObjectAllocator() {
+        @Override
         public IRubyObject allocate(Ruby runtime, RubyClass klass) {
             RubyException instance = new RubyException(runtime, klass);
 
@@ -328,18 +338,19 @@ public class RubyException extends RubyObject {
     };
 
     private static final ObjectMarshal EXCEPTION_MARSHAL = new ObjectMarshal() {
+        @Override
         public void marshalTo(Ruby runtime, Object obj, RubyClass type,
                               MarshalStream marshalStream) throws IOException {
             RubyException exc = (RubyException)obj;
 
             marshalStream.registerLinkTarget(exc);
             List<Variable<Object>> attrs = exc.getVariableList();
-            attrs.add(new VariableEntry<Object>(
-                    "mesg", exc.message == null ? runtime.getNil() : exc.message));
+            attrs.add(new VariableEntry<Object>("mesg", exc.getMessage()));
             attrs.add(new VariableEntry<Object>("bt", exc.getBacktrace()));
             marshalStream.dumpVariables(attrs);
         }
 
+        @Override
         public Object unmarshalFrom(Ruby runtime, RubyClass type,
                                     UnmarshalStream unmarshalStream) throws IOException {
             RubyException exc = (RubyException)type.allocate();
@@ -349,7 +360,7 @@ public class RubyException extends RubyObject {
             // just use real vars all the time for these?
             unmarshalStream.defaultVariablesUnmarshal(exc);
 
-            exc.message = (IRubyObject)exc.removeInternalVariable("mesg");
+            exc.setMessage((IRubyObject)exc.removeInternalVariable("mesg"));
             exc.set_backtrace((IRubyObject)exc.removeInternalVariable("bt"));
 
             return exc;
@@ -360,7 +371,7 @@ public class RubyException extends RubyObject {
         RubyClass exceptionClass = runtime.defineClass("Exception", runtime.getObject(), EXCEPTION_ALLOCATOR);
         runtime.setException(exceptionClass);
 
-        exceptionClass.index = ClassIndex.EXCEPTION;
+        exceptionClass.setClassIndex(ClassIndex.EXCEPTION);
         exceptionClass.setReifiedClass(RubyException.class);
 
         exceptionClass.setMarshal(EXCEPTION_MARSHAL);
@@ -378,9 +389,32 @@ public class RubyException extends RubyObject {
         return exceptionClass.callMethod(context, "new", message.convertToString());
     }
 
+    /**
+     * @return error message if provided or nil
+     */
+    @SuppressWarnings("deprecated")
+    public IRubyObject getMessage() {
+        return message == null ? getRuntime().getNil() : message;
+    }
+
+    /**
+     * Set the message for this NameError.
+     * @param message the message
+     */
+    @SuppressWarnings("deprecated")
+    public void setMessage(IRubyObject message) {
+        this.message = message;
+    }
+
+    public String getMessageAsJavaString() {
+        final IRubyObject msg = getMessage();
+        return msg.isNil() ? null : msg.toString();
+    }
+
     private BacktraceData backtraceData;
     private IRubyObject backtrace;
-    public IRubyObject message;
+    IRubyObject message;
+    IRubyObject cause;
 
     public static final int TRACE_HEAD = 8;
     public static final int TRACE_TAIL = 4;
