@@ -35,7 +35,10 @@ package org.jruby;
 
 import java.io.FileDescriptor;
 import java.io.IOException;
+import java.nio.file.attribute.FileTime;
+import java.util.concurrent.TimeUnit;
 
+import jnr.posix.NanosecondFileStat;
 import org.jruby.anno.JRubyClass;
 import org.jruby.anno.JRubyMethod;
 import jnr.posix.FileStat;
@@ -48,7 +51,6 @@ import org.jruby.runtime.Visibility;
 import org.jruby.runtime.builtin.IRubyObject;
 import org.jruby.util.FileResource;
 import org.jruby.util.JRubyFile;
-import org.jruby.util.JRubyNonExistentFile;
 import org.jruby.util.StringSupport;
 
 /**
@@ -61,6 +63,7 @@ public class RubyFileStat extends RubyObject {
     private static final int S_IRUGO = (FileStat.S_IRUSR | FileStat.S_IRGRP | FileStat.S_IROTH);
     private static final int S_IWUGO = (FileStat.S_IWUSR | FileStat.S_IWGRP | FileStat.S_IWOTH);
     private static final int S_IXUGO = (FileStat.S_IXUSR | FileStat.S_IXGRP | FileStat.S_IXOTH);
+    public static final int BILLION = 1000000000;
 
     private FileResource file;
     private FileStat stat;
@@ -70,6 +73,7 @@ public class RubyFileStat extends RubyObject {
     }
 
     private static ObjectAllocator ALLOCATOR = new ObjectAllocator() {
+        @Override
         public IRubyObject allocate(Ruby runtime, RubyClass klass) {
             return new RubyFileStat(runtime, klass);
         }
@@ -107,46 +111,67 @@ public class RubyFileStat extends RubyObject {
         return stat;
     }
 
+    public static RubyFileStat newFileStat(Ruby runtime, int fileno) {
+        RubyFileStat stat = new RubyFileStat(runtime, runtime.getFileStat());
+
+        stat.setup(fileno);
+
+        return stat;
+    }
+
     private void setup(FileDescriptor descriptor) {
         stat = getRuntime().getPosix().fstat(descriptor);
     }
+
+    private void setup(int fileno) {
+        stat = getRuntime().getPosix().fstat(fileno);
+    }
     
     private void setup(String filename, boolean lstat) {
         Ruby runtime = getRuntime();
 
         if (Platform.IS_WINDOWS && filename.length() == 2
                 && filename.charAt(1) == ':' && Character.isLetter(filename.charAt(0))) {
-            filename += "/";
+            filename += '/';
         }
 
-        file = JRubyFile.createResource(runtime.getPosix(), runtime.getCurrentDirectory(), filename);
+        file = JRubyFile.createResource(runtime, filename);
         stat = lstat ? file.lstat() : file.stat();
 
         if (stat == null) throw runtime.newErrnoFromInt(file.errno(), filename);
     }
 
-    @JRubyMethod(name = "initialize", required = 1, visibility = Visibility.PRIVATE, compat = CompatVersion.RUBY1_8)
     public IRubyObject initialize(IRubyObject fname, Block unusedBlock) {
+        return initialize19(fname, unusedBlock);
+    }
+
+    @JRubyMethod(name = "initialize", required = 1, visibility = Visibility.PRIVATE)
+    public IRubyObject initialize19(IRubyObject fname, Block unusedBlock) {
         Ruby runtime = getRuntime();
         ThreadContext context = runtime.getCurrentContext();
         RubyString path = StringSupport.checkEmbeddedNulls(runtime, RubyFile.get_path(context, fname));
-        setup(fname.convertToString().toString(), false);
+        setup(path.convertToString().toString(), false);
 
-        return this;
-    }
-
-    @JRubyMethod(name = "initialize", required = 1, visibility = Visibility.PRIVATE, compat = CompatVersion.RUBY1_9)
-    public IRubyObject initialize19(IRubyObject fname, Block unusedBlock) {
-        return initialize(RubyFile.get_path(getRuntime().getCurrentContext(), fname), unusedBlock);
+        return this;    
     }
     
     @JRubyMethod(name = "atime")
     public IRubyObject atime() {
         checkInitialized();
+        if (stat instanceof NanosecondFileStat) {
+            return RubyTime.newTimeFromNanoseconds(getRuntime(), stat.atime() * BILLION + ((NanosecondFileStat) stat).aTimeNanoSecs());
+        }
         return getRuntime().newTime(stat.atime() * 1000);
     }
     
     @JRubyMethod(name = "blksize")
+    public IRubyObject blockSize(ThreadContext context) {
+        checkInitialized();
+        if (Platform.IS_WINDOWS) return context.nil;
+        return context.runtime.newFixnum(stat.blockSize());
+    }
+
+    @Deprecated
     public RubyFixnum blksize() {
         checkInitialized();
         return getRuntime().newFixnum(stat.blockSize());
@@ -161,6 +186,7 @@ public class RubyFileStat extends RubyObject {
     @JRubyMethod(name = "blocks")
     public IRubyObject blocks() {
         checkInitialized();
+        if (Platform.IS_WINDOWS) return getRuntime().getNil();
         return getRuntime().newFixnum(stat.blocks());
     }
 
@@ -190,9 +216,20 @@ public class RubyFileStat extends RubyObject {
     @JRubyMethod(name = "ctime")
     public IRubyObject ctime() {
         checkInitialized();
+        if (stat instanceof NanosecondFileStat) {
+            return RubyTime.newTimeFromNanoseconds(getRuntime(), stat.ctime() * BILLION + ((NanosecondFileStat) stat).cTimeNanoSecs());
+        }
         return getRuntime().newTime(stat.ctime() * 1000);
     }
 
+    @JRubyMethod(name = "birthtime")
+    public IRubyObject birthtime() {
+        checkInitialized();
+        FileTime btime = RubyFile.getBirthtimeWithNIO(file.absolutePath());
+        if (btime != null) return getRuntime().newTime(btime.toMillis());
+        return ctime();
+    }
+
     @JRubyMethod(name = "dev")
     public IRubyObject dev() {
         checkInitialized();
@@ -202,12 +239,14 @@ public class RubyFileStat extends RubyObject {
     @JRubyMethod(name = "dev_major")
     public IRubyObject devMajor() {
         checkInitialized();
+        if (Platform.IS_WINDOWS) return getRuntime().getNil();
         return getRuntime().newFixnum(stat.major(stat.dev()));
     }
 
     @JRubyMethod(name = "dev_minor")
     public IRubyObject devMinor() {
         checkInitialized();
+        if (Platform.IS_WINDOWS) return getRuntime().getNil();
         return getRuntime().newFixnum(stat.minor(stat.dev()));
     }
 
@@ -250,14 +289,18 @@ public class RubyFileStat extends RubyObject {
     @JRubyMethod(name = "grpowned?")
     public IRubyObject group_owned_p() {
         checkInitialized();
+        if (Platform.IS_WINDOWS) return getRuntime().getFalse();
         return getRuntime().newBoolean(stat.isGroupOwned());
     }
     
     @JRubyMethod(name = "initialize_copy", required = 1, visibility = Visibility.PRIVATE)
+    @Override
     public IRubyObject initialize_copy(IRubyObject original) {
         if (!(original instanceof RubyFileStat)) {
             throw getRuntime().newTypeError("wrong argument class");
         }
+
+        checkFrozen();
         
         RubyFileStat originalFileStat = (RubyFileStat) original;
         
@@ -274,13 +317,15 @@ public class RubyFileStat extends RubyObject {
     }
 
     @JRubyMethod(name = "inspect")
+    @Override
     public IRubyObject inspect() {
         StringBuilder buf = new StringBuilder("#<");
         buf.append(getMetaClass().getRealClass().getName());
         if (stat == null) {
             buf.append(": uninitialized");
         } else {
-            buf.append(" ");
+            ThreadContext context = getRuntime().getCurrentContext();
+            buf.append(' ');
             // FIXME: Obvious issue that not all platforms can display all attributes.  Ugly hacks.
             // Using generic posix library makes pushing inspect behavior into specific system impls
             // rather painful.
@@ -292,14 +337,15 @@ public class RubyFileStat extends RubyObject {
             try { buf.append("gid=").append(stat.gid()); } catch (Exception e) {} finally { buf.append(", "); }
             try { buf.append("rdev=0x").append(Long.toHexString(stat.rdev())); } catch (Exception e) {} finally { buf.append(", "); }
             buf.append("size=").append(sizeInternal()).append(", ");
-            try { buf.append("blksize=").append(stat.blockSize()); } catch (Exception e) {} finally { buf.append(", "); }
-            try { buf.append("blocks=").append(stat.blocks()); } catch (Exception e) {} finally { buf.append(", "); }
+            try {
+                buf.append("blksize=").append(blockSize(context).inspect().toString()); } catch (Exception e) {} finally { buf.append(", "); }
+            try { buf.append("blocks=").append(blocks().inspect().toString()); } catch (Exception e) {} finally { buf.append(", "); }
 
             buf.append("atime=").append(atime()).append(", ");
             buf.append("mtime=").append(mtime()).append(", ");
             buf.append("ctime=").append(ctime());
         }
-        buf.append(">");
+        buf.append('>');
         
         return getRuntime().newString(buf.toString());
     }
@@ -319,19 +365,49 @@ public class RubyFileStat extends RubyObject {
     @JRubyMethod(name = "mtime")
     public IRubyObject mtime() {
         checkInitialized();
+        if (stat instanceof NanosecondFileStat) {
+            return RubyTime.newTimeFromNanoseconds(getRuntime(), stat.mtime() * BILLION + ((NanosecondFileStat) stat).mTimeNanoSecs());
+        }
         return getRuntime().newTime(stat.mtime() * 1000);
     }
     
     public IRubyObject mtimeEquals(IRubyObject other) {
-        return getRuntime().newBoolean(stat.mtime() == newFileStat(getRuntime(), other.convertToString().toString(), false).stat.mtime()); 
+        FileStat otherStat = newFileStat(getRuntime(), other.convertToString().toString(), false).stat;
+        boolean equal = stat.mtime() == otherStat.mtime();
+
+        if (stat instanceof NanosecondFileStat && otherStat instanceof NanosecondFileStat) {
+            equal = equal && ((NanosecondFileStat) stat).mTimeNanoSecs() == ((NanosecondFileStat) otherStat).mTimeNanoSecs();
+        }
+
+        return getRuntime().newBoolean(equal);
     }
 
     public IRubyObject mtimeGreaterThan(IRubyObject other) {
-        return getRuntime().newBoolean(stat.mtime() > newFileStat(getRuntime(), other.convertToString().toString(), false).stat.mtime()); 
+        FileStat otherStat = newFileStat(getRuntime(), other.convertToString().toString(), false).stat;
+        boolean gt;
+
+        if (stat instanceof NanosecondFileStat && otherStat instanceof NanosecondFileStat) {
+            gt = (stat.mtime() * BILLION + ((NanosecondFileStat) stat).mTimeNanoSecs())
+                    > (otherStat.mtime() * BILLION + ((NanosecondFileStat) otherStat).mTimeNanoSecs());
+        } else {
+            gt = stat.mtime() > otherStat.mtime();
+        }
+
+        return getRuntime().newBoolean(gt);
     }
 
     public IRubyObject mtimeLessThan(IRubyObject other) {
-        return getRuntime().newBoolean(stat.mtime() < newFileStat(getRuntime(), other.convertToString().toString(), false).stat.mtime()); 
+        FileStat otherStat = newFileStat(getRuntime(), other.convertToString().toString(), false).stat;
+        boolean lt;
+
+        if (stat instanceof NanosecondFileStat && otherStat instanceof NanosecondFileStat) {
+            lt = (stat.mtime() * BILLION + ((NanosecondFileStat) stat).mTimeNanoSecs())
+                    < (otherStat.mtime() * BILLION + ((NanosecondFileStat) otherStat).mTimeNanoSecs());
+        } else {
+            lt = stat.mtime() < otherStat.mtime();
+        }
+
+        return getRuntime().newBoolean(lt);
     }
 
     @JRubyMethod(name = "nlink")
@@ -361,12 +437,14 @@ public class RubyFileStat extends RubyObject {
     @JRubyMethod(name = "rdev_major")
     public IRubyObject rdevMajor() {
         checkInitialized();
+        if (Platform.IS_WINDOWS) return getRuntime().getNil();
         return getRuntime().newFixnum(stat.major(stat.rdev()));
     }
 
     @JRubyMethod(name = "rdev_minor")
     public IRubyObject rdevMinor() {
         checkInitialized();
+        if (Platform.IS_WINDOWS) return getRuntime().getNil();
         return getRuntime().newFixnum(stat.minor(stat.rdev()));
     }
 
@@ -464,12 +542,12 @@ public class RubyFileStat extends RubyObject {
         return getRuntime().newBoolean(stat.isEmpty());
     }
 
-    @JRubyMethod(name = "world_readable?", compat = CompatVersion.RUBY1_9)
+    @JRubyMethod(name = "world_readable?")
     public IRubyObject worldReadable(ThreadContext context) {
         return getWorldMode(context, FileStat.S_IROTH);
     }
 
-    @JRubyMethod(name = "world_writable?", compat = CompatVersion.RUBY1_9)
+    @JRubyMethod(name = "world_writable?")
     public IRubyObject worldWritable(ThreadContext context) {
         return getWorldMode(context, FileStat.S_IWOTH);
     }
