@@ -16,7 +16,7 @@ public class KeywordArgNode extends Node {
     private AssignableNode assignable;
     
     public KeywordArgNode(ISourcePosition position, AssignableNode assignable) {
-        super(position);
+        super(position, true);
         this.assignable = assignable;
     }
 
@@ -30,6 +30,10 @@ public class KeywordArgNode extends Node {
         return Node.createList(assignable);
     }
 
+    public int getIndex() {
+        return ((IScopedNode) assignable).getIndex();
+    }
+
     @Override
     public NodeType getNodeType() {
         return NodeType.KEYWORDARGNODE;
