@@ -29,6 +29,6 @@ public class AnnotationParameter {
     
     @Override
     public String toString() {
-        return "" + name + "=" + expression;
+        return name + '=' + expression;
     }
 }
