@@ -1,20 +1,34 @@
-/*
- * SkinnyMethodAdapter.java
+/***** BEGIN LICENSE BLOCK *****
+ * Version: EPL 1.0/GPL 2.0/LGPL 2.1
  *
- * Created on March 10, 2007, 2:52 AM
+ * The contents of this file are subject to the Eclipse Public
+ * License Version 1.0 (the "License"); you may not use this file
+ * except in compliance with the License. You may obtain a copy of
+ * the License at http://www.eclipse.org/legal/epl-v10.html
  *
- * To change this template, choose Tools | Template Manager
- * and open the template in the editor.
- */
-
+ * Software distributed under the License is distributed on an "AS
+ * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
+ * implied. See the License for the specific language governing
+ * rights and limitations under the License.
+ *
+ * Alternatively, the contents of this file may be used under the terms of
+ * either of the GNU General Public License Version 2 or later (the "GPL"),
+ * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
+ * in which case the provisions of the GPL or the LGPL are applicable instead
+ * of those above. If you wish to allow use of your version of this file only
+ * under the terms of either the GPL or the LGPL, and not to allow others to
+ * use your version of this file under the terms of the EPL, indicate your
+ * decision by deleting the provisions above and replace them with the notice
+ * and other provisions required by the GPL or the LGPL. If you do not delete
+ * the provisions above, a recipient may use your version of this file under
+ * the terms of any one of the EPL, the GPL or the LGPL.
+ ***** END LICENSE BLOCK *****/
 package org.jruby.compiler.impl;
 
 import java.io.PrintStream;
 import java.io.PrintWriter;
 import java.util.Map;
-import java.util.concurrent.Callable;
 
-import org.jruby.util.SafePropertyAccessor;
 import org.jruby.util.cli.Options;
 import static org.jruby.util.CodegenUtils.*;
 
@@ -26,24 +40,26 @@ import org.objectweb.asm.ClassWriter;
 import org.objectweb.asm.Handle;
 import org.objectweb.asm.Label;
 import org.objectweb.asm.MethodVisitor;
-import org.objectweb.asm.Opcodes;
 import org.objectweb.asm.Type;
 import org.objectweb.asm.util.Printer;
 import org.objectweb.asm.util.Textifier;
 import org.objectweb.asm.util.TraceMethodVisitor;
+import static org.objectweb.asm.Opcodes.*;
 
 /**
  *
  * @author headius
  */
-public class SkinnyMethodAdapter extends MethodVisitor implements Opcodes {
+public final class SkinnyMethodAdapter extends MethodVisitor {
     private final static boolean DEBUG = Options.COMPILE_DUMP.load();
+
+    private final String name;
+    private final ClassVisitor cv;
+    private final Label start;
+    private final Label end;
+
     private MethodVisitor method;
     private Printer printer;
-    private String name;
-    private ClassVisitor cv;
-    private Label start;
-    private Label end;
 
     public SkinnyMethodAdapter(ClassVisitor cv, int flags, String name, String signature, String something, String[] exceptions) {
         super(ASM4);
@@ -53,6 +69,10 @@ public class SkinnyMethodAdapter extends MethodVisitor implements Opcodes {
         this.start = new Label();
         this.end = new Label();
     }
+
+    public ClassVisitor getClassVisitor() {
+        return cv;
+    }
     
     public MethodVisitor getMethodVisitor() {
         return method;
@@ -170,19 +190,27 @@ public class SkinnyMethodAdapter extends MethodVisitor implements Opcodes {
     }
     
     public void invokestatic(String arg1, String arg2, String arg3) {
-        getMethodVisitor().visitMethodInsn(INVOKESTATIC, arg1, arg2, arg3);
+        getMethodVisitor().visitMethodInsn(INVOKESTATIC, arg1, arg2, arg3, false);
+    }
+
+    public void invokestatic(String arg1, String arg2, String arg3, boolean iface) {
+        getMethodVisitor().visitMethodInsn(INVOKESTATIC, arg1, arg2, arg3, iface);
     }
     
     public void invokespecial(String arg1, String arg2, String arg3) {
-        getMethodVisitor().visitMethodInsn(INVOKESPECIAL, arg1, arg2, arg3);
+        getMethodVisitor().visitMethodInsn(INVOKESPECIAL, arg1, arg2, arg3, false);
+    }
+
+    public void invokespecial(String arg1, String arg2, String arg3, boolean iface) {
+        getMethodVisitor().visitMethodInsn(INVOKESPECIAL, arg1, arg2, arg3, iface);
     }
     
     public void invokevirtual(String arg1, String arg2, String arg3) {
-        getMethodVisitor().visitMethodInsn(INVOKEVIRTUAL, arg1, arg2, arg3);
+        getMethodVisitor().visitMethodInsn(INVOKEVIRTUAL, arg1, arg2, arg3, false);
     }
     
     public void invokeinterface(String arg1, String arg2, String arg3) {
-        getMethodVisitor().visitMethodInsn(INVOKEINTERFACE, arg1, arg2, arg3);
+        getMethodVisitor().visitMethodInsn(INVOKEINTERFACE, arg1, arg2, arg3, true);
     }
 
     public void invokedynamic(String arg0, String arg1, Handle arg2, Object... arg3) {
@@ -535,41 +563,49 @@ public class SkinnyMethodAdapter extends MethodVisitor implements Opcodes {
         getMethodVisitor().visitCode();
         getMethodVisitor().visitLabel(start);
     }
-    
+
+    static final Runnable NO_LOCALS = new Runnable() { public void run() { /* no-op */ } };
+
     public void end() {
-        end(new Runnable() {
-            public void run() {
-            }
-        });
+        end(NO_LOCALS);
     }
 
     public void end(Runnable locals) {
-        if (DEBUG) {
-            PrintWriter pw = new PrintWriter(System.out);
-            String className = "(unknown class)";
-            if (cv instanceof ClassWriter) {
-                className = new ClassReader(((ClassWriter)cv).toByteArray()).getClassName();
-            }
-            if (name != null) {
-                pw.write("*** Dumping " + className + "." + name + " ***\n");
-            } else {
-                pw.write("*** Dumping ***\n");
-            }
-            printer.print(pw);
-            pw.flush();
-        }
+        if (DEBUG) printByteCode(getClassName());
         getMethodVisitor().visitLabel(end);
         locals.run();
         getMethodVisitor().visitMaxs(1, 1);
         getMethodVisitor().visitEnd();
     }
 
+    private String getClassName() {
+        if (cv instanceof ClassWriter) {
+            return new ClassReader(((ClassWriter) cv).toByteArray()).getClassName();
+        }
+        return "(unknown class)";
+    }
+
+    private PrintWriter outDebugWriter() {
+        return new PrintWriter(System.out);
+    }
+
+    private void printByteCode(final String className) {
+        PrintWriter pw = outDebugWriter();
+        if (name != null) {
+            pw.write("*** Dumping " + className + '.' + name + " ***\n");
+        } else {
+            pw.write("*** Dumping ***\n");
+        }
+        if (printer != null) printer.print(pw);
+        pw.flush();
+    }
+
     public void local(int index, String name, Class type) {
-        method.visitLocalVariable(name, ci(type), null, start, end, index);
+        getMethodVisitor().visitLocalVariable(name, ci(type), null, start, end, index);
     }
 
     public void local(int index, String name, Type type) {
-        method.visitLocalVariable(name, type.getDescriptor(), null, start, end, index);
+        getMethodVisitor().visitLocalVariable(name, type.getDescriptor(), null, start, end, index);
     }
 
     public void line(int line) {
@@ -771,7 +807,7 @@ public class SkinnyMethodAdapter extends MethodVisitor implements Opcodes {
     }
     
     public void f2i() {
-        getMethodVisitor().visitInsn(F2D);
+        getMethodVisitor().visitInsn(F2I);
     }
     
     public void f2l() {
@@ -899,9 +935,16 @@ public class SkinnyMethodAdapter extends MethodVisitor implements Opcodes {
     }
 
     @Override
+    @Deprecated
     public void visitMethodInsn(int arg0, String arg1, String arg2, String arg3) {
         getMethodVisitor().visitMethodInsn(arg0, arg1, arg2, arg3);
     }
+
+    @Override
+    @Deprecated
+    public void visitMethodInsn(int arg0, String arg1, String arg2, String arg3, boolean arg4) {
+        getMethodVisitor().visitMethodInsn(arg0, arg1, arg2, arg3, arg4);
+    }
     
     @Override
     public void visitInvokeDynamicInsn(String arg0, String arg1, Handle arg2, Object... arg3) {
