@@ -29,6 +29,8 @@ package org.jruby.ext.socket;
 import jnr.constants.platform.AddressFamily;
 import jnr.constants.platform.ProtocolFamily;
 import jnr.constants.platform.Sock;
+import jnr.constants.platform.SocketLevel;
+import jnr.constants.platform.SocketOption;
 import jnr.netdb.Protocol;
 import jnr.netdb.Service;
 import org.jruby.Ruby;
@@ -44,10 +46,13 @@ import org.jruby.exceptions.RaiseException;
 import org.jruby.runtime.ThreadContext;
 import org.jruby.runtime.builtin.IRubyObject;
 import org.jruby.util.ByteList;
+import org.jruby.util.TypeConverter;
 import org.jruby.util.io.Sockaddr;
 
+import java.net.Inet4Address;
 import java.net.Inet6Address;
 import java.net.InetAddress;
+import java.net.InetSocketAddress;
 import java.net.NetworkInterface;
 import java.net.SocketException;
 import java.net.UnknownHostException;
@@ -58,6 +63,7 @@ import java.util.regex.Matcher;
 import java.util.regex.Pattern;
 
 import static jnr.constants.platform.AddressFamily.AF_INET;
+import static jnr.constants.platform.AddressFamily.AF_INET6;
 import static jnr.constants.platform.IPProto.IPPROTO_TCP;
 import static jnr.constants.platform.IPProto.IPPROTO_UDP;
 import static jnr.constants.platform.NameInfo.NI_NUMERICHOST;
@@ -92,14 +98,14 @@ public class SocketUtils {
 
     public static IRubyObject gethostbyaddr(ThreadContext context, IRubyObject[] args) {
         Ruby runtime = context.runtime;
-        IRubyObject[] ret = new IRubyObject[4];
+        IRubyObject ret0, ret1, ret2, ret3;
 
-        ret[0] = runtime.newString(Sockaddr.addressFromString(runtime, args[0].convertToString().toString()).getCanonicalHostName());
-        ret[1] = runtime.newArray();
-        ret[2] = runtime.newFixnum(2); // AF_INET
-        ret[3] = args[0];
+        ret0 = runtime.newString(Sockaddr.addressFromString(runtime, args[0].convertToString().toString()).getCanonicalHostName());
+        ret1 = runtime.newArray();
+        ret2 = runtime.newFixnum(2); // AF_INET
+        ret3 = args[0];
 
-        return runtime.newArrayNoCopy(ret);
+        return RubyArray.newArray(runtime, ret0, ret1, ret2, ret3);
     }
 
     public static IRubyObject getservbyname(ThreadContext context, IRubyObject[] args) {
@@ -128,56 +134,34 @@ public class SocketUtils {
         return runtime.newFixnum(port);
     }
 
+    @Deprecated
     public static IRubyObject pack_sockaddr_in(ThreadContext context, IRubyObject port, IRubyObject host) {
-        int portNum = 0;
-        
-        if(!port.isNil()){
-          portNum = port instanceof RubyString ?
-                Integer.parseInt(port.convertToString().toString()) :
-                RubyNumeric.fix2int(port);
-        }
-
-        return Sockaddr.pack_sockaddr_in(
-                context,
-                portNum,
-                host.isNil() ? null : host.convertToString().toString());
+        return Sockaddr.pack_sockaddr_in(context, port, host);
     }
 
-    public static IRubyObject unpack_sockaddr_in(ThreadContext context, IRubyObject addr) {
+    @Deprecated
+    public static RubyArray unpack_sockaddr_in(ThreadContext context, IRubyObject addr) {
         return Sockaddr.unpack_sockaddr_in(context, addr);
     }
 
+    @Deprecated
     public static IRubyObject pack_sockaddr_un(ThreadContext context, IRubyObject filename) {
-        ByteList str = filename.convertToString().getByteList();
-
-        AddressFamily af = AddressFamily.AF_UNIX;
-        int high = (af.intValue() & 0xff00) >> 8;
-        int low = af.intValue() & 0xff;
-
-        ByteList bl = new ByteList();
-        bl.append((byte)high);
-        bl.append((byte)low);
-        bl.append(str);
-
-        for(int i=str.length();i<104;i++) {
-            bl.append((byte)0);
-        }
-
-        return context.runtime.newString(bl);
+        String path = filename.convertToString().asJavaString();
+        return Sockaddr.pack_sockaddr_un(context, path);
     }
 
     public static IRubyObject gethostbyname(ThreadContext context, IRubyObject hostname) {
         Ruby runtime = context.runtime;
 
         try {
-            InetAddress addr = getRubyInetAddress(hostname.convertToString().getByteList());
-            IRubyObject[] ret = new IRubyObject[4];
+            InetAddress addr = getRubyInetAddresses(hostname.convertToString().getByteList())[0];
+            IRubyObject ret0, ret1, ret2, ret3;
 
-            ret[0] = runtime.newString(addr.getCanonicalHostName());
-            ret[1] = runtime.newArray();
-            ret[2] = runtime.newFixnum(2); // AF_INET
-            ret[3] = runtime.newString(new ByteList(addr.getAddress()));
-            return runtime.newArrayNoCopy(ret);
+            ret0 = runtime.newString(addr.getCanonicalHostName());
+            ret1 = runtime.newArray();
+            ret2 = runtime.newFixnum(2); // AF_INET
+            ret3 = runtime.newString(new ByteList(addr.getAddress()));
+            return RubyArray.newArray(runtime, ret0, ret1, ret2, ret3);
 
         } catch(UnknownHostException e) {
             throw sockerr(runtime, "gethostbyname: name or service not known");
@@ -222,7 +206,7 @@ public class SocketUtils {
                     c[4] = runtime.newFixnum(is_ipv6 ? PF_INET6 : PF_INET);
                     c[5] = runtime.newFixnum(SOCK_DGRAM);
                     c[6] = runtime.newFixnum(IPPROTO_UDP);
-                    l.add(runtime.newArrayNoCopy(c));
+                    l.add(RubyArray.newArrayMayCopy(runtime, c));
                 }
 
                 if (sock_stream) {
@@ -234,7 +218,7 @@ public class SocketUtils {
                     c[4] = runtime.newFixnum(is_ipv6 ? PF_INET6 : PF_INET);
                     c[5] = runtime.newFixnum(SOCK_STREAM);
                     c[6] = runtime.newFixnum(IPPROTO_TCP);
-                    l.add(runtime.newArrayNoCopy(c));
+                    l.add(RubyArray.newArrayMayCopy(runtime, c));
                 }
             }
         });
@@ -264,15 +248,15 @@ public class SocketUtils {
 
                 if (sock_dgram) {
                     l.add(new Addrinfo(runtime, runtime.getClass("Addrinfo"),
-                            address,
-                            port,
+                            new InetSocketAddress(address, port),
+                            Sock.SOCK_DGRAM,
                             SocketType.DATAGRAM));
                 }
 
                 if (sock_stream) {
                     l.add(new Addrinfo(runtime, runtime.getClass("Addrinfo"),
-                            address,
-                            port,
+                            new InetSocketAddress(address, port),
+                            Sock.SOCK_STREAM,
                             SocketType.SOCKET));
                 }
             }
@@ -295,75 +279,47 @@ public class SocketUtils {
         IRubyObject port = args[1];
         boolean emptyHost = host.isNil() || host.convertToString().isEmpty();
 
+        IRubyObject family = args.length > 2 ? args[2] : context.nil;
+        IRubyObject socktype = args.length > 3 ? args[3] : context.nil;
+        IRubyObject protocol = args.length > 4 ? args[4] : context.nil;
+        IRubyObject flags = args.length > 5 ? args[5] : context.nil;
+        IRubyObject reverseArg = args.length > 6 ? args[6] : context.nil;
+
+        // The Ruby Socket.getaddrinfo function supports boolean/nil/Symbol values for the
+        // reverse_lookup parameter. We need to massage all valid inputs to true/false/null.
+        Boolean reverseLookup = RubyIPSocket.doReverseLookup(context, reverseArg);
+
+        AddressFamily addressFamily = family.isNil() ? null : addressFamilyFromArg(family);
+
+        Sock sock = socktype.isNil() ? SOCK_STREAM : sockFromArg(socktype);
+
+        if(port instanceof RubyString) {
+            port = getservbyname(context, new IRubyObject[]{port});
+        }
+
+        int p = port.isNil() ? 0 : (int)port.convertToInteger().getLongValue();
+
+        // TODO: implement flags
+        int flag = flags.isNil() ? 0 : RubyNumeric.fix2int(flags);
+
+        String hostString = null;
+
+        // The value of 1 is for Socket::AI_PASSIVE.
+        if ((flag == 1) && emptyHost) {
+            // use RFC 2732 style string to ensure that we get Inet6Address
+            hostString = (addressFamily == AddressFamily.AF_INET6) ? "[::]" : "0.0.0.0";
+        } else {
+            // get the addresses for the given host name
+            hostString = emptyHost ? "localhost" : host.convertToString().toString();
+        }
+
         try {
-            if(port instanceof RubyString) {
-                port = getservbyname(context, new IRubyObject[]{port});
-            }
-
-            IRubyObject family = args.length > 2 ? args[2] : context.nil;
-            IRubyObject socktype = args.length > 3 ? args[3] : context.nil;
-            //IRubyObject protocol = args[4];
-            IRubyObject flags = args.length > 5 ? args[5] : context.nil;
-            IRubyObject reverseArg = args.length > 6 ? args[6] : context.nil;
-
-            // The Ruby Socket.getaddrinfo function supports boolean/nil/Symbol values for the
-            // reverse_lookup parameter. We need to massage all valid inputs to true/false/null.
-            Boolean reverseLookup = null;
-            if (reverseArg instanceof RubyBoolean) {
-                reverseLookup = reverseArg.isTrue();
-            } else if (reverseArg instanceof RubySymbol) {
-                String reverseString = reverseArg.toString();
-                if ("hostname".equals(reverseString)) {
-                    reverseLookup = true;
-                } else if ("numeric".equals(reverseString)) {
-                    reverseLookup = false;
-                } else {
-                    throw runtime.newArgumentError("invalid reverse_lookup flag: :" + 
-                     reverseString);
-                }
-            }
-
-            AddressFamily addressFamily = AF_INET;
-            if (!family.isNil()) {
-                addressFamily = addressFamilyFromArg(family);
-            }
-            boolean is_ipv6 = addressFamily == AddressFamily.AF_INET6;
-            boolean sock_stream = true;
-            boolean sock_dgram = true;
-
-            Sock sock = SOCK_STREAM;
-            if(!socktype.isNil()) {
-                sockFromArg(socktype);
-
-                if(sock == SOCK_STREAM) {
-                    sock_dgram = false;
-
-                } else if (sock == SOCK_DGRAM) {
-                    sock_stream = false;
-
-                }
-            }
-
-            // When Socket::AI_PASSIVE and host is nil, return 'any' address.
-            InetAddress[] addrs = null;
-
-            if(!flags.isNil() && RubyFixnum.fix2int(flags) > 0) {
-                // The value of 1 is for Socket::AI_PASSIVE.
-                int flag = RubyNumeric.fix2int(flags);
-
-                if ((flag == 1) && emptyHost ) {
-                    // use RFC 2732 style string to ensure that we get Inet6Address
-                    addrs = InetAddress.getAllByName(is_ipv6 ? "[::]" : "0.0.0.0");
-                }
-
-            }
-
-            if (addrs == null) {
-                addrs = InetAddress.getAllByName(emptyHost ? (is_ipv6 ? "[::1]" : null) : host.convertToString().toString());
-            }
+            InetAddress[] addrs = InetAddress.getAllByName(hostString);
 
             for(int i = 0; i < addrs.length; i++) {
-                int p = port.isNil() ? 0 : (int)port.convertToInteger().getLongValue();
+                // filter out unrelated address families if specified
+                if (addressFamily == AF_INET6 && !(addrs[i] instanceof Inet6Address)) continue;
+                if (addressFamily == AF_INET && !(addrs[i] instanceof Inet4Address)) continue;
                 callback.addrinfo(addrs[i], p, sock, reverseLookup);
             }
 
@@ -380,40 +336,31 @@ public class SocketUtils {
         String host, port;
 
         if (arg0 instanceof RubyArray) {
-            List list = ((RubyArray)arg0).getList();
-            int len = list.size();
+            RubyArray ary = (RubyArray) arg0;
+            final int len = ary.size();
 
             if (len < 3 || len > 4) {
-                throw runtime.newArgumentError("array size should be 3 or 4, "+len+" given");
+                throw runtime.newArgumentError("array size should be 3 or 4, "+ len +" given");
             }
 
             // if array has 4 elements, third element is ignored
-            host = list.size() == 3 ? list.get(2).toString() : list.get(3).toString();
-            port = list.get(1).toString();
+            port = ary.eltInternal(1).toString();
+            host = len == 3 ? ary.eltInternal(2).toString() : ary.eltInternal(3).toString();
 
         } else if (arg0 instanceof RubyString) {
-            String arg = ((RubyString)arg0).toString();
+            String arg = ((RubyString) arg0).toString();
             Matcher m = STRING_IPV4_ADDRESS_PATTERN.matcher(arg);
 
             if (!m.matches()) {
-                IRubyObject obj = unpack_sockaddr_in(context, arg0);
-
-                if (obj instanceof RubyArray) {
-                    List list = ((RubyArray)obj).getList();
-                    int len = list.size();
-
-                    if (len != 2) {
-                        throw runtime.newArgumentError("invalid address representation");
-                    }
-
-                    host = list.get(1).toString();
-                    port = list.get(0).toString();
-
-                } else {
-                    throw runtime.newArgumentError("invalid address string");
+                RubyArray portAndHost = Sockaddr.unpack_sockaddr_in(context, arg0);
 
+                if (portAndHost.size() != 2) {
+                    throw runtime.newArgumentError("invalid address representation");
                 }
 
+                port = portAndHost.eltInternal(0).toString();
+                host = portAndHost.eltInternal(1).toString();
+
             } else if ((host = m.group(IPV4_HOST_GROUP)) == null || host.length() == 0 ||
                     (port = m.group(IPV4_PORT_GROUP)) == null || port.length() == 0) {
 
@@ -495,19 +442,19 @@ public class SocketUtils {
         }
     }
 
-    public static InetAddress getRubyInetAddress(ByteList address) throws UnknownHostException {
+    public static InetAddress[] getRubyInetAddresses(ByteList address) throws UnknownHostException {
         // switched to String because the ByteLists were not comparing properly in 1.9 mode (encoding?
         // FIXME: Need to properly decode this string (see Helpers.decodeByteList)
         String addressString = Helpers.byteListToString(address);
 
         if (addressString.equals(BROADCAST)) {
-            return InetAddress.getByAddress(INADDR_BROADCAST);
+            return new InetAddress[] {InetAddress.getByAddress(INADDR_BROADCAST)};
 
         } else if (addressString.equals(ANY)) {
-            return InetAddress.getByAddress(INADDR_ANY);
+            return new InetAddress[] {InetAddress.getByAddress(INADDR_ANY)};
 
         } else {
-            return InetAddress.getByName(addressString);
+            return InetAddress.getAllByName(addressString);
 
         }
     }
@@ -524,6 +471,16 @@ public class SocketUtils {
         return new RaiseException(runtime, runtime.getClass("SocketError"), msg, true);
     }
 
+    public static RuntimeException sockerr_with_trace(Ruby runtime, String msg, StackTraceElement[] trace) {
+        String eol = System.getProperty("line.separator");
+        StringBuilder sb = new StringBuilder();
+        sb.append(msg);
+        for (int i = 0, il = trace.length; i < il; i++) {
+            sb.append(eol).append(trace[i].toString());
+        }
+        return new RaiseException(runtime, runtime.getClass("SocketError"), sb.toString(), true);
+    }
+
     public static int getPortFrom(ThreadContext context, IRubyObject _port) {
         int port;
         if (_port instanceof RubyInteger) {
@@ -545,7 +502,7 @@ public class SocketUtils {
     private static String getHostAddress(ThreadContext context, InetAddress addr, Boolean reverse) {
         String ret;
         if (reverse == null) {
-            ret = context.runtime.isDoNotReverseLookupEnabled() ? 
+            ret = context.runtime.isDoNotReverseLookupEnabled() ?
                    addr.getHostAddress() : addr.getCanonicalHostName();
         } else if (reverse) {
             ret = addr.getCanonicalHostName();
@@ -555,12 +512,7 @@ public class SocketUtils {
         return ret;
     }
 
-    private static String getHostAddress(ThreadContext context, InetAddress addr) {
-        return getHostAddress(context, addr, null);
-    }
-
-    private static final Pattern STRING_IPV4_ADDRESS_PATTERN =
-            Pattern.compile("((.*)\\/)?([\\.0-9]+)(:([0-9]+))?");
+    private static final Pattern STRING_IPV4_ADDRESS_PATTERN = Pattern.compile("((.*)\\/)?([\\.0-9]+)(:([0-9]+))?");
 
     private static final int IPV4_HOST_GROUP = 3;
     private static final int IPV4_PORT_GROUP = 5;
@@ -570,65 +522,148 @@ public class SocketUtils {
     private static final String ANY = "<any>";
     private static final byte[] INADDR_ANY = new byte[] {0,0,0,0}; // 0.0.0.0
 
+    // MRI: address family part of rsock_family_to_int
     static AddressFamily addressFamilyFromArg(IRubyObject domain) {
-        AddressFamily addressFamily = null;
+        IRubyObject maybeString = TypeConverter.checkStringType(domain.getRuntime(), domain);
 
-        if(domain instanceof RubyString || domain instanceof RubySymbol) {
-            String domainString = domain.toString();
-            if (!domainString.startsWith("AF_")) domainString = "AF_" + domainString;
-            addressFamily = AddressFamily.valueOf(domainString);
-        } else {
-            int domainInt = RubyNumeric.fix2int(domain);
-            addressFamily = AddressFamily.valueOf(domainInt);
+        if (!maybeString.isNil()) {
+            domain = maybeString;
         }
 
-        return addressFamily;
+        try {
+            if (domain instanceof RubyString || domain instanceof RubySymbol) {
+                String domainString = domain.toString();
+                if (domainString.startsWith("AF_")) return AddressFamily.valueOf(domainString);
+                if (domainString.startsWith("PF_"))
+                    return AddressFamily.valueOf(ProtocolFamily.valueOf(domainString).intValue());
+                return AddressFamily.valueOf("AF_" + domainString);
+            }
+
+            int domainInt = RubyNumeric.fix2int(domain);
+            return AddressFamily.valueOf(domainInt);
+        } catch (IllegalArgumentException iae) {
+            throw SocketUtils.sockerr(domain.getRuntime(), "invalid address family: " + domain);
+        }
     }
 
     static Sock sockFromArg(IRubyObject type) {
-        Sock sockType = null;
+        IRubyObject maybeString = TypeConverter.checkStringType(type.getRuntime(), type);
 
-        if(type instanceof RubyString || type instanceof RubySymbol) {
-            String typeString = type.toString();
-            sockType = Sock.valueOf("SOCK_" + typeString);
-        } else {
-            int typeInt = RubyNumeric.fix2int(type);
-            sockType = Sock.valueOf(typeInt);
+        if (!maybeString.isNil()) {
+            type = maybeString;
         }
 
-        return sockType;
+        try {
+            if(type instanceof RubyString || type instanceof RubySymbol) {
+                String typeString = type.toString();
+                if (typeString.startsWith("SOCK_")) return Sock.valueOf(typeString.toString());
+                return Sock.valueOf("SOCK_" + typeString);
+            }
+
+            int typeInt = RubyNumeric.fix2int(type);
+            return Sock.valueOf(typeInt);
+        } catch (IllegalArgumentException iae) {
+            throw SocketUtils.sockerr(type.getRuntime(), "invalid socket type: " + type);
+        }
     }
 
+    // MRI: protocol family part of rsock_family_to_int
     static ProtocolFamily protocolFamilyFromArg(IRubyObject protocol) {
-        ProtocolFamily protocolFamily = null;
+        IRubyObject maybeString = TypeConverter.checkStringType(protocol.getRuntime(), protocol);
 
-        if(protocol instanceof RubyString || protocol instanceof RubySymbol) {
-            String protocolString = protocol.toString();
-            protocolFamily = ProtocolFamily.valueOf("PF_" + protocolString);
-        } else {
-            int protocolInt = RubyNumeric.fix2int(protocol);
-            if (protocolInt == 0) return null;
-            protocolFamily = ProtocolFamily.valueOf(protocolInt);
+        if (!maybeString.isNil()) {
+            protocol = maybeString;
         }
 
-        return protocolFamily;
+        try {
+            if (protocol instanceof RubyString || protocol instanceof RubySymbol) {
+                String protocolString = protocol.toString();
+                if (protocolString.startsWith("PF_")) return ProtocolFamily.valueOf(protocolString);
+                if (protocolString.startsWith("AF_")) return ProtocolFamily.valueOf(AddressFamily.valueOf(protocolString).intValue());
+                return ProtocolFamily.valueOf("PF_" + protocolString);
+            }
+
+            int protocolInt = RubyNumeric.fix2int(protocol);
+            return ProtocolFamily.valueOf(protocolInt);
+        } catch (IllegalArgumentException iae) {
+            throw SocketUtils.sockerr(protocol.getRuntime(), "invalid protocol family: " + protocol);
+        }
     }
 
     static Protocol protocolFromArg(IRubyObject protocol) {
-        Protocol proto;
+        IRubyObject maybeString = TypeConverter.checkStringType(protocol.getRuntime(), protocol);
 
-        if(protocol instanceof RubyString || protocol instanceof RubySymbol) {
-            String protocolString = protocol.toString();
-            proto = Protocol.getProtocolByName(protocolString);
-        } else {
-            int protocolInt = RubyNumeric.fix2int(protocol);
-            proto = Protocol.getProtocolByNumber(protocolInt);
+        if (!maybeString.isNil()) {
+            protocol = maybeString;
         }
 
-        return proto;
+        try {
+            if(protocol instanceof RubyString || protocol instanceof RubySymbol) {
+                String protocolString = protocol.toString();
+                return Protocol.getProtocolByName(protocolString);
+            }
+
+            int protocolInt = RubyNumeric.fix2int(protocol);
+            return Protocol.getProtocolByNumber(protocolInt);
+        } catch (IllegalArgumentException iae) {
+            throw SocketUtils.sockerr(protocol.getRuntime(), "invalid protocol: " + protocol);
+        }
     }
-    
+
+    static SocketLevel levelFromArg(IRubyObject level) {
+        IRubyObject maybeString = TypeConverter.checkStringType(level.getRuntime(), level);
+
+        if (!maybeString.isNil()) {
+            level = maybeString;
+        }
+
+        try {
+            if (level instanceof RubyString || level instanceof RubySymbol) {
+                String levelString = level.toString();
+                if(levelString.startsWith("SOL_")) return SocketLevel.valueOf(levelString);
+                return SocketLevel.valueOf("SOL_" + levelString);
+            }
+
+            return SocketLevel.valueOf(RubyNumeric.fix2int(level));
+        } catch (IllegalArgumentException iae) {
+            throw SocketUtils.sockerr(level.getRuntime(), "invalid socket level: " + level);
+        }
+    }
+
+    static SocketOption optionFromArg(IRubyObject opt) {
+        IRubyObject maybeString = TypeConverter.checkStringType(opt.getRuntime(), opt);
+
+        if (!maybeString.isNil()) {
+            opt = maybeString;
+        }
+
+        try {
+            if (opt instanceof RubyString || opt instanceof RubySymbol) {
+                String optString = opt.toString();
+                if (optString.startsWith("SO_")) return SocketOption.valueOf(optString);
+                return SocketOption.valueOf("SO_" + optString);
+            }
+
+            return SocketOption.valueOf(RubyNumeric.fix2int(opt));
+        } catch (IllegalArgumentException iae) {
+            throw SocketUtils.sockerr(opt.getRuntime(), "invalid socket option: " + opt);
+        }
+    }
+
     public static int portToInt(IRubyObject port) {
-        return port.isNil() ? 0 : RubyNumeric.fix2int(port);
+        if (port.isNil()) return 0;
+
+        Ruby runtime = port.getRuntime();
+
+        IRubyObject maybeStr = TypeConverter.checkStringType(runtime, port);
+        if (!maybeStr.isNil()) {
+            RubyString portStr = maybeStr.convertToString();
+            jnr.netdb.Service serv = jnr.netdb.Service.getServiceByName(portStr.toString(), null);
+
+            if (serv != null) return serv.getPort();
+
+            return RubyNumeric.fix2int(portStr.to_i());
+        }
+        return RubyNumeric.fix2int(port);
     }
 }
