@@ -26,40 +26,40 @@
  ***** END LICENSE BLOCK *****/
 package org.jruby.ext.zlib;
 
-import java.io.BufferedInputStream;
-import java.io.IOException;
-import java.io.InputStream;
-import java.util.ArrayList;
-import java.util.List;
-import static org.jruby.CompatVersion.RUBY1_8;
-import static org.jruby.CompatVersion.RUBY1_9;
-
 import com.jcraft.jzlib.GZIPException;
 import com.jcraft.jzlib.GZIPInputStream;
 import com.jcraft.jzlib.Inflater;
 import org.jruby.Ruby;
 import org.jruby.RubyClass;
+import org.jruby.RubyEnumerator;
 import org.jruby.RubyException;
-import org.jruby.RubyModule;
+import org.jruby.RubyInteger;
 import org.jruby.RubyNumeric;
 import org.jruby.RubyString;
 import org.jruby.anno.FrameField;
 import org.jruby.anno.JRubyClass;
 import org.jruby.anno.JRubyMethod;
 import org.jruby.exceptions.RaiseException;
-import org.jruby.internal.runtime.methods.JavaMethod;
-import org.jruby.runtime.Helpers;
 import org.jruby.runtime.Block;
+import org.jruby.runtime.Helpers;
 import org.jruby.runtime.ObjectAllocator;
 import org.jruby.runtime.ThreadContext;
-import org.jruby.runtime.Visibility;
-import static org.jruby.runtime.Visibility.PRIVATE;
 import org.jruby.runtime.builtin.IRubyObject;
 import org.jruby.util.ByteList;
 import org.jruby.util.IOInputStream;
 import org.jruby.util.StringSupport;
 import org.jruby.util.TypeConverter;
-import org.jruby.util.io.Stream;
+import org.jruby.util.io.EncodingUtils;
+import org.jruby.util.io.PosixShim;
+
+import java.io.BufferedInputStream;
+import java.io.IOException;
+import java.io.PushbackInputStream;
+import java.util.ArrayList;
+import java.util.List;
+
+import static org.jruby.RubyIO.PARAGRAPH_SEPARATOR;
+import static org.jruby.runtime.Visibility.PRIVATE;
 
 /**
  *
@@ -87,17 +87,11 @@ public class JZlibRubyGzipReader extends RubyGzipFile {
         return result;
     }
 
-    @JRubyMethod(name = "open", required = 1, optional = 1, meta = true, compat = RUBY1_8)
     public static IRubyObject open18(final ThreadContext context, IRubyObject recv, IRubyObject[] args, Block block) {
-        Ruby runtime = recv.getRuntime();
-        IRubyObject io = Helpers.invoke(context, runtime.getFile(), "open", args[0], runtime.newString("rb"));
-
-        JZlibRubyGzipReader gzio = newInstance(recv, new IRubyObject[]{io}, block);
-
-        return RubyGzipFile.wrapBlock(context, gzio, block);
+        return open19(context, recv, args, block);
     }
 
-    @JRubyMethod(name = "open", required = 1, optional = 1, meta = true, compat = RUBY1_9)
+    @JRubyMethod(name = "open", required = 1, optional = 1, meta = true)
     public static IRubyObject open19(final ThreadContext context, IRubyObject recv, IRubyObject[] args, Block block) {
         Ruby runtime = recv.getRuntime();
         args[0] = Helpers.invoke(context, runtime.getFile(), "open", args[0], runtime.newString("rb"));
@@ -111,7 +105,6 @@ public class JZlibRubyGzipReader extends RubyGzipFile {
         super(runtime, type);
     }
 
-    @JRubyMethod(name = "initialize", visibility = PRIVATE, compat = RUBY1_8)
     public IRubyObject initialize(ThreadContext context, IRubyObject stream) {
         Ruby runtime = context.runtime;
 
@@ -129,15 +122,11 @@ public class JZlibRubyGzipReader extends RubyGzipFile {
         } catch (IOException e) {
             RaiseException re = RubyZlib.newGzipFileError(runtime, "not in gzip format");
 
-            if (getRuntime().is1_9()) {
-                byte[] input = io.getAvailIn();
-
-                if (input != null && input.length > 0) {
-                    RubyException rubye = re.getException();
-
-                    rubye.setInstanceVariable("@input", 
-                            RubyString.newString(runtime, new ByteList(input, 0, input.length)));
-                }
+            byte[] input = io.getAvailIn();
+            if (input != null && input.length > 0) {
+                RubyException rubye = re.getException();
+                rubye.setInstanceVariable("@input", 
+                        RubyString.newString(runtime, new ByteList(input, 0, input.length)));
             }
 
             throw re;
@@ -145,12 +134,12 @@ public class JZlibRubyGzipReader extends RubyGzipFile {
 
         position = 0;
         line = 0;
-        bufferedStream = new BufferedInputStream(io);
+        bufferedStream = new PushbackInputStream(new BufferedInputStream(io), 512);
 
         return this;
     }
 
-    @JRubyMethod(name = "initialize", required = 1, optional = 1, visibility = PRIVATE, compat = RUBY1_9)
+    @JRubyMethod(name = "initialize", required = 1, optional = 1, visibility = PRIVATE)
     public IRubyObject initialize19(ThreadContext context, IRubyObject[] args) {
         Ruby runtime = context.runtime;
         IRubyObject obj = initialize(context, args[0]);
@@ -168,22 +157,6 @@ public class JZlibRubyGzipReader extends RubyGzipFile {
         return obj;
     }
 
-    // These methods are here to avoid defining a singleton #path on every instance, as in MRI
-
-    @JRubyMethod(compat = RUBY1_9)
-    public IRubyObject path(ThreadContext context) {
-        return this.realIo.callMethod(context, "path");
-    }
-
-    @JRubyMethod(name = "respond_to?", frame = true, compat = RUBY1_9)
-    public IRubyObject respond_to(ThreadContext context, IRubyObject name) {
-        if (name.asJavaString().equals("path")) {
-            return this.realIo.callMethod(context, "respond_to?", name);
-        }
-
-        return Helpers.invokeSuper(context, this, name, Block.NULL_BLOCK);
-    }
-
     /**
      * Get position within this stream including that has been read by users
      * calling read + what jzlib may have speculatively read in because of
@@ -203,7 +176,7 @@ public class JZlibRubyGzipReader extends RubyGzipFile {
 
         // should invoke seek on realIo...
         realIo.callMethod(context, "seek",
-                new IRubyObject[]{runtime.newFixnum(-internalPosition()), runtime.newFixnum(Stream.SEEK_CUR)});
+                new IRubyObject[]{runtime.newFixnum(-internalPosition()), runtime.newFixnum(PosixShim.SEEK_CUR)});
 
         // ... and then reinitialize
         initialize(context, realIo);
@@ -218,11 +191,9 @@ public class JZlibRubyGzipReader extends RubyGzipFile {
 
     @JRubyMethod(name = "readline", writes = FrameField.LASTLINE)
     public IRubyObject readline(ThreadContext context) {
-        IRubyObject dst = gets_18(context, new IRubyObject[0]);
+        IRubyObject dst = gets(context, IRubyObject.NULL_ARRAY);
 
-        if (dst.isNil()) {
-            throw getRuntime().newEOFError();
-        }
+        if (dst.isNil()) throw context.runtime.newEOFError();
 
         return dst;
     }
@@ -276,8 +247,14 @@ public class JZlibRubyGzipReader extends RubyGzipFile {
 
     private IRubyObject internalSepGets(ByteList sep, int limit) throws IOException {
         ByteList result = newReadByteList();
+        boolean stripNewlines = false;
 
-        if (sep.getRealSize() == 0) sep = Stream.PARAGRAPH_SEPARATOR;
+        if (sep.getRealSize() == 0) {
+            sep = PARAGRAPH_SEPARATOR;
+            stripNewlines = true;
+        }
+
+        if (stripNewlines) skipNewlines();
 
         int ce = -1;
         
@@ -292,24 +269,40 @@ public class JZlibRubyGzipReader extends RubyGzipFile {
             result.append(ce);
         }
         
-        if (getRuntime().is1_9()) fixBrokenTrailingCharacter(result);
+        fixBrokenTrailingCharacter(result);
+
+        if (stripNewlines) skipNewlines();
 
         // io.available() only returns 0 after EOF is encountered
         // so we need to differentiate between the empty string and EOF
         if (0 == result.length() && -1 == ce) return getRuntime().getNil();
 
         line++;
-        position = result.length();
+        position += result.length();
 
         return newStr(getRuntime(), result);
     }
 
-    @JRubyMethod(name = "gets", optional = 1, writes = FrameField.LASTLINE, compat = RUBY1_8)
+    private static final int NEWLINE = '\n';
+
+    private void skipNewlines() throws IOException {
+        while (true) {
+            int b = bufferedStream.read();
+            if (b == -1) break;
+
+            if (b != NEWLINE) {
+                bufferedStream.unread(b);
+                break;
+            }
+        }
+    }
+
+    @Deprecated
     public IRubyObject gets_18(ThreadContext context, IRubyObject[] args) {
         return gets(context, args);
     }
 
-    @JRubyMethod(name = "gets", optional = 2, writes = FrameField.LASTLINE, compat = RUBY1_9)
+    @JRubyMethod(name = "gets", optional = 2, writes = FrameField.LASTLINE)
     public IRubyObject gets(ThreadContext context, IRubyObject[] args) {
         try {
             IRubyObject result = internalGets(args);
@@ -422,7 +415,7 @@ public class JZlibRubyGzipReader extends RubyGzipFile {
             if (limit != -1) rest -= read;
         }
         
-        if (getRuntime().is1_9()) fixBrokenTrailingCharacter(val);
+        fixBrokenTrailingCharacter(val);
         
         this.position += val.length();
         return newStr(getRuntime(), val);
@@ -480,26 +473,25 @@ public class JZlibRubyGzipReader extends RubyGzipFile {
         }
     }
 
-    @JRubyMethod(name = {"getc", "getbyte"}, compat = RUBY1_8)
+    @JRubyMethod(name = "getbyte")
     public IRubyObject getc() {
         try {
             int value = bufferedStream.read();
             if (value == -1) return getRuntime().getNil();
 
             position++;
-            
+
             return getRuntime().newFixnum(value);
         } catch (IOException ioe) {
             throw getRuntime().newIOErrorFromException(ioe);
         }
     }
 
-    @JRubyMethod(name = "getbyte", compat = RUBY1_9)
     public IRubyObject getbyte() {
         return getc();
     }
 
-    @JRubyMethod(name = "readbyte", compat = RUBY1_9)
+    @JRubyMethod(name = "readbyte")
     public IRubyObject readbyte() {
         IRubyObject dst = getbyte();
         if (dst.isNil()) {
@@ -508,7 +500,7 @@ public class JZlibRubyGzipReader extends RubyGzipFile {
         return dst;
     }
 
-    @JRubyMethod(name = "getc", compat = RUBY1_9)
+    @JRubyMethod(name = "getc")
     public IRubyObject getc_19() {
         try {
             int value = bufferedStream.read();
@@ -517,7 +509,7 @@ public class JZlibRubyGzipReader extends RubyGzipFile {
             position++;
             // TODO: must handle encoding. Move encoding handling methods to util class from RubyIO and use it.
             // TODO: StringIO needs a love, too.
-            return getRuntime().newString("" + (char) (value & 0xFF));
+            return getRuntime().newString(String.valueOf((char) (value & 0xFF)));
         } catch (IOException ioe) {
             throw getRuntime().newIOErrorFromException(ioe);
         }
@@ -532,9 +524,13 @@ public class JZlibRubyGzipReader extends RubyGzipFile {
         // encountered the actual EOF during the reading.
         // So, we compensate for that to provide MRI
         // compatible behavior.
-        bufferedStream.mark(16);
-        bufferedStream.read();
-        bufferedStream.reset();
+        byte[] bytes = new byte[16];
+        int read = bufferedStream.read(bytes, 0, bytes.length);
+
+        // We are already at EOF.
+        if (read == -1) return true;
+
+        bufferedStream.unread(bytes, 0, read);
 
         return bufferedStream.available() == 0;
     }
@@ -637,7 +633,9 @@ public class JZlibRubyGzipReader extends RubyGzipFile {
 
     @JRubyMethod(optional = 1)
     public IRubyObject each(ThreadContext context, IRubyObject[] args, Block block) {
-        ByteList sep = ((RubyString) getRuntime().getGlobalVariables().get("$/")).getByteList();
+        if (!block.isGiven()) return RubyEnumerator.enumeratorize(context.runtime, this, "each", args);
+
+        ByteList sep = ((RubyString) context.runtime.getGlobalVariables().get("$/")).getByteList();
 
         if (args.length > 0 && !args[0].isNil()) {
             sep = args[0].convertToString().getByteList();
@@ -647,6 +645,47 @@ public class JZlibRubyGzipReader extends RubyGzipFile {
             for (IRubyObject result = internalSepGets(sep); !result.isNil(); result = internalSepGets(sep)) {
                 block.yield(context, result);
             }
+        } catch (IOException ioe) {
+            throw context.runtime.newIOErrorFromException(ioe);
+        }
+
+        return context.nil;
+    }
+
+    @JRubyMethod(optional = 1)
+    public IRubyObject each_line(ThreadContext context, IRubyObject[] args, Block block) {
+        if (!block.isGiven()) return RubyEnumerator.enumeratorize(context.runtime, this, "each_line", args);
+
+        return each(context, args, block);
+    }
+
+    @JRubyMethod
+    public IRubyObject ungetc(ThreadContext context, IRubyObject c) {
+        if (c.isNil()) return c;
+        if (c instanceof RubyInteger) {
+            c = EncodingUtils.encUintChr(context, ((RubyInteger) c).getIntValue(), getReadEncoding());
+        } else {
+            c = c.convertToString();
+        }
+
+        try {
+            byte[] bytes = ((RubyString) c).getBytes();
+            bufferedStream.unread(bytes);
+            position -= bytes.length;
+        } catch (IOException ioe) {
+            throw context.runtime.newIOErrorFromException(ioe);
+        }
+
+        return context.nil;
+    }
+
+    @JRubyMethod
+    public IRubyObject ungetbyte(IRubyObject b) {
+        if (b.isNil()) return b;
+
+        try {
+            bufferedStream.unread(b.convertToInteger().getIntValue());
+            position--;
         } catch (IOException ioe) {
             throw getRuntime().newIOErrorFromException(ioe);
         }
@@ -654,25 +693,15 @@ public class JZlibRubyGzipReader extends RubyGzipFile {
         return getRuntime().getNil();
     }
 
-    @JRubyMethod(optional = 1)
-    public IRubyObject each_line(ThreadContext context, IRubyObject[] args, Block block) {
-        return each(context, args, block);
-    }
-
-    @JRubyMethod
-    public IRubyObject ungetc(IRubyObject arg) {
-        return getRuntime().getNil();
-    }
-
     @JRubyMethod(optional = 1)
     public IRubyObject readlines(ThreadContext context, IRubyObject[] args) {
-        List<IRubyObject> array = new ArrayList<IRubyObject>();
+        List<IRubyObject> array = new ArrayList<>();
 
         if (args.length != 0 && args[0].isNil()) {
-            array.add(read(context, new IRubyObject[0]));
+            array.add(read(context, IRubyObject.NULL_ARRAY));
 
         } else {
-            ByteList sep = ((RubyString) getRuntime().getGlobalVariables().get("$/")).getByteList();
+            ByteList sep = ((RubyString) context.runtime.getGlobalVariables().get("$/")).getByteList();
 
             if (args.length > 0) sep = args[0].convertToString().getByteList();
 
@@ -681,7 +710,7 @@ public class JZlibRubyGzipReader extends RubyGzipFile {
                     array.add(result);
                 }
             } catch (IOException ioe) {
-                throw getRuntime().newIOErrorFromException(ioe);
+                throw context.runtime.newIOErrorFromException(ioe);
             }
         }
 
@@ -690,19 +719,22 @@ public class JZlibRubyGzipReader extends RubyGzipFile {
 
     @JRubyMethod
     public IRubyObject each_byte(ThreadContext context, Block block) {
+        final Ruby runtime = context.runtime;
+        if (!block.isGiven()) return RubyEnumerator.enumeratorize(runtime, this, "each_byte");
+
         try {
             int value = bufferedStream.read();
 
             while (value != -1) {
                 position++;
-                block.yield(context, getRuntime().newFixnum(value));
+                block.yield(context, runtime.newFixnum(value));
                 value = bufferedStream.read();
             }
         } catch (IOException ioe) {
-            throw getRuntime().newIOErrorFromException(ioe);
+            throw runtime.newIOErrorFromException(ioe);
         }
 
-        return getRuntime().getNil();
+        return context.nil;
     }
 
     private void fixBrokenTrailingCharacter(ByteList result) throws IOException {
@@ -721,5 +753,5 @@ public class JZlibRubyGzipReader extends RubyGzipFile {
     private long position = 0;
     private IOInputStream ioInputStream;
     private GZIPInputStream io;
-    private InputStream bufferedStream;
+    private PushbackInputStream bufferedStream;
 }
