@@ -2,19 +2,20 @@ package org.jruby.ir.instructions;
 
 import org.jruby.ir.IRVisitor;
 import org.jruby.ir.Operation;
-import org.jruby.ir.operands.Operand;
+import org.jruby.ir.transformations.inlining.CloneInfo;
 
-public class ExceptionRegionEndMarkerInstr extends Instr {
+public class ExceptionRegionEndMarkerInstr extends NoOperandInstr implements FixedArityInstr {
     public ExceptionRegionEndMarkerInstr() {
         super(Operation.EXC_REGION_END);
     }
 
-    public Operand[] getOperands() {
-        return EMPTY_OPERANDS;
-    }
-
     @Override
     public void visit(IRVisitor visitor) {
         visitor.ExceptionRegionEndMarkerInstr(this);
     }
+
+    @Override
+    public Instr clone(CloneInfo ii) {
+        return this;
+    }
 }
