@@ -1,7 +1,10 @@
 package org.jruby.ir.operands;
 
 import org.jruby.ir.IRVisitor;
-import org.jruby.ir.transformations.inlining.InlinerInfo;
+import org.jruby.ir.persistence.IRReaderDecoder;
+import org.jruby.ir.persistence.IRWriterEncoder;
+import org.jruby.ir.transformations.inlining.CloneInfo;
+import org.jruby.parser.StaticScope;
 import org.jruby.runtime.DynamicScope;
 import org.jruby.runtime.ThreadContext;
 import org.jruby.runtime.builtin.IRubyObject;
@@ -13,23 +16,29 @@ public class AsString extends Operand {
     final private Operand source;
 
     public AsString(Operand source) {
-        if (source == null) source = new StringLiteral("");
-        this.source = source;
+        super();
+
+        this.source = source == null ? StringLiteral.EMPTY_STRING : source;
     }
 
     @Override
-    public Object retrieve(ThreadContext context, IRubyObject self, DynamicScope currDynScope, Object[] temp) {
-        return ((IRubyObject)source.retrieve(context, self, currDynScope, temp)).asString();
+    public OperandType getOperandType() {
+        return OperandType.AS_STRING;
+    }
+
+    @Override
+    public Object retrieve(ThreadContext context, IRubyObject self, StaticScope currScope, DynamicScope currDynScope, Object[] temp) {
+        return ((IRubyObject) source.retrieve(context, self, currScope, currDynScope, temp)).asString();
     }
 
     @Override
     public Operand getSimplifiedOperand(Map<Operand, Operand> valueMap, boolean force) {
         Operand newSource = source.getSimplifiedOperand(valueMap, force);
-        return (newSource == source) ? this : new AsString(newSource);
+        return newSource == source ? this : new AsString(newSource);
     }
 
     @Override
-    public Operand cloneForInlining(InlinerInfo ii) {
+    public Operand cloneForInlining(CloneInfo ii) {
         return new AsString(source.cloneForInlining(ii));
     }
 
@@ -38,11 +47,6 @@ public class AsString extends Operand {
         source.addUsedVariables(l);
     }
 
-    @Override
-    public String toString() {
-        return "#{" + source + "}";
-    }
-
     public Operand getSource() {
         return source;
     }
@@ -51,4 +55,19 @@ public class AsString extends Operand {
     public void visit(IRVisitor visitor) {
         visitor.AsString(this);
     }
+
+    @Override
+    public void encode(IRWriterEncoder e) {
+        super.encode(e);
+        e.encode(getSource());
+    }
+
+    public static AsString decode(IRReaderDecoder d) {
+        return new AsString(d.decodeOperand());
+    }
+
+    @Override
+    public String toString() {
+        return "#{" + source + "}";
+    }
 }
