@@ -11,13 +11,11 @@ import org.jruby.javasupport.JavaMethod;
 public abstract class MethodInvoker extends RubyToJavaInvoker {
 
     MethodInvoker(RubyModule host, List<Method> methods) {
-        super(host, methods.toArray(new Method[methods.size()]));
-        trySetAccessible(getAccessibleObjects());
+        super(host, setAccessible( methods.toArray(new Method[methods.size()]) ) );
     }
 
     MethodInvoker(RubyModule host, Method method) {
-        super(host, new Method[] { method });
-        trySetAccessible(getAccessibleObjects());
+        super(host, setAccessible(method));
     }
 
     @Override
@@ -46,6 +44,7 @@ public abstract class MethodInvoker extends RubyToJavaInvoker {
     }
 
     @Override
+    @Deprecated
     protected final boolean isMemberVarArgs(Member member) {
         return ((Method) member).isVarArgs();
     }
