@@ -2,15 +2,19 @@ package org.jruby.java.proxies;
 
 import java.lang.reflect.Constructor;
 import java.lang.reflect.Method;
+import java.lang.reflect.Modifier;
 import java.util.Arrays;
+
 import org.jruby.Ruby;
 import org.jruby.RubyArray;
+import org.jruby.RubyBoolean;
 import org.jruby.RubyClass;
 import org.jruby.RubyInstanceConfig;
 import org.jruby.RubyModule;
 import org.jruby.anno.JRubyMethod;
 import org.jruby.exceptions.RaiseException;
 import org.jruby.internal.runtime.methods.DynamicMethod;
+import org.jruby.internal.runtime.methods.JavaMethod;
 import org.jruby.internal.runtime.methods.JavaMethod.JavaMethodN;
 import org.jruby.internal.runtime.methods.JavaMethod.JavaMethodOne;
 import org.jruby.internal.runtime.methods.JavaMethod.JavaMethodOneBlock;
@@ -42,6 +46,7 @@ public class JavaInterfaceTemplate {
         return JavaInterfaceTemplate;
     }
 
+    @Deprecated // not used - should go away in >= 9.2
     // not intended to be called directly by users (private)
     // OLD TODO from Ruby code:
     // This should be implemented in JavaClass.java, where we can
@@ -182,37 +187,14 @@ public class JavaInterfaceTemplate {
 
             // If we hold a Java object, we need a java_class accessor
             clazz.addMethod("java_class", new JavaClassAccessor(clazz));
-
-            // Because we implement Java interfaces now, we need a new === that's
-            // aware of those additional "virtual" supertypes
-            if (!clazz.searchMethod("===").isUndefined()) {
-                clazz.defineAlias("old_eqq", "===");
-                clazz.addMethod("===", new JavaMethodOne(clazz, Visibility.PUBLIC) {
-
-                    @Override
-                    public IRubyObject call(ThreadContext context, IRubyObject self, RubyModule clazz, String name, IRubyObject arg) {
-                        // TODO: WRONG - get interfaces from class
-                        if (arg.respondsTo("java_object")) {
-                            IRubyObject interfaces = self.getMetaClass().getInstanceVariables().getInstanceVariable("@java_interfaces");
-                            assert interfaces instanceof RubyArray : "interface list was not an array";
-
-                            return context.runtime.newBoolean(((RubyArray) interfaces).op_diff(
-                                    ((JavaObject) arg.dataGetStruct()).java_class().interfaces()).equals(RubyArray.newArray(context.runtime)));
-                        } else {
-                            return Helpers.invoke(context, self, "old_eqq", arg);
-                        }
-                    }
-                });
-            }
         }
 
         // Now we add an "implement" and "implement_all" methods to the class
         if ( ! clazz.isMethodBound("implement", false) ) {
             final RubyClass singleton = clazz.getSingletonClass();
 
-            // implement is called to force this class to create stubs for all
-            // methods in the given interface, so they'll show up in the list
-            // of methods and be invocable without passing through method_missing
+            // implement is called to force this class to create stubs for all methods in the given interface,
+            // so they'll show up in the list of methods and be invocable without passing through method_missing
             singleton.addMethod("implement", new JavaMethodOne(clazz, Visibility.PRIVATE) {
 
                 @Override
@@ -225,8 +207,7 @@ public class JavaInterfaceTemplate {
                 }
             });
 
-            // implement all forces implementation of all interfaces we intend
-            // for this class to implement
+            // implement all forces implementation of all interfaces we intend for this class to implement
             singleton.addMethod("implement_all", new JavaMethodOne(clazz, Visibility.PRIVATE) {
 
                 @Override
@@ -243,12 +224,17 @@ public class JavaInterfaceTemplate {
         }
     }
 
-    private static class InterfaceProxyFactory extends JavaMethodN { // __jcreate! and __jcreate_meta!
+    private static final class InterfaceProxyFactory extends JavaMethodN { // __jcreate! and __jcreate_meta!
 
         InterfaceProxyFactory(final RubyClass clazz) { super(clazz, Visibility.PRIVATE); }
 
+        @Override // will be called with zero args
+        public IRubyObject call(ThreadContext context, IRubyObject self, RubyModule clazz, String name, Block block) {
+            return newInterfaceProxy(self);
+        }
+
         @Override
-        public final IRubyObject call(ThreadContext context, IRubyObject self, RubyModule clazz, String name, IRubyObject[] args) {
+        public IRubyObject call(ThreadContext context, IRubyObject self, RubyModule clazz, String name, IRubyObject[] args) {
             return newInterfaceProxy(self);
         }
 
@@ -276,6 +262,7 @@ public class JavaInterfaceTemplate {
     public static void addRealImplClassNew(final RubyClass clazz) {
         clazz.setAllocator(new ObjectAllocator() {
             private Constructor proxyConstructor;
+
             public IRubyObject allocate(Ruby runtime, RubyClass klazz) {
                 // if we haven't been here before, reify the class
                 Class reifiedClass = klazz.getReifiedClass();
@@ -295,8 +282,8 @@ public class JavaInterfaceTemplate {
     private static IRubyObject newInterfaceProxy(final IRubyObject self) {
         final RubyClass current = self.getMetaClass();
         // construct the new interface impl and set it into the object
-        IRubyObject newObject = Java.newInterfaceImpl(self, Java.getInterfacesFromRubyClass(current));
-        JavaUtilities.set_java_object(self, self, newObject);
+        JavaObject newObject = Java.newInterfaceImpl(self, Java.getInterfacesFromRubyClass(current));
+        JavaUtilities.set_java_object(self, self, newObject); // self.dataWrapStruct(newObject);
         return newObject;
     }
 
@@ -305,9 +292,9 @@ public class JavaInterfaceTemplate {
         // included together. make it so.
         final Ruby runtime = context.runtime;
 
+        final IRubyObject java_class = module.getInstanceVariables().getInstanceVariable("@java_class");
         // not allowed for existing Java interface modules
-        if (module.getInstanceVariables().hasInstanceVariable("@java_class") &&
-            module.getInstanceVariables().getInstanceVariable("@java_class").isTrue()) {
+        if (java_class != null && java_class.isTrue()) {
             throw runtime.newTypeError("can not add Java interface to existing Java interface");
         }
 
@@ -341,7 +328,7 @@ public class JavaInterfaceTemplate {
             }
 
             final RubyModule target = (RubyModule) arg;
-            target.include( getInterfaceModules(self).toJavaArray() );
+            target.include( getInterfaceModules(self).toJavaArrayMaybeUnsafe() );
 
             return Helpers.invokeAs(context, clazz.getSuperClass(), self, name, arg, block);
         }
@@ -350,11 +337,8 @@ public class JavaInterfaceTemplate {
 
     @JRubyMethod
     public static IRubyObject extended(ThreadContext context, IRubyObject self, IRubyObject object) {
-        if ( ! (self instanceof RubyModule) ) {
-            throw context.runtime.newTypeError(self, context.runtime.getModule());
-        }
         RubyClass singleton = object.getSingletonClass();
-        singleton.include(new IRubyObject[] { self });
+        singleton.include(context, self);
         return singleton;
     }
 
@@ -363,7 +347,7 @@ public class JavaInterfaceTemplate {
         return JavaProxy.op_aref(context, self, args);
     }
 
-    @JRubyMethod(name = "impl", rest = true)
+    @JRubyMethod(name = "impl", rest = true) // impl(methods = true)
     public static IRubyObject impl(ThreadContext context, IRubyObject self, IRubyObject[] args, final Block implBlock) {
         final Ruby runtime = context.runtime;
 
@@ -371,25 +355,60 @@ public class JavaInterfaceTemplate {
             throw runtime.newArgumentError("block required to call #impl on a Java interface");
         }
 
+        boolean allMethods = true;
         final IRubyObject[] methodNames;
         if ( args.length == 0 ) methodNames = null;
+        else if ( args.length == 1 && args[0] instanceof RubyBoolean ) {
+            allMethods = args[0].isTrue(); // impl(false) ... allMethods = false
+            methodNames = null;
+        }
         else {
             methodNames = args.clone();
             Arrays.sort(methodNames); // binarySearch needs a sorted array
-            // RubySymbol implements a Java compareTo thus will allways work
+            // RubySymbol implements a Java compareTo thus will always work
         }
 
-        RubyClass implClass = RubyClass.newClass(runtime, runtime.getObject());
-        implClass.include(new IRubyObject[] { self });
+        RubyClass implClass = RubyClass.newClass(runtime, runtime.getObject()); // ImplClass = Class.new
+        implClass.include(context, self); // ImplClass.include Interface
 
-        final IRubyObject implObject = implClass.callMethod(context, "new");
+        final BlockInterfaceImpl ifaceImpl = new BlockInterfaceImpl(implClass, implBlock, methodNames);
+        implClass.addMethod("method_missing", ifaceImpl); // def ImplClass.method_missing ...
 
-        implClass.addMethod("method_missing", new BlockInterfaceImpl(implClass, implBlock, methodNames));
+        final Class<?> ifaceClass = JavaClass.getJavaClass(context, ((RubyModule) self));
+        if ( methodNames == null ) {
+            final BlockInterfaceImpl.ConcreteMethod implMethod = ifaceImpl.getConcreteMethod();
+            for ( Method method : ifaceClass.getMethods() ) {
+                if ( method.isBridge() || method.isSynthetic() ) continue;
+                if ( Modifier.isStatic( method.getModifiers() ) ) continue;
+                // override default methods (by default) - users should pass down method names or impl(false) { ... }
+                if ( ! allMethods && ! Modifier.isAbstract( method.getModifiers() ) ) continue;
+                implClass.addMethodInternal(method.getName(), implMethod); // might add twice - its fine
+            }
+        }
+        else {
+            final BlockInterfaceImpl.ConcreteMethod implMethod = ifaceImpl.getConcreteMethod();
+            final Method[] decMethods = ifaceClass.getDeclaredMethods();
+            loop: for ( IRubyObject methodName : methodNames ) {
+                final String name = methodName.toString();
+                for ( int i = 0; i < decMethods.length; i++ ) {
+                    final Method method = decMethods[i];
+                    if ( method.isBridge() || method.isSynthetic() ) continue;
+                    if ( Modifier.isStatic( method.getModifiers() ) ) continue;
+                    // add if its a declared method of the interface or its super-interfaces
+                    if ( name.equals(decMethods[i].getName()) ) {
+                        implClass.addMethodInternal(name, implMethod);
+                        continue loop;
+                    }
+                }
+                // did not continue (main) loop - passed method name not found in interface
+                runtime.getWarnings().warn("`" + name + "' is not a declared method in interface " + ifaceClass.getName());
+            }
+        }
 
-        return implObject;
+        return implClass.callMethod(context, "new"); // ImplClass.new
     }
 
-    private static class BlockInterfaceImpl extends org.jruby.internal.runtime.methods.JavaMethod {
+    private static final class BlockInterfaceImpl extends JavaMethod {
 
         private final IRubyObject[] methodNames; // RubySymbol[]
         private final Block implBlock;
@@ -419,6 +438,11 @@ public class JavaInterfaceTemplate {
             return clazz.getSuperClass().callMethod(context, "method_missing", args, block);
         }
 
+        @Override
+        public final IRubyObject call(ThreadContext context, IRubyObject self, RubyModule klazz, String name, Block block) {
+            return callImpl(context, klazz, block); // avoids checkArgumentCount
+        }
+
         @Override
         public final IRubyObject call(ThreadContext context, IRubyObject self, RubyModule klazz, String name, IRubyObject arg0, Block block) {
             return callImpl(context, klazz, block, arg0); // avoids checkArgumentCount
@@ -434,30 +458,69 @@ public class JavaInterfaceTemplate {
             return callImpl(context, klazz, block, arg0, arg1, arg2); // avoids checkArgumentCount
         }
 
-        //public DynamicMethod dup() { return this; }
+        public DynamicMethod dup() { return this; }
 
-    }
+        final ConcreteMethod getConcreteMethod() { return new ConcreteMethod(); }
 
-    @JRubyMethod(name = "new", rest = true)
-    public static IRubyObject new_impl(ThreadContext context, IRubyObject self, IRubyObject[] args, Block block) {
-        final Ruby runtime = context.runtime;
+        private final class ConcreteMethod extends JavaMethod {
+
+            ConcreteMethod() {
+                super(BlockInterfaceImpl.this.implementationClass, Visibility.PUBLIC);
+            }
+
+            @Override
+            public IRubyObject call(ThreadContext context, IRubyObject self, RubyModule klazz, String name, Block block) {
+                final IRubyObject[] nargs = new IRubyObject[] { context.runtime.newSymbol(name) };
+                return BlockInterfaceImpl.this.callImpl(context, klazz, block, nargs);
+            }
+
+            @Override
+            public IRubyObject call(ThreadContext context, IRubyObject self, RubyModule klazz, String name, IRubyObject arg0, Block block) {
+                final IRubyObject[] nargs = new IRubyObject[] { context.runtime.newSymbol(name), arg0 };
+                return BlockInterfaceImpl.this.callImpl(context, klazz, block, nargs);
+            }
+
+            @Override
+            public IRubyObject call(ThreadContext context, IRubyObject self, RubyModule klazz, String name, IRubyObject arg0, IRubyObject arg1, Block block) {
+                final IRubyObject[] nargs = new IRubyObject[] { context.runtime.newSymbol(name), arg0, arg1 };
+                return BlockInterfaceImpl.this.callImpl(context, klazz, block, nargs);
+            }
+
+            @Override
+            public IRubyObject call(ThreadContext context, IRubyObject self, RubyModule klazz, String name, IRubyObject arg0, IRubyObject arg1, IRubyObject arg2, Block block) {
+                final IRubyObject[] nargs = new IRubyObject[] { context.runtime.newSymbol(name), arg0, arg1, arg2 };
+                return BlockInterfaceImpl.this.callImpl(context, klazz, block, nargs);
+            }
+
+            @Override
+            public IRubyObject call(ThreadContext context, IRubyObject self, RubyModule klazz, String name, IRubyObject[] args, Block block) {
+                switch (args.length) {
+                    case 0:
+                        return call(context, self, klazz, name, block);
+                    case 1:
+                        return call(context, self, klazz, name, args[0], block);
+                    case 2:
+                        return call(context, self, klazz, name, args[0], args[1], block);
+                    case 3:
+                        return call(context, self, klazz, name, args[0], args[1], args[2], block);
+                    default:
+                        final IRubyObject[] nargs = new IRubyObject[args.length + 1];
+                        nargs[0] = context.runtime.newSymbol(name);
+                        System.arraycopy(args, 0, nargs, 1, args.length);
+                        return BlockInterfaceImpl.this.callImpl(context, klazz, block, nargs);
+                }
+            }
 
-        RubyClass implClass = (RubyClass) self.getInstanceVariables().getInstanceVariable("@__implementation");
-        if (implClass == null) {
-            implClass = RubyClass.newClass(runtime, runtime.getClass("InterfaceJavaProxy"));
-            implClass.include(new IRubyObject[] { self });
-            Helpers.setInstanceVariable(implClass, self, "@__implementation");
         }
 
-        return Helpers.invoke(context, implClass, "new", args, block);
     }
 
     private static JavaClass getJavaClassForInterface(final IRubyObject module) {
         return (JavaClass) module.getInstanceVariables().getInstanceVariable("@java_class");
     }
 
-    private static RubyArray getJavaInterfaces(final IRubyObject module) {
-        return (RubyArray) module.getInstanceVariables().getInstanceVariable("@java_interfaces");
+    private static RubyArray getJavaInterfaces(final IRubyObject clazz) {
+        return (RubyArray) clazz.getInstanceVariables().getInstanceVariable("@java_interfaces");
     }
 
     private static RubyArray getInterfaceModules(final IRubyObject module) {
