@@ -55,8 +55,10 @@ import org.jruby.javasupport.proxy.InternalJavaProxy;
 import org.jruby.javasupport.proxy.JavaProxyClass;
 import org.jruby.javasupport.proxy.JavaProxyMethod;
 import org.jruby.runtime.ObjectAllocator;
+import org.jruby.runtime.ThreadContext;
 import org.jruby.runtime.builtin.IRubyObject;
 
+import static org.jruby.RubyModule.undefinedMethodMessage;
 import static org.jruby.util.CodegenUtils.getBoxType;
 import static org.jruby.util.CodegenUtils.prettyParams;
 
@@ -129,7 +131,7 @@ public class JavaMethod extends JavaCallable {
             return create(runtime, javaClass.getMethod(methodName, argumentTypes));
         }
         catch (NoSuchMethodException e) {
-            throw runtime.newNameError("undefined method '" + methodName + "' for class '" + javaClass.getName() + "'", methodName);
+            throw runtime.newNameError(undefinedMethodMessage(methodName, javaClass.getName(), false), methodName);
         }
     }
 
@@ -139,7 +141,7 @@ public class JavaMethod extends JavaCallable {
             return create(runtime, javaClass.getDeclaredMethod(methodName, argumentTypes));
         }
         catch (NoSuchMethodException e) {
-            throw runtime.newNameError("undefined method '" + methodName + "' for class '" + javaClass.getName() + "'", methodName);
+            throw runtime.newNameError(undefinedMethodMessage(methodName, javaClass.getName(), false), methodName);
         }
     }
 
@@ -209,14 +211,14 @@ public class JavaMethod extends JavaCallable {
     }
 
     @JRubyMethod(rest = true)
-    public IRubyObject invoke(IRubyObject[] args) {
+    public IRubyObject invoke(ThreadContext context, IRubyObject[] args) {
         checkArity(args.length - 1);
 
         final IRubyObject invokee = args[0];
         final Object[] arguments = convertArguments(args, 1);
 
         if ( invokee.isNil() ) {
-            return invokeWithExceptionHandling(method, null, arguments);
+            return invokeWithExceptionHandling(context, method, null, arguments);
         }
 
         final Object javaInvokee;
@@ -243,7 +245,7 @@ public class JavaMethod extends JavaCallable {
                 JavaProxyClass jpc = ((InternalJavaProxy) javaInvokee).___getProxyClass();
                 JavaProxyMethod jpm = jpc.getMethod( method.getName(), parameterTypes );
                 if ( jpm != null && jpm.hasSuperImplementation() ) {
-                    return invokeWithExceptionHandling(jpm.getSuperMethod(), javaInvokee, arguments);
+                    return invokeWithExceptionHandling(context, jpm.getSuperMethod(), javaInvokee, arguments);
                 }
             }
         }
@@ -251,15 +253,15 @@ public class JavaMethod extends JavaCallable {
             javaInvokee = null;
         }
 
-        return invokeWithExceptionHandling(method, javaInvokee, arguments);
+        return invokeWithExceptionHandling(context, method, javaInvokee, arguments);
     }
 
     @JRubyMethod(rest = true)
-    public IRubyObject invoke_static(IRubyObject[] args) {
+    public IRubyObject invoke_static(ThreadContext context, IRubyObject[] args) {
         checkArity(args.length);
 
         final Object[] arguments = convertArguments(args, 0);
-        return invokeWithExceptionHandling(method, null, arguments);
+        return invokeWithExceptionHandling(context, method, null, arguments);
     }
 
     @JRubyMethod
@@ -277,105 +279,105 @@ public class JavaMethod extends JavaCallable {
         return Java.getInstance(getRuntime(), method.getTypeParameters());
     }
 
-    public IRubyObject invokeDirect(Object javaInvokee, Object[] args) {
+    public IRubyObject invokeDirect(ThreadContext context, Object javaInvokee, Object[] args) {
         checkArity(args.length);
         checkInstanceof(javaInvokee);
 
         if (mightBeProxy(javaInvokee)) {
-            return tryProxyInvocation(javaInvokee, args);
+            return tryProxyInvocation(context, javaInvokee, args);
         }
 
-        return invokeDirectWithExceptionHandling(method, javaInvokee, args);
+        return invokeDirectWithExceptionHandling(context, method, javaInvokee, args);
     }
 
-    public IRubyObject invokeDirect(Object javaInvokee) {
+    public IRubyObject invokeDirect(ThreadContext context, Object javaInvokee) {
         assert method.getDeclaringClass().isInstance(javaInvokee);
 
         checkArity(0);
 
         if (mightBeProxy(javaInvokee)) {
-            return tryProxyInvocation(javaInvokee);
+            return tryProxyInvocation(context, javaInvokee);
         }
 
-        return invokeDirectWithExceptionHandling(method, javaInvokee);
+        return invokeDirectWithExceptionHandling(context, method, javaInvokee);
     }
 
-    public IRubyObject invokeDirect(Object javaInvokee, Object arg0) {
+    public IRubyObject invokeDirect(ThreadContext context, Object javaInvokee, Object arg0) {
         assert method.getDeclaringClass().isInstance(javaInvokee);
 
         checkArity(1);
 
         if (mightBeProxy(javaInvokee)) {
-            return tryProxyInvocation(javaInvokee, arg0);
+            return tryProxyInvocation(context, javaInvokee, arg0);
         }
 
-        return invokeDirectWithExceptionHandling(method, javaInvokee, arg0);
+        return invokeDirectWithExceptionHandling(context, method, javaInvokee, arg0);
     }
 
-    public IRubyObject invokeDirect(Object javaInvokee, Object arg0, Object arg1) {
+    public IRubyObject invokeDirect(ThreadContext context, Object javaInvokee, Object arg0, Object arg1) {
         assert method.getDeclaringClass().isInstance(javaInvokee);
 
         checkArity(2);
 
         if (mightBeProxy(javaInvokee)) {
-            return tryProxyInvocation(javaInvokee, arg0, arg1);
+            return tryProxyInvocation(context, javaInvokee, arg0, arg1);
         }
 
-        return invokeDirectWithExceptionHandling(method, javaInvokee, arg0, arg1);
+        return invokeDirectWithExceptionHandling(context, method, javaInvokee, arg0, arg1);
     }
 
-    public IRubyObject invokeDirect(Object javaInvokee, Object arg0, Object arg1, Object arg2) {
+    public IRubyObject invokeDirect(ThreadContext context, Object javaInvokee, Object arg0, Object arg1, Object arg2) {
         assert method.getDeclaringClass().isInstance(javaInvokee);
 
         checkArity(3);
 
         if (mightBeProxy(javaInvokee)) {
-            return tryProxyInvocation(javaInvokee, arg0, arg1, arg2);
+            return tryProxyInvocation(context, javaInvokee, arg0, arg1, arg2);
         }
 
-        return invokeDirectWithExceptionHandling(method, javaInvokee, arg0, arg1, arg2);
+        return invokeDirectWithExceptionHandling(context, method, javaInvokee, arg0, arg1, arg2);
     }
 
-    public IRubyObject invokeDirect(Object javaInvokee, Object arg0, Object arg1, Object arg2, Object arg3) {
+    public IRubyObject invokeDirect(ThreadContext context, Object javaInvokee, Object arg0, Object arg1, Object arg2, Object arg3) {
         assert method.getDeclaringClass().isInstance(javaInvokee);
 
         checkArity(4);
 
         if (mightBeProxy(javaInvokee)) {
-            return tryProxyInvocation(javaInvokee, arg0, arg1, arg2, arg3);
+            return tryProxyInvocation(context, javaInvokee, arg0, arg1, arg2, arg3);
         }
 
-        return invokeDirectWithExceptionHandling(method, javaInvokee, arg0, arg1, arg2, arg3);
+        return invokeDirectWithExceptionHandling(context, method, javaInvokee, arg0, arg1, arg2, arg3);
     }
 
-    public IRubyObject invokeStaticDirect(Object[] args) {
+    public IRubyObject invokeStaticDirect(ThreadContext context, Object[] args) {
         checkArity(args.length);
-        return invokeDirectWithExceptionHandling(method, null, args);
+        return invokeDirectWithExceptionHandling(context, method, null, args);
     }
 
-    public IRubyObject invokeStaticDirect() {
+    public IRubyObject invokeStaticDirect(ThreadContext context) {
         checkArity(0);
-        return invokeDirectWithExceptionHandling(method, null);
+        return invokeDirectWithExceptionHandling(context, method, null);
     }
 
-    public IRubyObject invokeStaticDirect(Object arg0) {
+    public IRubyObject invokeStaticDirect(ThreadContext context, Object arg0) {
         checkArity(1);
-        return invokeDirectWithExceptionHandling(method, null, arg0);
+        return invokeDirectWithExceptionHandling(context, method, null, arg0);
     }
 
-    public IRubyObject invokeStaticDirect(Object arg0, Object arg1) {
+    public IRubyObject invokeStaticDirect(ThreadContext context, Object arg0, Object arg1) {
         checkArity(2);
-        return invokeDirectWithExceptionHandling(method, null, arg0, arg1);
+        return invokeDirectWithExceptionHandling(context, method, null, arg0, arg1);
     }
 
-    public IRubyObject invokeStaticDirect(Object arg0, Object arg1, Object arg2) {
+    public IRubyObject invokeStaticDirect(ThreadContext context, Object arg0, Object arg1, Object arg2) {
         checkArity(3);
-        return invokeDirectWithExceptionHandling(method, null, arg0, arg1, arg2);
+        return invokeDirectWithExceptionHandling(context, method, null, arg0, arg1, arg2);
     }
 
-    public IRubyObject invokeStaticDirect(Object arg0, Object arg1, Object arg2, Object arg3) {
+    public IRubyObject invokeStaticDirect(ThreadContext context, Object arg0, Object arg1, Object arg2, Object arg3) {
         checkArity(4);
-        return invokeDirectWithExceptionHandling(method, null, arg0, arg1, arg2, arg3);
+        return invokeDirectWithExceptionHandling(context, method, null, arg0, arg1, arg2, arg3);
     }
 
     private void checkInstanceof(Object javaInvokee) throws RaiseException {
@@ -384,7 +386,7 @@ public class JavaMethod extends JavaCallable {
         }
     }
 
-    private IRubyObject invokeWithExceptionHandling(Method method, Object javaInvokee, Object[] arguments) {
+    private IRubyObject invokeWithExceptionHandling(ThreadContext context, Method method, Object javaInvokee, Object[] arguments) {
         try {
             Object result = method.invoke(javaInvokee, arguments);
             return returnConverter.convert(getRuntime(), result);
@@ -393,13 +395,13 @@ public class JavaMethod extends JavaCallable {
         } catch (IllegalAccessException iae) {
             return handleIllegalAccessEx(iae, method);
         } catch (InvocationTargetException ite) {
-            return handleInvocationTargetEx(ite, method);
+            return handleInvocationTargetEx(context, ite);
         } catch (Throwable t) {
-            return handleThrowable(t, method);
+            return handleThrowable(context, t);
         }
     }
 
-    private IRubyObject invokeDirectSuperWithExceptionHandling(Method method, Object javaInvokee, Object... arguments) {
+    private IRubyObject invokeDirectSuperWithExceptionHandling(ThreadContext context, Method method, Object javaInvokee, Object... arguments) {
         // super calls from proxies must use reflected method
         // FIXME: possible to make handles do the superclass call?
         try {
@@ -410,13 +412,13 @@ public class JavaMethod extends JavaCallable {
         } catch (IllegalAccessException iae) {
             return handleIllegalAccessEx(iae, method);
         } catch (InvocationTargetException ite) {
-            return handleInvocationTargetEx(ite, method);
+            return handleInvocationTargetEx(context, ite);
         } catch (Throwable t) {
-            return handleThrowable(t, method);
+            return handleThrowable(context, t);
         }
     }
 
-    private IRubyObject invokeDirectWithExceptionHandling(Method method, Object javaInvokee, Object[] arguments) {
+    private IRubyObject invokeDirectWithExceptionHandling(ThreadContext context, Method method, Object javaInvokee, Object[] arguments) {
         try {
             Object result = method.invoke(javaInvokee, arguments);
             return convertReturn(result);
@@ -425,13 +427,13 @@ public class JavaMethod extends JavaCallable {
         } catch (IllegalAccessException iae) {
             return handleIllegalAccessEx(iae, method);
         } catch (InvocationTargetException ite) {
-            return handleInvocationTargetEx(ite, method);
+            return handleInvocationTargetEx(context, ite);
         } catch (Throwable t) {
-            return handleThrowable(t, method);
+            return handleThrowable(context, t);
         }
     }
 
-    private IRubyObject invokeDirectWithExceptionHandling(Method method, Object javaInvokee) {
+    private IRubyObject invokeDirectWithExceptionHandling(ThreadContext context, Method method, Object javaInvokee) {
         try {
             Object result = method.invoke(javaInvokee);
             return convertReturn(result);
@@ -440,13 +442,13 @@ public class JavaMethod extends JavaCallable {
         } catch (IllegalAccessException iae) {
             return handleIllegalAccessEx(iae, method);
         } catch (InvocationTargetException ite) {
-            return handleInvocationTargetEx(ite, method);
+            return handleInvocationTargetEx(context, ite);
         } catch (Throwable t) {
-            return handleThrowable(t, method);
+            return handleThrowable(context, t);
         }
     }
 
-    private IRubyObject invokeDirectWithExceptionHandling(Method method, Object javaInvokee, Object arg0) {
+    private IRubyObject invokeDirectWithExceptionHandling(ThreadContext context, Method method, Object javaInvokee, Object arg0) {
         try {
             Object result = method.invoke(javaInvokee, arg0);
             return convertReturn(result);
@@ -455,13 +457,13 @@ public class JavaMethod extends JavaCallable {
         } catch (IllegalAccessException iae) {
             return handleIllegalAccessEx(iae, method);
         } catch (InvocationTargetException ite) {
-            return handleInvocationTargetEx(ite, method);
+            return handleInvocationTargetEx(context, ite);
         } catch (Throwable t) {
-            return handleThrowable(t, method);
+            return handleThrowable(context, t);
         }
     }
 
-    private IRubyObject invokeDirectWithExceptionHandling(Method method, Object javaInvokee, Object arg0, Object arg1) {
+    private IRubyObject invokeDirectWithExceptionHandling(ThreadContext context, Method method, Object javaInvokee, Object arg0, Object arg1) {
         try {
             Object result = method.invoke(javaInvokee, arg0, arg1);
             return convertReturn(result);
@@ -470,13 +472,13 @@ public class JavaMethod extends JavaCallable {
         } catch (IllegalAccessException iae) {
             return handleIllegalAccessEx(iae, method);
         } catch (InvocationTargetException ite) {
-            return handleInvocationTargetEx(ite, method);
+            return handleInvocationTargetEx(context, ite);
         } catch (Throwable t) {
-            return handleThrowable(t, method);
+            return handleThrowable(context, t);
         }
     }
 
-    private IRubyObject invokeDirectWithExceptionHandling(Method method, Object javaInvokee, Object arg0, Object arg1, Object arg2) {
+    private IRubyObject invokeDirectWithExceptionHandling(ThreadContext context, Method method, Object javaInvokee, Object arg0, Object arg1, Object arg2) {
         try {
             Object result = method.invoke(javaInvokee, arg0, arg1, arg2);
             return convertReturn(result);
@@ -485,13 +487,13 @@ public class JavaMethod extends JavaCallable {
         } catch (IllegalAccessException iae) {
             return handleIllegalAccessEx(iae, method);
         } catch (InvocationTargetException ite) {
-            return handleInvocationTargetEx(ite, method);
+            return handleInvocationTargetEx(context, ite);
         } catch (Throwable t) {
-            return handleThrowable(t, method);
+            return handleThrowable(context, t);
         }
     }
 
-    private IRubyObject invokeDirectWithExceptionHandling(Method method, Object javaInvokee, Object arg0, Object arg1, Object arg2, Object arg3) {
+    private IRubyObject invokeDirectWithExceptionHandling(ThreadContext context, Method method, Object javaInvokee, Object arg0, Object arg1, Object arg2, Object arg3) {
         try {
             Object result = method.invoke(javaInvokee, arg0, arg1, arg2, arg3);
             return convertReturn(result);
@@ -500,9 +502,9 @@ public class JavaMethod extends JavaCallable {
         } catch (IllegalAccessException iae) {
             return handleIllegalAccessEx(iae, method);
         } catch (InvocationTargetException ite) {
-            return handleInvocationTargetEx(ite, method);
+            return handleInvocationTargetEx(context, ite);
         } catch (Throwable t) {
-            return handleThrowable(t, method);
+            return handleThrowable(context, t);
         }
     }
 
@@ -525,6 +527,10 @@ public class JavaMethod extends JavaCallable {
     //    return parameterTypes;
     //}
 
+    public String getName() {
+        return method.getName();
+    }
+
     @Override
     public Class<?>[] getExceptionTypes() {
         return method.getExceptionTypes();
@@ -561,8 +567,8 @@ public class JavaMethod extends JavaCallable {
         str.append("#<");
         str.append( getType().toString() ).append('/').append(method.getName());
         inspectParameterTypes(str, this);
-        str.append(">");
-        return getRuntime().newString( str.toString() );
+        str.append('>');
+        return RubyString.newString(getRuntime(), str);
     }
 
     @JRubyMethod(name = "static?")
@@ -598,63 +604,63 @@ public class JavaMethod extends JavaCallable {
         return javaInvokee instanceof InternalJavaProxy && !isFinal;
     }
 
-    private IRubyObject tryProxyInvocation(Object javaInvokee, Object... args) {
+    private IRubyObject tryProxyInvocation(ThreadContext context, Object javaInvokee, Object... args) {
         JavaProxyClass jpc = ((InternalJavaProxy) javaInvokee).___getProxyClass();
         JavaProxyMethod jpm;
         if ((jpm = jpc.getMethod(method.getName(), parameterTypes)) != null && jpm.hasSuperImplementation()) {
-            return invokeDirectSuperWithExceptionHandling(jpm.getSuperMethod(), javaInvokee, args);
+            return invokeDirectSuperWithExceptionHandling(context, jpm.getSuperMethod(), javaInvokee, args);
         } else {
-            return invokeDirectWithExceptionHandling(method, javaInvokee, args);
+            return invokeDirectWithExceptionHandling(context, method, javaInvokee, args);
         }
     }
 
-    private IRubyObject tryProxyInvocation(Object javaInvokee) {
+    private IRubyObject tryProxyInvocation(ThreadContext context, Object javaInvokee) {
         JavaProxyClass jpc = ((InternalJavaProxy) javaInvokee).___getProxyClass();
         JavaProxyMethod jpm;
         if ((jpm = jpc.getMethod(method.getName(), parameterTypes)) != null && jpm.hasSuperImplementation()) {
-            return invokeDirectSuperWithExceptionHandling(jpm.getSuperMethod(), javaInvokee);
+            return invokeDirectSuperWithExceptionHandling(context, jpm.getSuperMethod(), javaInvokee);
         } else {
-            return invokeDirectWithExceptionHandling(method, javaInvokee);
+            return invokeDirectWithExceptionHandling(context, method, javaInvokee);
         }
     }
 
-    private IRubyObject tryProxyInvocation(Object javaInvokee, Object arg0) {
+    private IRubyObject tryProxyInvocation(ThreadContext context, Object javaInvokee, Object arg0) {
         JavaProxyClass jpc = ((InternalJavaProxy) javaInvokee).___getProxyClass();
         JavaProxyMethod jpm;
         if ((jpm = jpc.getMethod(method.getName(), parameterTypes)) != null && jpm.hasSuperImplementation()) {
-            return invokeDirectSuperWithExceptionHandling(jpm.getSuperMethod(), javaInvokee, arg0);
+            return invokeDirectSuperWithExceptionHandling(context, jpm.getSuperMethod(), javaInvokee, arg0);
         } else {
-            return invokeDirectWithExceptionHandling(method, javaInvokee, arg0);
+            return invokeDirectWithExceptionHandling(context, method, javaInvokee, arg0);
         }
     }
 
-    private IRubyObject tryProxyInvocation(Object javaInvokee, Object arg0, Object arg1) {
+    private IRubyObject tryProxyInvocation(ThreadContext context, Object javaInvokee, Object arg0, Object arg1) {
         JavaProxyClass jpc = ((InternalJavaProxy) javaInvokee).___getProxyClass();
         JavaProxyMethod jpm;
         if ((jpm = jpc.getMethod(method.getName(), parameterTypes)) != null && jpm.hasSuperImplementation()) {
-            return invokeDirectSuperWithExceptionHandling(jpm.getSuperMethod(), javaInvokee, arg0, arg1);
+            return invokeDirectSuperWithExceptionHandling(context, jpm.getSuperMethod(), javaInvokee, arg0, arg1);
         } else {
-            return invokeDirectWithExceptionHandling(method, javaInvokee, arg0, arg1);
+            return invokeDirectWithExceptionHandling(context, method, javaInvokee, arg0, arg1);
         }
     }
 
-    private IRubyObject tryProxyInvocation(Object javaInvokee, Object arg0, Object arg1, Object arg2) {
+    private IRubyObject tryProxyInvocation(ThreadContext context, Object javaInvokee, Object arg0, Object arg1, Object arg2) {
         JavaProxyClass jpc = ((InternalJavaProxy) javaInvokee).___getProxyClass();
         JavaProxyMethod jpm;
         if ((jpm = jpc.getMethod(method.getName(), parameterTypes)) != null && jpm.hasSuperImplementation()) {
-            return invokeDirectSuperWithExceptionHandling(jpm.getSuperMethod(), javaInvokee, arg0, arg1, arg2);
+            return invokeDirectSuperWithExceptionHandling(context, jpm.getSuperMethod(), javaInvokee, arg0, arg1, arg2);
         } else {
-            return invokeDirectWithExceptionHandling(method, javaInvokee, arg0, arg1, arg2);
+            return invokeDirectWithExceptionHandling(context, method, javaInvokee, arg0, arg1, arg2);
         }
     }
 
-    private IRubyObject tryProxyInvocation(Object javaInvokee, Object arg0, Object arg1, Object arg2, Object arg3) {
+    private IRubyObject tryProxyInvocation(ThreadContext context, Object javaInvokee, Object arg0, Object arg1, Object arg2, Object arg3) {
         JavaProxyClass jpc = ((InternalJavaProxy) javaInvokee).___getProxyClass();
         JavaProxyMethod jpm;
         if ((jpm = jpc.getMethod(method.getName(), parameterTypes)) != null && jpm.hasSuperImplementation()) {
-            return invokeDirectSuperWithExceptionHandling(jpm.getSuperMethod(), javaInvokee, arg0, arg1, arg2, arg3);
+            return invokeDirectSuperWithExceptionHandling(context, jpm.getSuperMethod(), javaInvokee, arg0, arg1, arg2, arg3);
         } else {
-            return invokeDirectWithExceptionHandling(method, javaInvokee, arg0, arg1, arg2, arg3);
+            return invokeDirectWithExceptionHandling(context, method, javaInvokee, arg0, arg1, arg2, arg3);
         }
     }
 
