@@ -3,20 +3,18 @@ package org.jruby.javasupport.binding;
 import org.jruby.Ruby;
 import org.jruby.RubyClass;
 import org.jruby.RubyModule;
-import org.jruby.internal.runtime.methods.DynamicMethod;
 import org.jruby.internal.runtime.methods.JavaMethod;
 import org.jruby.javasupport.Java;
 import org.jruby.javasupport.JavaClass;
 import org.jruby.javasupport.JavaSupport;
 import org.jruby.javasupport.JavaUtil;
-import org.jruby.runtime.Block;
-import org.jruby.runtime.Helpers;
 import org.jruby.runtime.ThreadContext;
 import org.jruby.runtime.builtin.IRubyObject;
 import org.jruby.util.IdUtil;
 import org.jruby.util.log.Logger;
 import org.jruby.util.log.LoggerFactory;
 
+import java.lang.annotation.Annotation;
 import java.lang.reflect.Field;
 import java.lang.reflect.Method;
 import java.lang.reflect.Modifier;
@@ -25,7 +23,6 @@ import java.util.Arrays;
 import java.util.Collections;
 import java.util.HashMap;
 import java.util.List;
-import java.util.ListIterator;
 import java.util.Map;
 
 import static org.jruby.runtime.Visibility.PUBLIC;
@@ -34,34 +31,39 @@ import static org.jruby.runtime.Visibility.PUBLIC;
 * Created by headius on 2/26/15.
 */
 public abstract class Initializer {
+
+    static final Logger LOG = LoggerFactory.getLogger(Initializer.class);
+
+    public static final boolean DEBUG_SCALA = false;
+
     protected final Ruby runtime;
     protected final JavaSupport javaSupport;
     protected final Class javaClass;
 
-    private static final Logger LOG = LoggerFactory.getLogger("Initializer");
-
-    private static final int ACC_BRIDGE    = 0x00000040;
-
-    public static final boolean DEBUG_SCALA = false;
+    private static final int ACC_BRIDGE = 0x00000040;
 
     public static final String METHOD_MANGLE = "__method";
 
     private static final Map<String, String> SCALA_OPERATORS;
 
     // TODO: other reserved names?
-    private static final Map<String, AssignedName> RESERVED_NAMES = new HashMap<String, AssignedName>();
-    static {
+    private static Map<String, AssignedName> newReservedNamesMap(final int size) {
+        HashMap<String, AssignedName> RESERVED_NAMES = new HashMap<>(size + 4, 1);
         RESERVED_NAMES.put("__id__", new AssignedName("__id__", Priority.RESERVED));
         RESERVED_NAMES.put("__send__", new AssignedName("__send__", Priority.RESERVED));
         // JRUBY-5132: java.awt.Component.instance_of?() expects 2 args
         RESERVED_NAMES.put("instance_of?", new AssignedName("instance_of?", Priority.RESERVED));
+        return RESERVED_NAMES;
     }
-    protected static final Map<String, AssignedName> STATIC_RESERVED_NAMES = new HashMap<String, AssignedName>(RESERVED_NAMES);
+
+    protected static final Map<String, AssignedName> STATIC_RESERVED_NAMES;
     static {
+        STATIC_RESERVED_NAMES = newReservedNamesMap(1);
         STATIC_RESERVED_NAMES.put("new", new AssignedName("new", Priority.RESERVED));
     }
-    protected static final Map<String, AssignedName> INSTANCE_RESERVED_NAMES = new HashMap<String, AssignedName>(RESERVED_NAMES);
+    protected static final Map<String, AssignedName> INSTANCE_RESERVED_NAMES;
     static {
+        INSTANCE_RESERVED_NAMES = newReservedNamesMap(2);
         // only possible for "getClass" to be an instance method in Java
         INSTANCE_RESERVED_NAMES.put("class", new AssignedName("class", Priority.RESERVED));
         // "initialize" has meaning only for an instance (as opposed to a class)
@@ -74,18 +76,43 @@ public abstract class Initializer {
         this.javaClass = javaClass;
     }
 
-    public static RubyModule setupProxyClass(Ruby runtime, final Class<?> javaClass, final RubyClass proxy) {
+    public static RubyModule setupProxyClass(Ruby runtime, final Class<?> javaClass, RubyClass proxy) {
         setJavaClassFor(javaClass, proxy);
 
-        return new ClassInitializer(runtime, javaClass).initialize(proxy);
+        proxy.setReifiedClass((Class) javaClass);
+
+        if ( javaClass.isArray() ) {
+            flagAsJavaProxy(proxy); return proxy;
+        }
+
+        if ( javaClass.isPrimitive() ) {
+            final RubyClass proxySingleton = proxy.getSingletonClass();
+            proxySingleton.undefineMethod("new"); // remove ConcreteJavaProxy class method 'new'
+            if ( javaClass == Void.TYPE ) {
+                // special treatment ... while Java::int[4] is OK Java::void[2] is NOT!
+                proxySingleton.undefineMethod("[]"); // from JavaProxy
+                proxySingleton.undefineMethod("new_array"); // from JavaProxy
+            }
+            flagAsJavaProxy(proxy); return proxy;
+        }
+
+        proxy = new ClassInitializer(runtime, javaClass).initialize(proxy);
+        flagAsJavaProxy(proxy); return proxy;
     }
 
-    public static RubyModule setupProxyModule(Ruby runtime, final Class<?> javaClass, final RubyModule proxy) {
+    public static RubyModule setupProxyModule(Ruby runtime, final Class<?> javaClass, RubyModule proxy) {
         setJavaClassFor(javaClass, proxy);
 
         assert javaClass.isInterface();
 
-        return new InterfaceInitializer(runtime, javaClass).initialize(proxy);
+        proxy = new InterfaceInitializer(runtime, javaClass).initialize(proxy);
+        flagAsJavaProxy(proxy); return proxy;
+    }
+
+    private static void flagAsJavaProxy(final RubyModule proxy) {
+        // flag the class as a Java class proxy.
+        proxy.setJavaProxy(true);
+        proxy.getSingletonClass().setJavaProxy(true);
     }
 
     protected static void addField(
@@ -142,91 +169,104 @@ public abstract class Initializer {
     }
 
     protected static void assignStaticAliases(final State state) {
-        for (Map.Entry<String, NamedInstaller> entry : state.staticInstallers.entrySet()) {
+        final Map<String, NamedInstaller> installers = state.staticInstallers;
+        for (Map.Entry<String, NamedInstaller> entry : installers.entrySet()) {
             // no aliases for __method methods
-            if (entry.getKey().endsWith("__method")) continue;
+            if (entry.getKey().endsWith(METHOD_MANGLE)) continue;
 
             if (entry.getValue().type == NamedInstaller.STATIC_METHOD && entry.getValue().hasLocalMethod()) {
-                assignAliases((MethodInstaller) entry.getValue(), state.staticNames);
+                assignAliases((MethodInstaller) entry.getValue(), state.staticNames, installers);
             }
         }
     }
 
-    protected static void assignAliases(MethodInstaller installer, Map<String, AssignedName> assignedNames) {
-        String name = installer.name;
+    static void assignAliases(final MethodInstaller installer,
+        final Map<String, AssignedName> assignedNames, final Map<String, NamedInstaller> installers) {
+
+        final String name = installer.name;
         String rubyCasedName = JavaUtil.getRubyCasedName(name);
-        addUnassignedAlias(rubyCasedName, assignedNames, installer);
+        addUnassignedAlias(rubyCasedName, assignedNames, installer, Priority.ALIAS);
 
         String javaPropertyName = JavaUtil.getJavaPropertyName(name);
-        String rubyPropertyName = null;
 
-        for (Method method: installer.methods) {
+        final List<Method> methods = installer.methods;
+
+        for ( int i = 0; i < methods.size(); i++ ) {
+            final Method method = methods.get(i);
             Class<?>[] argTypes = method.getParameterTypes();
             Class<?> resultType = method.getReturnType();
             int argCount = argTypes.length;
 
             // Add scala aliases for apply/update to roughly equivalent Ruby names
-            if (rubyCasedName.equals("apply")) {
-                addUnassignedAlias("[]", assignedNames, installer);
+            if (name.equals("apply")) {
+                addUnassignedAlias("[]", assignedNames, installer, Priority.ALIAS);
             }
-            if (rubyCasedName.equals("update") && argCount == 2) {
-                addUnassignedAlias("[]=", assignedNames, installer);
+            if (name.equals("update") && argCount == 2) {
+                addUnassignedAlias("[]=", assignedNames, installer, Priority.ALIAS);
             }
-
             // Scala aliases for $ method names
             if (name.startsWith("$")) {
-                addUnassignedAlias(ClassInitializer.fixScalaNames(name), assignedNames, installer);
+                addUnassignedAlias(ClassInitializer.fixScalaNames(name), assignedNames, installer, Priority.ALIAS);
             }
 
+            String rubyPropertyName = null;
+
             // Add property name aliases
             if (javaPropertyName != null) {
                 if (rubyCasedName.startsWith("get_")) {
                     rubyPropertyName = rubyCasedName.substring(4);
-                    if (argCount == 0 ||                                // getFoo      => foo
-                            argCount == 1 && argTypes[0] == int.class) {    // getFoo(int) => foo(int)
-
-                        addUnassignedAlias(javaPropertyName, assignedNames, installer);
-                        addUnassignedAlias(rubyPropertyName, assignedNames, installer);
+                    if (argCount == 0) {  // getFoo      => foo
+                        addUnassignedAlias(javaPropertyName, assignedNames, installer, Priority.GET_ALIAS);
+                        addUnassignedAlias(rubyPropertyName, assignedNames, installer, Priority.GET_ALIAS);
                     }
                 } else if (rubyCasedName.startsWith("set_")) {
-                    rubyPropertyName = rubyCasedName.substring(4);
-                    if (argCount == 1 && resultType == void.class) {    // setFoo(Foo) => foo=(Foo)
-                        addUnassignedAlias(javaPropertyName + '=', assignedNames, installer);
-                        addUnassignedAlias(rubyPropertyName + '=', assignedNames, installer);
+                    rubyPropertyName = rubyCasedName.substring(4); // TODO do not add foo? for setFoo (returning boolean)
+                    if (argCount == 1 && resultType == void.class) {  // setFoo(Foo) => foo=(Foo)
+                        addUnassignedAlias(javaPropertyName + '=', assignedNames, installer, Priority.ALIAS);
+                        addUnassignedAlias(rubyPropertyName + '=', assignedNames, installer, Priority.ALIAS);
                     }
                 } else if (rubyCasedName.startsWith("is_")) {
                     rubyPropertyName = rubyCasedName.substring(3);
-                    if (resultType == boolean.class) {                  // isFoo() => foo, isFoo(*) => foo(*)
-                        addUnassignedAlias(javaPropertyName, assignedNames, installer);
-                        addUnassignedAlias(rubyPropertyName, assignedNames, installer);
+                    // TODO (9.2) should be another check here to make sure these are only for getters
+                    // ... e.g. isFoo() and not arbitrary isFoo(param) see GH-4432
+                    if (resultType == boolean.class) {  // isFoo() => foo, isFoo(*) => foo(*)
+                        addUnassignedAlias(javaPropertyName, assignedNames, installer, Priority.IS_ALIAS);
+                        addUnassignedAlias(rubyPropertyName, assignedNames, installer, Priority.IS_ALIAS);
+                        // foo? is added bellow
                     }
                 }
             }
 
             // Additionally add ?-postfixed aliases to any boolean methods and properties.
             if (resultType == boolean.class) {
-                // is_something?, contains_thing?
-                addUnassignedAlias(rubyCasedName + '?', assignedNames, installer);
-                if (rubyPropertyName != null) {
-                    // something?
-                    addUnassignedAlias(rubyPropertyName + '?', assignedNames, installer);
+                // isFoo -> isFoo?, contains -> contains?
+                addUnassignedAlias(rubyCasedName + '?', assignedNames, installer, Priority.ALIAS);
+                if (rubyPropertyName != null) { // isFoo -> foo?
+                    addUnassignedAlias(rubyPropertyName + '?', assignedNames, installer, Priority.ALIAS);
                 }
             }
         }
     }
 
-    private static void addUnassignedAlias(String name, Map<String, AssignedName> assignedNames,
-                                           MethodInstaller installer) {
-        if (name == null) return;
+    private static boolean addUnassignedAlias(final String name,
+        final Map<String, AssignedName> assignedNames, final MethodInstaller installer,
+        final Priority aliasType) {
 
         AssignedName assignedName = assignedNames.get(name);
-        // TODO: missing additional logic for dealing with conflicting protected fields.
-        if (Priority.ALIAS.moreImportantThan(assignedName)) {
-            installer.addAlias(name);
-            assignedNames.put(name, new AssignedName(name, Priority.ALIAS));
-        } else if (Priority.ALIAS.asImportantAs(assignedName)) {
+
+        if (aliasType.moreImportantThan(assignedName)) {
             installer.addAlias(name);
+            assignedNames.put(name, new AssignedName(name, aliasType));
+            return true;
         }
+        if (aliasType.asImportantAs(assignedName)) {
+            installer.addAlias(name);
+            return true;
+        }
+
+        // TODO: missing additional logic for dealing with conflicting protected fields.
+
+        return false;
     }
 
     protected static String fixScalaNames(final String name) {
@@ -238,7 +278,7 @@ public abstract class Initializer {
     }
 
     static {
-        HashMap<String, String> scalaOperators = new HashMap<String, String>();
+        HashMap<String, String> scalaOperators = new HashMap<>(24, 1);
         scalaOperators.put("\\$plus", "+");
         scalaOperators.put("\\$minus", "-");
         scalaOperators.put("\\$colon", ":");
@@ -266,47 +306,59 @@ public abstract class Initializer {
             final ClassLoader loader = javaClass.getClassLoader();
             if ( loader == null ) return; //this is a core class, bail
 
+            // scan annotations for "scala" packages; if none present, it's not scala
+            boolean scalaAnno = false;
+            for ( Annotation anno : javaClass.getAnnotations() ) {
+                Package pkg = anno.annotationType().getPackage();
+                if ( pkg != null && pkg.getName() != null && pkg.getName().startsWith("scala.") ) {
+                    scalaAnno = true; break;
+                }
+            }
+            if ( ! scalaAnno ) return;
+
             Class<?> companionClass = loader.loadClass(javaClass.getName() + '$');
             final Field field = companionClass.getField("MODULE$");
             final Object singleton = field.get(null);
             if ( singleton == null ) return;
 
-            final List<Method> scalaMethods = getMethods(companionClass);
-            for ( int j = scalaMethods.size() - 1; j >= 0; j-- ) {
-                final Method method = scalaMethods.get(j);
-                String name = method.getName();
+            final Map<String, List<Method>> scalaMethods = getMethods(companionClass);
+            for (Map.Entry<String, List<Method>> entry : scalaMethods.entrySet()) {
+                final List<Method> methods = entry.getValue();
+                for (int j = 0; j < methods.size(); j++) {
+                    final Method method = methods.get(j);
+                    String name = method.getName();
 
-                if (DEBUG_SCALA) LOG.debug("Companion object method {} for {}", name, companionClass);
+                    if (DEBUG_SCALA) LOG.debug("Companion object method {} for {}", name, companionClass);
 
-                if ( name.indexOf('$') >= 0 ) name = fixScalaNames(name);
+                    if (name.indexOf('$') >= 0) name = fixScalaNames(name);
 
-                if ( ! Modifier.isStatic(method.getModifiers()) ) {
-                    AssignedName assignedName = state.staticNames.get(name);
-                    // For JRUBY-4505, restore __method methods for reserved names
-                    if (INSTANCE_RESERVED_NAMES.containsKey(method.getName())) {
-                        if (DEBUG_SCALA) LOG.debug("in reserved " + name);
-                        setupSingletonMethods(state.staticInstallers, javaClass, singleton, method, name + METHOD_MANGLE);
-                        continue;
-                    }
-                    if (assignedName == null) {
-                        state.staticNames.put(name, new AssignedName(name, Priority.METHOD));
-                        if (DEBUG_SCALA) LOG.debug("Assigned name is null");
-                    } else {
-                        if (Priority.METHOD.lessImportantThan(assignedName)) {
-                            if (DEBUG_SCALA) LOG.debug("Less important");
+                    if (!Modifier.isStatic(method.getModifiers())) {
+                        AssignedName assignedName = state.staticNames.get(name);
+                        // For JRUBY-4505, restore __method methods for reserved names
+                        if (INSTANCE_RESERVED_NAMES.containsKey(method.getName())) {
+                            if (DEBUG_SCALA) LOG.debug("in reserved " + name);
+                            setupSingletonMethods(state.staticInstallers, javaClass, singleton, method, name + METHOD_MANGLE);
                             continue;
                         }
-                        if (!Priority.METHOD.asImportantAs(assignedName)) {
-                            state.staticInstallers.remove(name);
-                            state.staticInstallers.remove(name + '=');
+                        if (assignedName == null) {
                             state.staticNames.put(name, new AssignedName(name, Priority.METHOD));
+                            if (DEBUG_SCALA) LOG.debug("Assigned name is null");
+                        } else {
+                            if (Priority.METHOD.lessImportantThan(assignedName)) {
+                                if (DEBUG_SCALA) LOG.debug("Less important");
+                                continue;
+                            }
+                            if (!Priority.METHOD.asImportantAs(assignedName)) {
+                                state.staticInstallers.remove(name);
+                                state.staticInstallers.remove(name + '=');
+                                state.staticNames.put(name, new AssignedName(name, Priority.METHOD));
+                            }
                         }
+                        if (DEBUG_SCALA) LOG.debug("Installing {} {} {}", name, method, singleton);
+                        setupSingletonMethods(state.staticInstallers, javaClass, singleton, method, name);
+                    } else {
+                        if (DEBUG_SCALA) LOG.debug("Method {} is sadly static", method);
                     }
-                    if (DEBUG_SCALA) LOG.debug("Installing {} {} {}", name, method, singleton);
-                    setupSingletonMethods(state.staticInstallers, javaClass, singleton, method, name);
-                }
-                else {
-                    if (DEBUG_SCALA) LOG.debug("Method {} is sadly static", method);
                 }
             }
         }
@@ -384,46 +436,23 @@ public abstract class Initializer {
 
     public abstract RubyModule initialize(RubyModule proxy);
 
-    public void initializeBase(RubyModule proxy) {
-        proxy.addMethod("__jsend!", new org.jruby.internal.runtime.methods.JavaMethod.JavaMethodNBlock(proxy, PUBLIC) {
-            @Override
-            public IRubyObject call(ThreadContext context, IRubyObject self, RubyModule clazz, String name, IRubyObject[] args, Block block) {
-                String callName = args[0].asJavaString();
-
-                DynamicMethod method = self.getMetaClass().searchMethod(callName);
-                int v = method.getArity().getValue();
-
-                IRubyObject[] newArgs = new IRubyObject[args.length - 1];
-                System.arraycopy(args, 1, newArgs, 0, newArgs.length);
-
-                context.runtime.getWarnings().warn("#__jsend! is deprecated (and will no longer work in 9K); use #java_send instead");
-
-                if (v < 0 || v == (newArgs.length)) {
-                    return Helpers.invoke(context, self, callName, newArgs, Block.NULL_BLOCK);
-                }
-                RubyClass superClass = self.getMetaClass().getSuperClass();
-                return Helpers.invokeAs(context, superClass, self, callName, newArgs, Block.NULL_BLOCK);
-            }
-        });
-    }
-
     public static class State {
 
         final Map<String, AssignedName> staticNames;
         final Map<String, AssignedName> instanceNames;
-        final Map<String, NamedInstaller> staticInstallers = new HashMap<String, NamedInstaller>();
-        final Map<String, NamedInstaller> instanceInstallers = new HashMap<String, NamedInstaller>();
-        final List<ConstantField> constantFields = new ArrayList<ConstantField>();
+        final Map<String, NamedInstaller> staticInstallers = new HashMap<>();
+        final Map<String, NamedInstaller> instanceInstallers = new HashMap<>();
+        final List<ConstantField> constantFields = new ArrayList<>();
 
         ConstructorInvokerInstaller constructorInstaller;
 
         State(final Ruby runtime, final Class superClass) {
             if (superClass == null) {
-                staticNames = new HashMap<String, AssignedName>();
-                instanceNames = new HashMap<String, AssignedName>();
+                staticNames = new HashMap<>(8);
+                instanceNames = new HashMap<>(26);
             } else {
-                staticNames = new HashMap<String, AssignedName>(runtime.getJavaSupport().getStaticAssignedNames().get(superClass));
-                instanceNames = new HashMap<String, AssignedName>(runtime.getJavaSupport().getInstanceAssignedNames().get(superClass));
+                staticNames = new HashMap<>(runtime.getJavaSupport().getStaticAssignedNames().get(superClass));
+                instanceNames = new HashMap<>(runtime.getJavaSupport().getInstanceAssignedNames().get(superClass));
             }
             staticNames.putAll(STATIC_RESERVED_NAMES);
             instanceNames.putAll(INSTANCE_RESERVED_NAMES);
@@ -431,11 +460,29 @@ public abstract class Initializer {
 
     }
 
-    static List<Method> getMethods(final Class<?> javaClass) {
-        HashMap<String, List<Method>> nameMethods = new HashMap<String, List<Method>>(32);
+    public static final java.lang.ClassValue<Method[]> DECLARED_METHODS = new java.lang.ClassValue<Method[]>() {
+        @Override
+        public Method[] computeValue(Class cls) {
+            return cls.getDeclaredMethods();
+        }
+    };
 
-        // to better size the final ArrayList below
-        int totalMethods = 0;
+    public static final java.lang.ClassValue<Method[]> METHODS = new java.lang.ClassValue<Method[]>() {
+        @Override
+        public Method[] computeValue(Class cls) {
+            return cls.getMethods();
+        }
+    };
+
+    public static final java.lang.ClassValue<Class<?>[]> INTERFACES = new java.lang.ClassValue<Class<?>[]>() {
+        @Override
+        public Class<?>[] computeValue(Class cls) {
+            return cls.getInterfaces();
+        }
+    };
+
+    static Map<String, List<Method>> getMethods(final Class<?> javaClass) {
+        HashMap<String, List<Method>> nameMethods = new HashMap<>(32);
 
         // we scan all superclasses, but avoid adding superclass methods with
         // same name+signature as subclass methods (see JRUBY-3130)
@@ -447,40 +494,35 @@ public abstract class Initializer {
                 try {
                     // add methods, including static if this is the actual class,
                     // and replacing child methods with equivalent parent methods
-                    totalMethods += addNewMethods(nameMethods, klass.getDeclaredMethods(), klass == javaClass, true);
+                    addNewMethods(nameMethods, DECLARED_METHODS.get(klass), klass == javaClass, true);
                 }
                 catch (SecurityException e) { /* ignored */ }
             }
 
             // then do the same for each interface
-            for ( Class iface : klass.getInterfaces() ) {
+            for ( Class iface : INTERFACES.get(klass) ) {
                 try {
                     // add methods, not including static (should be none on
                     // interfaces anyway) and not replacing child methods with
                     // parent methods
-                    totalMethods += addNewMethods(nameMethods, iface.getMethods(), false, false);
+                    addNewMethods(nameMethods, METHODS.get(iface), false, false);
                 }
                 catch (SecurityException e) { /* ignored */ }
             }
         }
 
-        // now only bind the ones that remain
-        ArrayList<Method> finalList = new ArrayList<Method>(totalMethods);
-
-        for ( Map.Entry<String, List<Method>> entry : nameMethods.entrySet() ) {
-            finalList.addAll( entry.getValue() );
-        }
-
-        return finalList;
+        return nameMethods;
     }
 
     private static boolean methodsAreEquivalent(Method child, Method parent) {
+        int childModifiers, parentModifiers;
+
         return parent.getDeclaringClass().isAssignableFrom(child.getDeclaringClass())
                 && child.getReturnType() == parent.getReturnType()
                 && child.isVarArgs() == parent.isVarArgs()
-                && Modifier.isPublic(child.getModifiers()) == Modifier.isPublic(parent.getModifiers())
-                && Modifier.isProtected(child.getModifiers()) == Modifier.isProtected(parent.getModifiers())
-                && Modifier.isStatic(child.getModifiers()) == Modifier.isStatic(parent.getModifiers())
+                && Modifier.isPublic(childModifiers = child.getModifiers()) == Modifier.isPublic(parentModifiers = parent.getModifiers())
+                && Modifier.isProtected(childModifiers) == Modifier.isProtected(parentModifiers)
+                && Modifier.isStatic(childModifiers) == Modifier.isStatic(parentModifiers)
                 && Arrays.equals(child.getParameterTypes(), parent.getParameterTypes());
     }
 
@@ -511,22 +553,21 @@ public abstract class Initializer {
             List<Method> childMethods = nameMethods.get(method.getName());
             if (childMethods == null) {
                 // first method of this name, add a collection for it
-                childMethods = new ArrayList<Method>(4);
+                childMethods = new ArrayList<>(4);
                 childMethods.add(method); added++;
                 nameMethods.put(method.getName(), childMethods);
             }
             else {
                 // we have seen other methods; check if we already have an equivalent one
-                final ListIterator<Method> childMethodsIterator = childMethods.listIterator();
-                while ( childMethodsIterator.hasNext() ) {
-                    final Method current = childMethodsIterator.next();
+                for (int i = 0; i < childMethods.size(); i++) {
+                    final Method current = childMethods.get(i);
                     if ( methodsAreEquivalent(current, method) ) {
                         if (removeDuplicate) {
                             // Replace the existing method, since the super call is more general
                             // and virtual dispatch will call the subclass impl anyway.
                             // Used for instance methods, for which we usually want to use the highest-up
                             // callable implementation.
-                            childMethodsIterator.set(method);
+                            childMethods.set(i, method);
                         } else {
                             // just skip the new method, since we don't need it (already found one)
                             // used for interface methods, which we want to add unconditionally
