@@ -2,7 +2,6 @@ package org.jruby.javasupport.binding;
 
 import org.jruby.RubyClass;
 import org.jruby.RubyModule;
-import org.jruby.internal.runtime.methods.DynamicMethod;
 import org.jruby.java.invokers.SingletonMethodInvoker;
 
 /**
@@ -21,11 +20,6 @@ public class SingletonMethodInvokerInstaller extends StaticMethodInvokerInstalle
         // we don't check haveLocalMethod() here because it's not local and we know
         // that we always want to go ahead and install it
         final RubyClass singletonClass = proxy.getSingletonClass();
-        DynamicMethod method = new SingletonMethodInvoker(this.singleton, singletonClass, methods);
-        singletonClass.addMethod(name, method);
-        if ( aliases != null && isPublic() ) {
-            singletonClass.defineAliases(aliases, name);
-            //aliases = null;
-        }
+        defineMethods(singletonClass, new SingletonMethodInvoker(this.singleton, singletonClass, methods), false);
     }
 }
