@@ -37,10 +37,13 @@ import java.lang.reflect.Method;
 import java.security.AccessController;
 import java.security.PrivilegedActionException;
 import java.security.PrivilegedExceptionAction;
+import java.util.Collection;
 import java.util.ArrayList;
 import java.util.Arrays;
+import java.util.Collections;
 import java.util.HashMap;
 import java.util.HashSet;
+import java.util.LinkedHashSet;
 import java.util.List;
 import java.util.Map;
 import java.util.Set;
@@ -57,11 +60,10 @@ import org.jruby.anno.JRubyClass;
 import org.jruby.anno.JRubyMethod;
 import org.jruby.exceptions.RaiseException;
 import org.jruby.internal.runtime.methods.DynamicMethod;
-import org.jruby.javasupport.JavaClass;
-import org.jruby.javasupport.JavaObject;
-import org.jruby.javasupport.JavaUtil;
+import org.jruby.javasupport.*;
 import org.jruby.runtime.ObjectAllocator;
 import org.jruby.runtime.builtin.IRubyObject;
+import org.jruby.util.ClassDefiningClassLoader;
 
 import static org.jruby.javasupport.JavaClass.EMPTY_CLASS_ARRAY;
 import static org.jruby.javasupport.JavaCallable.inspectParameterTypes;
@@ -83,15 +85,21 @@ import static org.jruby.javasupport.JavaCallable.inspectParameterTypes;
  *
  */
 public class JavaProxyClass extends JavaProxyReflectionObject {
-    static ThreadLocal<Ruby> runtimeTLS = new ThreadLocal<Ruby>();
+
+    @Deprecated // moved to JavaProxyClassFactory
+    static ThreadLocal<Ruby> runtimeTLS = JavaProxyClassFactory.runtimeTLS;
 
     private final Class proxyClass;
-    private final ArrayList<JavaProxyMethod> methods = new ArrayList<JavaProxyMethod>();
-    private final HashMap<String, ArrayList<JavaProxyMethod>> methodMap = new HashMap<String, ArrayList<JavaProxyMethod>>();
+    private final ArrayList<JavaProxyMethod> methods = new ArrayList<>();
+    private final HashMap<String, ArrayList<JavaProxyMethod>> methodMap = new HashMap<>();
 
     /* package scope */
     JavaProxyClass(final Class<?> proxyClass) {
-        super(getThreadLocalRuntime(), getThreadLocalRuntime().getModule("Java").getClass("JavaProxyClass"));
+        this(getThreadLocalRuntime(), proxyClass);
+    }
+
+    private JavaProxyClass(final Ruby runtime, final Class<?> proxyClass) {
+        super(runtime, runtime.getModule("Java").getClass("JavaProxyClass"));
         this.proxyClass = proxyClass;
     }
 
@@ -114,22 +122,47 @@ public class JavaProxyClass extends JavaProxyReflectionObject {
         return runtimeTLS.get();
     }
 
-    public static JavaProxyClass getProxyClass(Ruby runtime, Class superClass,
-            Class[] interfaces, Set<String> names) throws InvocationTargetException {
-        Ruby save = runtimeTLS.get();
-        runtimeTLS.set(runtime);
-        try {
-            ClassLoader loader = runtime.getJRubyClassLoader();
-
-            return runtime.getJavaProxyClassFactory().newProxyClass(runtime, loader, null, superClass, interfaces, names);
-        } finally {
-            runtimeTLS.set(save);
-        }
+    @Deprecated // renamed to newProxyClass
+    public static JavaProxyClass getProxyClass(final Ruby runtime, Class superClass,
+        Class[] interfaces, Set<String> names) throws InvocationTargetException {
+        return newProxyClass(runtime, superClass, interfaces, names);
     }
 
+    @Deprecated // renamed to newProxyClass
     public static JavaProxyClass getProxyClass(Ruby runtime, Class superClass,
+                                               Class[] interfaces) throws InvocationTargetException {
+        return newProxyClass(runtime, superClass, interfaces, null);
+    }
+
+    /**
+     * Returns a new (generated) proxy class based on arguments.
+     * @param runtime
+     * @param superClass
+     * @param interfaces
+     * @param names
+     * @return proxy class
+     * @throws InvocationTargetException
+     */
+    public static JavaProxyClass newProxyClass(final Ruby runtime, Class superClass,
+        Class[] interfaces, Set<String> names) throws InvocationTargetException {
+
+        if (superClass == null) superClass = Object.class;
+        if (interfaces == null) interfaces = EMPTY_CLASS_ARRAY;
+        if (names == null) names = Collections.EMPTY_SET; // so we can assume names != null
+
+        // NOTE: currently we regenerate proxy classes when a Ruby method is added on the type
+        JavaSupport.ProxyClassKey classKey = JavaSupport.ProxyClassKey.getInstance(superClass, interfaces, names);
+        JavaProxyClass proxyClass = JavaSupportImpl.fetchJavaProxyClass(runtime, classKey);
+        if ( proxyClass != null ) return proxyClass;
+
+        final ClassLoader loader = runtime.getJRubyClassLoader();
+        proxyClass = runtime.getJavaProxyClassFactory().genProxyClass(runtime, (ClassDefiningClassLoader) loader, null, superClass, interfaces, names);
+        return JavaSupportImpl.saveJavaProxyClass(runtime, classKey, proxyClass);
+    }
+
+    public static JavaProxyClass newProxyClass(Ruby runtime, Class superClass,
             Class[] interfaces) throws InvocationTargetException {
-        return getProxyClass(runtime, superClass, interfaces, null);
+        return newProxyClass(runtime, superClass, interfaces, null);
     }
 
     public static Object newProxyInstance(Ruby runtime, Class superClass, Class[] interfaces,
@@ -138,7 +171,7 @@ public class JavaProxyClass extends JavaProxyReflectionObject {
             InstantiationException, IllegalAccessException, InvocationTargetException,
             SecurityException, NoSuchMethodException {
 
-        JavaProxyClass proxyClass = getProxyClass(runtime, superClass, interfaces);
+        JavaProxyClass proxyClass = newProxyClass(runtime, superClass, interfaces);
         JavaProxyConstructor constructor = proxyClass.getConstructor(
             constructorParameters == null ? EMPTY_CLASS_ARRAY : constructorParameters
         );
@@ -201,11 +234,8 @@ public class JavaProxyClass extends JavaProxyReflectionObject {
         return null;
     }
 
-    /** return the class of instances of this proxy class */
-    final Class getProxyClass() { return proxyClass; }
-
     @Override
-    public Class getJavaClass() {
+    public final Class getJavaClass() {
         return proxyClass;
     }
 
@@ -308,6 +338,7 @@ public class JavaProxyClass extends JavaProxyReflectionObject {
             return method.toString();
         }
 
+        @Deprecated
         public Object defaultResult() {
             final Class returnType = method.getReturnType();
 
@@ -355,11 +386,6 @@ public class JavaProxyClass extends JavaProxyReflectionObject {
             return getRuntime().newFixnum(getArity());
         }
 
-        @Deprecated
-        protected String nameOnInspection() {
-            return getDeclaringClass().nameOnInspection() + "/" + getName();
-        }
-
         @Override
         @JRubyMethod
         public RubyString inspect() {
@@ -367,8 +393,8 @@ public class JavaProxyClass extends JavaProxyReflectionObject {
             str.append("#<");
             str.append( getDeclaringClass().nameOnInspection() ).append('/').append( getName() );
             inspectParameterTypes(str, this);
-            str.append(">");
-            return getRuntime().newString( str.toString() );
+            str.append('>');
+            return RubyString.newString(getRuntime(), str);
         }
 
         @JRubyMethod(name = "invoke", rest = true)
@@ -433,7 +459,7 @@ public class JavaProxyClass extends JavaProxyReflectionObject {
 
             ArrayList<JavaProxyMethod> methodsWithName = this.methodMap.get(name);
             if (methodsWithName == null) {
-                methodsWithName = new ArrayList<JavaProxyMethod>(2);
+                methodsWithName = new ArrayList<>(2);
                 methodMap.put(name, methodsWithName);
             }
             methodsWithName.add(proxyMethod);
@@ -452,47 +478,46 @@ public class JavaProxyClass extends JavaProxyReflectionObject {
     }
 
     private static Class[] parse(final ClassLoader loader, String desc) throws ClassNotFoundException {
-        final List<Class> types = new ArrayList<Class>();
+        final ArrayList<Class> types = new ArrayList<>(8);
         int idx = 1;
         while (desc.charAt(idx) != ')') {
 
             int arr = 0;
             while (desc.charAt(idx) == '[') {
-                idx += 1;
-                arr += 1;
+                idx++; arr += 1;
             }
 
             Class type;
 
             switch (desc.charAt(idx)) {
-            case 'L':
-                int semi = desc.indexOf(';', idx);
-                final String name = desc.substring(idx + 1, semi);
-                idx = semi;
-                try {
-                    type = AccessController.doPrivileged(new PrivilegedExceptionAction<Class>() {
-                        public Class run() throws ClassNotFoundException {
-                            return Class.forName(name.replace('/', '.'), false, loader);
-                        }
-                    });
-                } catch (PrivilegedActionException e) {
-                    throw (ClassNotFoundException) e.getException();
-                }
-                break;
+                case 'L':
+                    int semi = desc.indexOf(';', idx);
+                    final String name = desc.substring(idx + 1, semi);
+                    idx = semi;
+                    try {
+                        type = AccessController.doPrivileged(new PrivilegedExceptionAction<Class>() {
+                            public Class run() throws ClassNotFoundException {
+                                return Class.forName(name.replace('/', '.'), false, loader);
+                            }
+                        });
+                    } catch (PrivilegedActionException e) {
+                        throw (ClassNotFoundException) e.getException();
+                    }
+                    break;
 
-            case 'B': type = Byte.TYPE; break;
-            case 'C': type = Character.TYPE; break;
-            case 'Z': type = Boolean.TYPE; break;
-            case 'S': type = Short.TYPE; break;
-            case 'I': type = Integer.TYPE; break;
-            case 'J': type = Long.TYPE; break;
-            case 'F': type = Float.TYPE; break;
-            case 'D': type = Double.TYPE; break;
-            default:
-                throw new InternalError("cannot parse " + desc + "[" + idx + "]");
+                case 'B': type = Byte.TYPE; break;
+                case 'C': type = Character.TYPE; break;
+                case 'Z': type = Boolean.TYPE; break;
+                case 'S': type = Short.TYPE; break;
+                case 'I': type = Integer.TYPE; break;
+                case 'J': type = Long.TYPE; break;
+                case 'F': type = Float.TYPE; break;
+                case 'D': type = Double.TYPE; break;
+                default:
+                    throw new InternalError("cannot parse " + desc + '[' + idx + ']');
             }
 
-            idx += 1;
+            idx++;
 
             if (arr != 0) {
                 type = Array.newInstance(type, new int[arr]).getClass();
@@ -501,18 +526,13 @@ public class JavaProxyClass extends JavaProxyReflectionObject {
             types.add(type);
         }
 
-        return types.toArray(new Class[types.size()]);
+        return types.isEmpty() ? EMPTY_CLASS_ARRAY : types.toArray(new Class[types.size()]);
     }
 
     //
     // Ruby-level methods
     //
 
-    @Deprecated
-    public static void createJavaProxyModule(Ruby runtime) {
-        createJavaProxyClasses(runtime, runtime.getJavaSupport().getJavaModule());
-    }
-
     public static void createJavaProxyClasses(final Ruby runtime, final RubyModule Java) {
         JavaProxyClass.createJavaProxyClassClass(runtime, Java);
         ProxyMethodImpl.createJavaProxyMethodClass(runtime, Java);
@@ -539,19 +559,14 @@ public class JavaProxyClass extends JavaProxyReflectionObject {
         try {
             return getProxyClass(runtime, type.javaClass(), EMPTY_CLASS_ARRAY);
         }
-        catch (Error e) {
-            RaiseException ex = runtime.newArgumentError("unable to create proxy class for " + type.getValue());
-            ex.initCause(e);
-            throw ex;
-        }
-        catch (InvocationTargetException e) {
+        catch (InvocationTargetException|Error e) {
             RaiseException ex = runtime.newArgumentError("unable to create proxy class for " + type.getValue());
             ex.initCause(e);
             throw ex;
         }
     }
 
-    private static final HashSet<String> EXCLUDE_MODULES = new HashSet<String>();
+    private static final HashSet<String> EXCLUDE_MODULES = new HashSet<>(8, 1);
     static {
         EXCLUDE_MODULES.add("Kernel");
         EXCLUDE_MODULES.add("Java");
@@ -559,42 +574,43 @@ public class JavaProxyClass extends JavaProxyReflectionObject {
         EXCLUDE_MODULES.add("Enumerable");
     }
 
-    private static final HashSet<String> EXCLUDE_METHODS = new HashSet<String>();
-    static {
-        EXCLUDE_METHODS.add("class");
-        EXCLUDE_METHODS.add("finalize");
-        EXCLUDE_METHODS.add("initialize");
-        EXCLUDE_METHODS.add("java_class");
-        EXCLUDE_METHODS.add("java_object");
-        EXCLUDE_METHODS.add("__jcreate!");
-        EXCLUDE_METHODS.add("__jsend!");
+    private static boolean isExcludedMethod(final String name) {
+        switch (name) {
+            case "class" : return true;
+            case "finalize" : return true;
+            case "initialize" : return true;
+            case "java_class" : return true;
+            case "java_object" : return true;
+            case "__jcreate!" : return true;
+            default: return false;
+        }
     }
 
     @JRubyMethod(meta = true)
-    public static RubyObject get_with_class(IRubyObject recv, IRubyObject obj) {
-        final Ruby runtime = recv.getRuntime();
+    public static RubyObject get_with_class(final IRubyObject self, IRubyObject obj) {
+        final Ruby runtime = self.getRuntime();
 
         if (!(obj instanceof RubyClass)) {
             throw runtime.newTypeError(obj, runtime.getClassClass());
         }
 
-        RubyClass clazz = (RubyClass)obj;
+        return getProxyClass(runtime, (RubyClass) obj);
+    }
 
-        // Let's only generate methods for those the user may actually
-        // intend to override.  That includes any defined in the current
-        // class, and any ancestors that are also JavaProxyClasses (but none
-        // from any other ancestor classes). Methods defined in mixins will
-        // be considered intentionally overridden, except those from Kernel,
-        // Java, and JavaProxyMethods, as well as Enumerable.
+    public static JavaProxyClass getProxyClass(final Ruby runtime, final RubyClass clazz) {
+
+        // Let's only generate methods for those the user may actually intend to override.
+        // That includes any defined in the current class, and any ancestors that are also JavaProxyClasses
+        // (but none from any other ancestor classes). Methods defined in mixins will be considered
+        // intentionally overridden, except those from Kernel, Java, and JavaProxyMethods, as well as Enumerable.
         // TODO: may want to exclude other common mixins?
 
         JavaClass javaClass = null;
-        Set<String> names = new HashSet<String>(); // need names ordered for key generation later
-        List<Class<?>> interfaceList = new ArrayList<Class<?>>();
+        HashSet<String> names = new HashSet<>(); // need names ordered for key generation later
+        Collection<Class<?>> interfaceList = new LinkedHashSet<>(8);
 
-        List<IRubyObject> ancestors = clazz.getAncestorList();
         boolean skipRemainingClasses = false;
-        for (IRubyObject ancestorObject: ancestors) {
+        for ( IRubyObject ancestorObject: clazz.getAncestorList() ) {
             RubyModule ancestor = (RubyModule) ancestorObject;
             if (ancestor instanceof RubyClass) {
                 if (skipRemainingClasses) continue;
@@ -608,48 +624,32 @@ public class JavaProxyClass extends JavaProxyReflectionObject {
                 // get JavaClass if this is the new proxy class; verify it
                 // matches if this is a superclass proxy.
                 IRubyObject var = ancestor.getInstanceVariables().getInstanceVariable("@java_class");
-                if (var == null) {
-                    throw runtime.newTypeError(
-                            "no java_class defined for proxy (or ancestor): " + ancestor);
-                } else if (!(var instanceof JavaClass)) {
-                    throw runtime.newTypeError(
-                            "invalid java_class defined for proxy (or ancestor): " +
-                            ancestor + ": " + var);
+                if ( ! (var instanceof JavaClass) ) {
+                    if (var == null) {
+                        throw runtime.newTypeError("no java_class defined for proxy (or ancestor): " + ancestor);
+                    }
+                    throw runtime.newTypeError("invalid java_class defined for proxy (or ancestor): " + ancestor + ": " + var);
                 }
-                if (javaClass == null) {
-                    javaClass = (JavaClass)var;
-                } else if (javaClass != var) {
-                    throw runtime.newTypeError(
-                            "java_class defined for " + clazz + " (" + javaClass +
-                            ") does not match java_class for ancestor " + ancestor +
-                            " (" + var + ")");
+                if (javaClass == null) javaClass = (JavaClass) var;
+                else if (javaClass != var) {
+                    throw runtime.newTypeError("java_class defined for " + clazz + " (" + javaClass +
+                            ") does not match java_class for ancestor " + ancestor + " (" + var + ")");
                 }
                 // get any included interfaces
                 var = ancestor.getInstanceVariables().getInstanceVariable("@java_interfaces");
                 if (var != null && !(var instanceof RubyNil)) {
                     if (!(var instanceof RubyArray)) {
-                        throw runtime.newTypeError(
-                                "invalid java_interfaces defined for proxy (or ancestor): " +
-                                ancestor + ": " + var);
+                        throw runtime.newTypeError("invalid java_interfaces defined for proxy (or ancestor): " + ancestor + ": " + var);
                     }
-                    RubyArray ifcArray = (RubyArray)var;
-                    int size = ifcArray.size();
-                    for (int i = size; --i >= 0; ) {
-                        IRubyObject ifc = ifcArray.eltInternal(i);
-                        if (!(ifc instanceof JavaClass)) {
-                            throw runtime.newTypeError(
-                                "invalid java interface defined for proxy (or ancestor): " +
-                                ancestor + ": " + ifc);
-                        }
-                        Class interfaceClass = ((JavaClass)ifc).javaClass();
+                    RubyArray interfaces = (RubyArray) var;
+                    for (int i = interfaces.size(); --i >= 0; ) {
+                        IRubyObject iface = interfaces.eltInternal(i);
+                        Class interfaceClass = ((JavaClass) iface).javaClass();
                         if (!interfaceClass.isInterface()) {
-                            throw runtime.newTypeError(
-                                    "invalid java interface defined for proxy (or ancestor): " +
-                                    ancestor + ": " + ifc + " (not an interface)");
-                        }
-                        if (!interfaceList.contains(interfaceClass)) {
-                            interfaceList.add(interfaceClass);
+                            throw runtime.newTypeError("invalid java interface defined for proxy (or ancestor): " +
+                                    ancestor + ": " + iface + " (not an interface)");
                         }
+                        interfaceList.add(interfaceClass);
                     }
                 }
                 // set this class's method names in var @__java_ovrd_methods if this
@@ -658,45 +658,31 @@ public class JavaProxyClass extends JavaProxyReflectionObject {
 
                 // FIXME: shouldn't need @__java_ovrd_methods, just query locally defined methods.
 
-                var = ancestor.getInstanceVariables().getInstanceVariable("@__java_ovrd_methods");
-                if (var == null) {
-                    // lock in the overridden methods for the new class, and any as-yet
-                    // uninstantiated ancestor class.
+                Collection<String> methodNames = (Collection<String>) ancestor.getInternalVariable("__java_ovrd_methods");
+
+                if (methodNames == null) {
+                    // lock in the overridden methods for the new class, and any as-yet uninstantiated ancestor class.
                     Map<String, DynamicMethod> methods;
-                    RubyArray methodNames;
                     synchronized(methods = ancestor.getMethods()) {
-                        methodNames = RubyArray.newArrayLight(runtime,methods.size());
+                        methodNames = new ArrayList<>(methods.size());
                         for (String methodName: methods.keySet()) {
-                            if (!EXCLUDE_METHODS.contains(methodName)) {
+                            if ( ! isExcludedMethod(methodName) ) {
                                 names.add(methodName);
-                                methodNames.append(runtime.newString(methodName));
+                                methodNames.add(methodName);
                             }
                         }
                     }
-                    ancestor.setInstanceVariable("@__java_ovrd_methods",methodNames);
-                } else {
-                    if (!(var instanceof RubyArray)) {
-                        throw runtime.newTypeError(
-                                "invalid @__java_ovrd_methods defined for proxy: " +
-                                ancestor + ": " + var);
-                    }
-                    RubyArray methodNames = (RubyArray)var;
-                    int size = methodNames.size();
-                    for (int i = size; --i >= 0; ) {
-                        IRubyObject methodName = methodNames.eltInternal(i);
-                        if (!(methodName instanceof RubyString)) {
-                            throw runtime.newTypeError(
-                                    "invalid method name defined for proxy (or ancestor): " +
-                                    ancestor + ": " + methodName);
-                        }
-                        names.add(methodName.asJavaString());
-                    }
+                    ancestor.setInternalVariable("__java_ovrd_methods", methodNames);
                 }
-            } else if (!EXCLUDE_MODULES.contains(ancestor.getName())) {
+                else {
+                    names.addAll(methodNames);
+                }
+            }
+            else if (!EXCLUDE_MODULES.contains(ancestor.getName())) {
                 Map<String, DynamicMethod> methods;
                 synchronized(methods = ancestor.getMethods()) {
                     for (String methodName: methods.keySet()) {
-                        if (!EXCLUDE_METHODS.contains(methodName)) {
+                        if ( ! isExcludedMethod(methodName) ) {
                             names.add(methodName);
                         }
                     }
@@ -704,28 +690,20 @@ public class JavaProxyClass extends JavaProxyReflectionObject {
             }
         }
 
-        if (javaClass == null) {
-            throw runtime.newArgumentError("unable to create proxy class: no java_class defined for " + clazz);
-        }
+        if (javaClass == null) throw runtime.newArgumentError("unable to create proxy class: no java_class defined for " + clazz);
 
-        int interfaceCount = interfaceList.size();
-        Class<?>[] interfaces = new Class<?>[interfaceCount];
-        for (int i = interfaceCount; --i >= 0; ) {
-            interfaces[i] = interfaceList.get(i);
-        }
+        Class<?>[] interfaces = interfaceList.isEmpty() ? EMPTY_CLASS_ARRAY : interfaceList.toArray(new Class<?>[interfaceList.size()]);
 
         try {
-            return getProxyClass(runtime, javaClass.javaClass(), interfaces, names);
+            return newProxyClass(runtime, javaClass.javaClass(), interfaces, names);
         }
-        catch (Error e) {
-            RaiseException ex = runtime.newArgumentError("unable to create proxy class for " + javaClass.getValue() + " : " + e.getMessage());
-            //e.printStackTrace();
-            ex.initCause(e);
-            throw ex;
+        catch (RaiseException e) {
+            throw e;
         }
-        catch (InvocationTargetException e) {
-            RaiseException ex = runtime.newArgumentError("unable to create proxy class for " + javaClass.getValue() + " : " + e.getMessage());
-            //e.printStackTrace();
+        catch (Exception e) {
+            String msg = e.getLocalizedMessage();
+            if ( msg == null ) msg = e.toString();
+            RaiseException ex = runtime.newArgumentError("unable to create proxy class for " + javaClass + " : " + msg);
             ex.initCause(e);
             throw ex;
         }
@@ -752,6 +730,6 @@ public class JavaProxyClass extends JavaProxyReflectionObject {
     }
 
     public final String nameOnInspection() {
-        return "[Proxy:" + getSuperclass().getName() + "]";
+        return "[Proxy:" + getSuperclass().getName() + ']';
     }
 }
