@@ -2,7 +2,9 @@ package org.jruby.runtime.backtrace;
 
 import org.jruby.lexer.yacc.ISourcePosition;
 
-public class BacktraceElement {
+public final class BacktraceElement implements Cloneable {
+
+    public static final BacktraceElement[] EMPTY_ARRAY = new BacktraceElement[0];
 
     public BacktraceElement() {
     }
@@ -15,7 +17,7 @@ public class BacktraceElement {
 
     @Override
     public String toString() {
-        return method + " at " + filename + ":" + line;
+        return method + " at " + filename + ':' + line;
     }
 
     @Override
@@ -23,6 +25,7 @@ public class BacktraceElement {
         return new BacktraceElement(method, filename, line);
     }
 
+    @Deprecated
     public static void update(BacktraceElement backtrace, String method, ISourcePosition position) {
         backtrace.method = method;
         backtrace.filename = position.getFile();
