@@ -3,15 +3,13 @@ package org.jruby.runtime.encoding;
 import org.jcodings.Encoding;
 import org.jcodings.EncodingDB;
 import org.jcodings.EncodingDB.Entry;
-import org.jcodings.ascii.AsciiTables;
 import org.jcodings.specific.ASCIIEncoding;
 import org.jcodings.specific.ISO8859_16Encoding;
 import org.jcodings.util.CaseInsensitiveBytesHash;
 import org.jcodings.util.Hash.HashEntryIterator;
 import org.jruby.Ruby;
-import org.jruby.RubyConverter;
 import org.jruby.RubyEncoding;
-import org.jruby.exceptions.MainExitException;
+import org.jruby.exceptions.RaiseException;
 import org.jruby.platform.Platform;
 import org.jruby.runtime.builtin.IRubyObject;
 import org.jruby.util.ByteList;
@@ -20,12 +18,14 @@ import java.io.Console;
 import java.lang.reflect.Field;
 import java.nio.charset.Charset;
 import java.nio.charset.UnsupportedCharsetException;
-import java.util.Iterator;
+
 import org.jcodings.specific.USASCIIEncoding;
 import org.jruby.RubyFixnum;
 import org.jruby.RubyString;
 import org.jruby.ext.nkf.RubyNKF;
+import org.jruby.util.SafePropertyAccessor;
 import org.jruby.util.encoding.ISO_8859_16;
+import org.jruby.util.io.EncodingUtils;
 
 public final class EncodingService {
     private final CaseInsensitiveBytesHash<Entry> encodings;
@@ -37,7 +37,7 @@ public final class EncodingService {
     private RubyEncoding[] encodingIndex = new RubyEncoding[4];
     // the runtime
     private final Ruby runtime;
-    
+
     private final Encoding ascii8bit;
     private final Encoding javaDefault;
 
@@ -85,7 +85,7 @@ public final class EncodingService {
         }
         return consoleEncoding;
     }
-    
+
     // mri: rb_usascii_encoding
     public Encoding getUSAsciiEncoding() {
         return USASCIIEncoding.INSTANCE;
@@ -95,12 +95,12 @@ public final class EncodingService {
     public Encoding getAscii8bitEncoding() {
         return ascii8bit;
     }
-    
+
     // mri: rb_filesystem_encoding
-    public Encoding getFileSystemEncoding(Ruby runtime) {
+    public Encoding getFileSystemEncoding() {
         return SpecialEncoding.FILESYSTEM.toEncoding(runtime);
     }
-    
+
     public CaseInsensitiveBytesHash<Entry> getEncodings() {
         return encodings;
     }
@@ -137,6 +137,12 @@ public final class EncodingService {
 
     // rb_locale_charmap...mostly
     public Encoding getLocaleEncoding() {
+        final Encoding consoleEncoding = getConsoleEncoding();
+
+        if (consoleEncoding != null) {
+            return consoleEncoding;
+        }
+
         Entry entry = findEncodingOrAliasEntry(new ByteList(Charset.defaultCharset().name().getBytes()));
         return entry == null ? ASCIIEncoding.INSTANCE : entry.getEncoding();
     }
@@ -165,6 +171,7 @@ public final class EncodingService {
         if (index < encodingIndex.length && (rubyEncoding = encodingIndex[index]) != null) {
             return rubyEncoding;
         }
+
         enc = loadEncoding(new ByteList(enc.getName(), false));
         return encodingIndex[enc.getIndex()];
     }
@@ -172,85 +179,35 @@ public final class EncodingService {
     public void defineEncodings() {
         HashEntryIterator hei = encodings.entryIterator();
         while (hei.hasNext()) {
-            CaseInsensitiveBytesHash.CaseInsensitiveBytesHashEntry<Entry> e = 
-                ((CaseInsensitiveBytesHash.CaseInsensitiveBytesHashEntry<Entry>)hei.next());
+            CaseInsensitiveBytesHash.CaseInsensitiveBytesHashEntry<Entry> e =
+                    ((CaseInsensitiveBytesHash.CaseInsensitiveBytesHashEntry<Entry>)hei.next());
             Entry ee = e.value;
+
             RubyEncoding encoding = RubyEncoding.newEncoding(runtime, e.bytes, e.p, e.end, ee.isDummy());
             encodingList[ee.getIndex()] = encoding;
-            defineEncodingConstants(runtime, encoding, e.bytes, e.p, e.end);
+
+            for (String constName : EncodingUtils.encodingNames(e.bytes, e.p, e.end)) {
+                defineEncodingConstant(runtime, (RubyEncoding) encodingList[ee.getIndex()], constName);
+            }
         }
     }
 
     public void defineAliases() {
         HashEntryIterator hei = aliases.entryIterator();
         while (hei.hasNext()) {
-            CaseInsensitiveBytesHash.CaseInsensitiveBytesHashEntry<Entry> e = 
-                ((CaseInsensitiveBytesHash.CaseInsensitiveBytesHashEntry<Entry>)hei.next());
-            Entry ee = e.value; 
-            RubyEncoding encoding = (RubyEncoding)encodingList[ee.getIndex()];
-            defineEncodingConstants(runtime, encoding, e.bytes, e.p, e.end);
-        }
-    }
+            CaseInsensitiveBytesHash.CaseInsensitiveBytesHashEntry<Entry> e =
+                    ((CaseInsensitiveBytesHash.CaseInsensitiveBytesHashEntry<Entry>)hei.next());
+            Entry ee = e.value;
 
-    private void defineEncodingConstants(Ruby runtime, RubyEncoding encoding, byte[]name, int p,
-            int end) {
-        Encoding enc = ASCIIEncoding.INSTANCE;
-        int s = p;
-
-        int code = name[s] & 0xff;
-        if (enc.isDigit(code)) return;
-
-        boolean hasUpper = false;
-        boolean hasLower = false;
-        if (enc.isUpper(code)) {
-            hasUpper = true;
-            while (++s < end && (enc.isAlnum(name[s] & 0xff) || name[s] == (byte)'_')) {
-                if (enc.isLower(name[s] & 0xff)) hasLower = true;
-            }
-        }
-
-        boolean isValid = false;
-        if (s >= end) {
-            isValid = true;
-            defineEncodingConstant(runtime, encoding, name, p, end);
-        }
-
-        if (!isValid || hasLower) {
-            if (!hasLower || !hasUpper) {
-                do {
-                    code = name[s] & 0xff;
-                    if (enc.isLower(code)) hasLower = true;
-                    if (enc.isUpper(code)) hasUpper = true;
-                } while (++s < end && (!hasLower || !hasUpper));
-            }
-
-            byte[]constName = new byte[end - p];
-            System.arraycopy(name, p, constName, 0, end - p);
-            s = 0;
-            code = constName[s] & 0xff;
-
-            if (!isValid) {
-                if (enc.isLower(code)) constName[s] = AsciiTables.ToUpperCaseTable[code];
-                for (; s < constName.length; ++s) {
-                    if (!enc.isAlnum(constName[s] & 0xff)) constName[s] = (byte)'_';
-                }
-                if (hasUpper) {
-                    defineEncodingConstant(runtime, encoding, constName, 0, constName.length);
-                }
-            }
-            if (hasLower) {
-                for (s = 0; s < constName.length; ++s) {
-                    code = constName[s] & 0xff;
-                    if (enc.isLower(code)) constName[s] = AsciiTables.ToUpperCaseTable[code];
-                }
-                defineEncodingConstant(runtime, encoding, constName, 0, constName.length);
+            // The constant names must be treated by the the <code>encodingNames</code> helper.
+            for (String constName : EncodingUtils.encodingNames(e.bytes, e.p, e.end)) {
+                defineEncodingConstant(runtime, (RubyEncoding) encodingList[ee.getIndex()], constName);
             }
         }
     }
 
-    private void defineEncodingConstant(Ruby runtime, RubyEncoding encoding, byte[]constName,
-            int constP, int constEnd) {
-        runtime.getEncoding().defineConstant(new String(constName, constP , constEnd), encoding);
+    private void defineEncodingConstant(Ruby runtime, RubyEncoding encoding, String constName) {
+        runtime.getEncoding().defineConstant(constName, encoding);
     }
 
     public IRubyObject getDefaultExternal() {
@@ -276,6 +233,17 @@ public final class EncodingService {
         return defaultEncoding != null ? getEncoding(defaultEncoding) : runtime.getNil();
     }
 
+    public IRubyObject findEncodingObject(byte[] bytes) {
+        Entry entry = findEncodingEntry(bytes);
+        Encoding enc;
+        if (entry != null) {
+            enc = entry.getEncoding();
+        } else {
+            enc = ASCIIEncoding.INSTANCE;
+        }
+        return convertEncodingToRubyEncoding(enc);
+    }
+
     public Encoding getJavaDefault() {
         return javaDefault;
     }
@@ -294,41 +262,44 @@ public final class EncodingService {
 
         if (arg instanceof RubyEncoding) {
             return ((RubyEncoding) arg).getEncoding();
-        } else if (arg instanceof RubyFixnum && RubyNKF.NKFCharsetMap.containsKey((int)arg.convertToInteger().getLongValue())) {
-            return getEncodingFromNKFId(arg);
-        } else if ((arg = arg.checkStringType19()).isNil()) {
+        }
+        if (arg instanceof RubyFixnum) {
+            final int id = (int) arg.convertToInteger().getLongValue();
+            final String name = RubyNKF.NKFCharsetMap.get(id);
+            if ( name != null ) return getEncodingFromNKFName(name);
+        }
+        if ( ( arg = arg.checkStringType19() ).isNil() ) {
             return null;
-        } else if (!((RubyString)arg).getEncoding().isAsciiCompatible()) {
+        }
+        if ( ! ((RubyString) arg).getEncoding().isAsciiCompatible() ) {
             return null;
+        }
+        if (error) {
+            return findEncoding((RubyString)arg);
         } else {
-            if (error) {
-                return findEncoding((RubyString)arg);
-            } else {
-                return findEncodingNoError((RubyString)arg);
-            }
+            return findEncodingNoError((RubyString)arg);
         }
     }
-    
-    private Encoding getEncodingFromNKFId(IRubyObject id) {
-        String name = RubyNKF.NKFCharsetMap.get((int)id.convertToInteger().getLongValue());
+
+    private Encoding getEncodingFromNKFName(final String name) {
         HashEntryIterator hei = encodings.entryIterator();
         while (hei.hasNext()) {
-            CaseInsensitiveBytesHash.CaseInsensitiveBytesHashEntry<Entry> e = 
-                ((CaseInsensitiveBytesHash.CaseInsensitiveBytesHashEntry<Entry>)hei.next());
-            EncodingDB.Entry ee = e.value;
-            String className = ee.getEncodingClass();
-            if (className.equals(name)) {
-                Encoding enc = ee.getEncoding();
-                return enc;
+            @SuppressWarnings("unchecked")
+            CaseInsensitiveBytesHash.CaseInsensitiveBytesHashEntry<Entry> e =
+                ((CaseInsensitiveBytesHash.CaseInsensitiveBytesHashEntry<Entry>) hei.next());
+            EncodingDB.Entry entry = e.value;
+            String className = entry.getEncodingClass();
+            if ( className.equals(name) ) {
+                return entry.getEncoding();
             }
         }
-        return null;   
+        return null;
     }
-    
+
     public Encoding getEncodingFromString(String string) {
         if (string == null) return null;
 
-        ByteList name = new ByteList(ByteList.plain(string));
+        ByteList name = new ByteList(ByteList.plain(string), false);
         checkAsciiEncodingName(name);
 
         SpecialEncoding special = SpecialEncoding.valueOf(name);
@@ -362,18 +333,26 @@ public final class EncodingService {
     public Encoding findEncodingNoError(IRubyObject str) {
         return findEncodingCommon(str, false);
     }
-    
+
+    public Encoding findEncodingNoError(ByteList str) {
+        return findEncodingCommon(str, false);
+    }
+
     private Encoding findEncodingCommon(IRubyObject str, boolean error) {
         ByteList name = str.convertToString().getByteList();
+        return findEncodingCommon(name, error);
+    }
+
+    private Encoding findEncodingCommon(ByteList name, boolean error) {
         checkAsciiEncodingName(name);
 
         SpecialEncoding special = SpecialEncoding.valueOf(name);
         if (special != null) {
             return special.toEncoding(runtime);
         }
-        
+
         if (error) return findEncodingWithError(name);
-        
+
         Entry e = findEncodingOrAliasEntry(name);
 
         if (e == null) return null;
@@ -399,12 +378,12 @@ public final class EncodingService {
 
         return findEntryWithError(name);
     }
-    
+
     public IRubyObject rubyEncodingFromObject(IRubyObject str) {
         if (str instanceof RubyEncoding) {
             return str;
         }
-        
+
         Entry entry = findEntry(str);
         if (entry == null) return runtime.getNil();
         return getEncodingList()[entry.getIndex()];
@@ -412,13 +391,11 @@ public final class EncodingService {
 
     /**
      * Get a java.nio Charset for the given encoding, or null if impossible
-     * 
+     *
      * @param encoding the encoding
      * @return the charset
      */
     public Charset charsetForEncoding(Encoding encoding) {
-        Charset charset = encoding.getCharset();
-
         if (encoding.toString().equals("ASCII-8BIT")) {
             return Charset.forName("ISO-8859-1");
         }
@@ -466,8 +443,15 @@ public final class EncodingService {
             case EXTERNAL: return runtime.getDefaultExternalEncoding();
             case INTERNAL: return runtime.getDefaultInternalEncoding();
             case FILESYSTEM:
-                // This needs to do something different on Windows. See encoding.c,
-                // in the enc_set_filesystem_encoding function.
+                if (Platform.IS_WINDOWS) {
+                    String fileEncoding = SafePropertyAccessor.getProperty("file.encoding", "UTF-8");
+                    try {
+                        return service.getEncodingFromString(fileEncoding);
+                    } catch (RaiseException re) {
+                        runtime.getWarnings().warning("could not load encoding for file.encoding of " + fileEncoding + ", using default external");
+                        if (runtime.isDebug()) re.printStackTrace();
+                    }
+                }
                 return runtime.getDefaultExternalEncoding();
             default:
                 throw new RuntimeException("invalid SpecialEncoding: " + this);
@@ -503,4 +487,9 @@ public final class EncodingService {
         if (e == null) return null;
         return findEncodingEntry(new ByteList(e.getName()));
     }
+
+    @Deprecated
+    public Encoding getFileSystemEncoding(Ruby runtime) {
+        return getFileSystemEncoding();
+    }
 }
\ No newline at end of file
