@@ -11,7 +11,7 @@
  * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
  * implied. See the License for the specific language governing
  * rights and limitations under the License.
- * 
+ *
  * Alternatively, the contents of this file may be used under the terms of
  * either of the GNU General Public License Version 2 or later (the "GPL"),
  * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
@@ -37,16 +37,19 @@ import org.jruby.RubyBasicObject;
 import org.jruby.RubyClass;
 import org.jruby.runtime.ObjectSpace;
 import org.jruby.runtime.builtin.IRubyObject;
+import org.jruby.util.ArraySupport;
+import org.jruby.util.cli.Options;
 import org.jruby.util.unsafe.UnsafeHolder;
+import static org.jruby.util.StringSupport.EMPTY_STRING_ARRAY;
 
 /**
  * This class encapculates all logic relating to the management of instance
  * variable tables in RubyBasicObject instances.
- * 
+ *
  * The logic originally lived in both RubyBasicObject and RubyClass, tightly
  * coupled to each and difficult to follow as it bounced back and forth. We
  * moved the logic here for a couple reasons:
- * 
+ *
  * <li>To have a single place from which we could follow ivar logic.</li>
  * <li>To make it easier to swap in new implementations of instance variable
  * logic as we work toward reifying ivars into fields.</li>
@@ -55,34 +58,30 @@ import org.jruby.util.unsafe.UnsafeHolder;
 public class VariableTableManager {
     /** the "real" class associated with this table */
     private final RubyClass realClass;
-    
-    /** an empty array of String */
-    private static String[] EMPTY_STRING_ARRAY = new String[0];
-    
+
     /** a map from strings to accessors for this table */
-    private Map<String, VariableAccessor> variableAccessors = (Map<String, VariableAccessor>)Collections.EMPTY_MAP;
-    
+    @SuppressWarnings("unchecked")
+    private Map<String, VariableAccessor> variableAccessors = Collections.EMPTY_MAP;
+
     /** an array of all registered variable names */
     private volatile String[] variableNames = EMPTY_STRING_ARRAY;
 
     /** whether a slot has been allocated to object_id */
     private volatile boolean hasObjectID = false;
-    
+
     /** whether objects associated with this table use fields */
     private volatile int fieldVariables = 0;
-    
+
     /** a lazy accessor for object_id */
     private final VariableAccessorField objectIdVariableAccessorField = new VariableAccessorField("object_id");
-    /** a lazy accessor for C ext handle */
-    private final VariableAccessorField cextHandleVariableAccessorField = new VariableAccessorField("cext");
     /** a lazy accessor for FFI handle */
     private final VariableAccessorField ffiHandleVariableAccessorField = new VariableAccessorField("ffi");
     /** a lazy accessor for object group */
     private final VariableAccessorField objectGroupVariableAccessorField = new VariableAccessorField("objectspace_group");
-    
+
     /**
      * Construct a new VariableTable Manager for the given "real" class.
-     * 
+     *
      * @param realClass the "real" class associated with this table
      */
     public VariableTableManager(RubyClass realClass) {
@@ -91,26 +90,26 @@ public class VariableTableManager {
 
     /**
      * Get the map of all current variable accessors with intent to read from it.
-     * 
+     *
      * @return a map of current variable accessors
      */
     public Map<String, VariableAccessor> getVariableAccessorsForRead() {
         return variableAccessors;
     }
-    
+
     /**
      * Whether this table has been used to allocate space for an object_id.
-     * 
+     *
      * @return true if object_id has been allocated; false otherwise
      */
     public boolean hasObjectID() {
         return hasObjectID;
     }
-    
+
     /**
      * Get the object_id from a given RubyBasicObject, possibly allocating
      * space for it.
-     * 
+     *
      * @param self the object from which to get object_id
      * @return the object's object_id (possibly new)
      */
@@ -118,7 +117,7 @@ public class VariableTableManager {
         VariableAccessor objectIdAccessor = getObjectIdAccessorField().getVariableAccessorForRead();
         Long id = (Long)objectIdAccessor.get(self);
         if (id != null) return id;
-        
+
         synchronized (self) {
             objectIdAccessor = getObjectIdAccessorField().getVariableAccessorForRead();
             id = (Long)objectIdAccessor.get(self);
@@ -127,10 +126,10 @@ public class VariableTableManager {
             return initObjectId(self, getObjectIdAccessorField().getVariableAccessorForWrite(this));
         }
     }
-    
+
     /**
      * Virtual entry point for setting a variable into an object.
-     * 
+     *
      * @param self the object into which to set the value
      * @param index the index allocated for the value
      * @param value the value
@@ -142,10 +141,10 @@ public class VariableTableManager {
             StampedVariableAccessor.setVariable(self,realClass,index,value);
         }
     }
-    
+
     /**
      * Static entry point for setting a variable in an object.
-     * 
+     *
      * @param realClass the "real" class of the object
      * @param self the object into which to set the variable
      * @param index the index allocated for the variable
@@ -162,7 +161,7 @@ public class VariableTableManager {
     /**
      * Get the variable accessor for the given name with intent to use it for
      * writing.
-     * 
+     *
      * @param name the name of the variable
      * @return an accessor appropriate for writing
      */
@@ -188,7 +187,7 @@ public class VariableTableManager {
         }
         return ivarAccessor;
     }
-    
+
     public VariableAccessor getVariableAccessorForVar(String name, int index) {
         VariableAccessor ivarAccessor = variableAccessors.get(name);
         if (ivarAccessor == null) {
@@ -215,7 +214,7 @@ public class VariableTableManager {
     /**
      * Get the variable accessor for the given name with intent to use it for
      * reading.
-     * 
+     *
      * @param name the name of the variable
      * @return an accessor appropriate for reading
      */
@@ -227,43 +226,16 @@ public class VariableTableManager {
 
     /**
      * Retrieve the lazy accessor (VariableAccessorField) for object_id.
-     * 
+     *
      * @return the lazy accessor for object_id
      */
     public VariableAccessorField getObjectIdAccessorField() {
         return objectIdVariableAccessorField;
     }
 
-    /**
-     * Retrieve the lazy accessor (VariableAccessorField) for C ext handle.
-     * 
-     * @return the lazy accessor for C ext handle
-     */
-    public VariableAccessorField getNativeHandleAccessorField() {
-        return cextHandleVariableAccessorField;
-    }
-
-    /**
-     * Retrieve the read accessor for C ext handle.
-     * 
-     * @return the read accessor for C ext handle
-     */
-    public VariableAccessor getNativeHandleAccessorForRead() {
-        return cextHandleVariableAccessorField.getVariableAccessorForRead();
-    }
-
-    /**
-     * Retrieve the write accessor for C ext handle.
-     * 
-     * @return the write accessor for C ext handle
-     */
-    public VariableAccessor getNativeHandleAccessorForWrite() {
-        return cextHandleVariableAccessorField.getVariableAccessorForWrite(this);
-    }
-
     /**
      * Retrieve the lazy accessor (VariableAccessorField) for FFI handle.
-     * 
+     *
      * @return the lazy accessor for FFI handle
      */
     public VariableAccessorField getFFIHandleAccessorField() {
@@ -272,7 +244,7 @@ public class VariableTableManager {
 
     /**
      * Retrieve the read accessor for FFI handle.
-     * 
+     *
      * @return the read accessor for FFI handle
      */
     public VariableAccessor getFFIHandleAccessorForRead() {
@@ -281,7 +253,7 @@ public class VariableTableManager {
 
     /**
      * Retrieve the write accessor for FFI handle.
-     * 
+     *
      * @return the write accessor for FFI handle
      */
     public VariableAccessor getFFIHandleAccessorForWrite() {
@@ -290,7 +262,7 @@ public class VariableTableManager {
 
     /**
      * Retrieve the lazy accessor (VariableAccessorField) for object group.
-     * 
+     *
      * @return the lazy accessor for object group
      */
     public VariableAccessorField getObjectGroupAccessorField() {
@@ -299,7 +271,7 @@ public class VariableTableManager {
 
     /**
      * Retrieve the read accessor for object group.
-     * 
+     *
      * @return the read accessor for object group
      */
     public VariableAccessor getObjectGroupAccessorForRead() {
@@ -308,40 +280,16 @@ public class VariableTableManager {
 
     /**
      * Retrieve the write accessor for object group.
-     * 
+     *
      * @return the write accessor for object group
      */
     public VariableAccessor getObjectGroupAccessorForWrite() {
         return objectGroupVariableAccessorField.getVariableAccessorForWrite(this);
     }
-    
-    /**
-     * Retrieve the C ext handle for the given object.
-     * 
-     * @param self the object
-     * @return the object's C ext handle
-     */
-    public final Object getNativeHandle(RubyBasicObject self) {
-        return getNativeHandleAccessorForRead().get(self);
-    }
-
-    /**
-     * Set the C ext handle for the given object.
-     * 
-     * @param self the object
-     * @param value the object's C ext handle
-     */
-    public final void setNativeHandle(RubyBasicObject self, Object value) {
-        int index = getNativeHandleAccessorForRead().getIndex();
-        if(index == -1) {
-            return;
-        }
-        setVariableInternal(realClass, self, index, value);
-    }
 
     /**
      * Retrieve the FFI ext handle for the given object.
-     * 
+     *
      * @param self the object
      * @return the object's FFI handle
      */
@@ -351,7 +299,7 @@ public class VariableTableManager {
 
     /**
      * Set the FFI handle for the given object.
-     * 
+     *
      * @param self the object
      * @param self the object's FFI handle
      */
@@ -363,7 +311,7 @@ public class VariableTableManager {
     /**
      * Get the size of the variable table, excluding extra vars (object_id,
      * etc).
-     * 
+     *
      * @return the variable table's size, excluding extras
      */
     public int getVariableTableSize() {
@@ -373,8 +321,8 @@ public class VariableTableManager {
     /**
      * Get the size of the variable table, including extra vars (object_etc,
      * etc).
-     * 
-     * @return 
+     *
+     * @return
      */
     public int getVariableTableSizeWithExtras() {
         return variableNames.length;
@@ -382,7 +330,7 @@ public class VariableTableManager {
 
     /**
      * Get a Map representing all variables registered in the variable table.
-     * 
+     *
      * @return a map of names to accessors for all variables
      */
     public Map<String, VariableAccessor> getVariableTableCopy() {
@@ -397,16 +345,13 @@ public class VariableTableManager {
      * @return a copy of the array of known instance variable names
      */
     public String[] getVariableNames() {
-        String[] original = variableNames;
-        String[] copy = new String[original.length];
-        System.arraycopy(original, 0, copy, 0, original.length);
-        return copy;
+        return variableNames.clone();
     }
-    
+
     /**
      * Sync one this object's variables with other's - this is used to make
      * rbClone work correctly.
-     * 
+     *
      * @param self the object into which to sync variables
      * @param other the object from which to sync variables
      */
@@ -416,9 +361,9 @@ public class VariableTableManager {
 
         if (sameTable && fieldVariables == 0) {
             int idIndex = otherRealClass.getObjectIdAccessorField().getVariableAccessorForRead().getIndex();
-            
+
             Object[] otherVars = ((RubyBasicObject) other).varTable;
-            
+
             if(UnsafeHolder.U == null)
             {
                 synchronized (self) {
@@ -433,18 +378,18 @@ public class VariableTableManager {
                     // acquire exclusive write mode
                     if(!UnsafeHolder.U.compareAndSwapInt(self, RubyBasicObject.STAMP_OFFSET, oldStamp, ++oldStamp))
                         continue;
-                    
+
                     Object[] currentTable = (Object[]) UnsafeHolder.U.getObjectVolatile(self, RubyBasicObject.VAR_TABLE_OFFSET);
                     Object[] newTable = makeSyncedTable(currentTable,otherVars, idIndex);
-                    
+
                     UnsafeHolder.U.putOrderedObject(self, RubyBasicObject.VAR_TABLE_OFFSET, newTable);
-                    
+
                     // release write mode
                     self.varTableStamp = oldStamp+1;
                     break;
                 }
 
-                
+
             }
 
         } else {
@@ -462,7 +407,7 @@ public class VariableTableManager {
             }
         }
     }
-    
+
     /**
      * Returns true if object has any variables, defined as:
      * <ul>
@@ -500,12 +445,12 @@ public class VariableTableManager {
             getVariableAccessorForWrite(name).set(object, value);
         }
     }
-    
+
     public Object clearVariable(RubyBasicObject object, String name) {
         synchronized(object) {
             Object value = getVariableAccessorForRead(name).get(object);
             getVariableAccessorForWrite(name).set(object, null);
-            
+
             return value;
         }
     }
@@ -528,7 +473,7 @@ public class VariableTableManager {
      * Synchronization here ends up being a bottleneck for every object
      * created from the class that contains this VariableTableManager. See
      * GH #1400.
-     * 
+     *
      * @param objectIdAccessor The variable accessor to use for storing the
      * generated object ID
      * @return The generated object ID
@@ -536,13 +481,13 @@ public class VariableTableManager {
     private long initObjectId(RubyBasicObject self, VariableAccessor objectIdAccessor) {
         Ruby runtime = self.getRuntime();
         long id;
-        
+
         if (runtime.isObjectSpaceEnabled()) {
             id = runtime.getObjectSpace().createAndRegisterObjectId(self);
         } else {
             id = ObjectSpace.calculateObjectId(self);
         }
-        
+
         // we use a direct path here to avoid frozen checks
         setObjectId(realClass, self, objectIdAccessor.getIndex(), id);
 
@@ -551,7 +496,7 @@ public class VariableTableManager {
 
     /**
      * Update object_id with the given value.
-     * 
+     *
      * @param self the object into which to set object_id
      * @param index the index allocated to store object_id
      * @param value the value of object_id
@@ -564,62 +509,69 @@ public class VariableTableManager {
     /**
      * Make a new variable table based on the values in a current and other
      * (incoming) table, excluding object_id at specified index.
-     * 
+     *
      * @param currentTable the current table
      * @param otherTable the other (incoming) table
      * @param objectIdIdx the index of object_id to exclude
      * @return a new table formed by combining the given tables
      */
     private static Object[] makeSyncedTable(Object[] currentTable, Object[] otherTable, int objectIdIdx) {
-        if(currentTable == null || currentTable.length < otherTable.length) {
+        if (currentTable == null || currentTable.length < otherTable.length) {
             currentTable = otherTable.clone();
         } else {
-            System.arraycopy(otherTable, 0, currentTable, 0, otherTable.length);
+            ArraySupport.copy(otherTable, currentTable, 0, otherTable.length);
         }
-    
+
         // null out object ID so we don't share it
         if (objectIdIdx >= 0 && objectIdIdx < currentTable.length) {
             currentTable[objectIdIdx] = null;
         }
-        
+
         return currentTable;
     }
 
     /**
      * Allocate a new VariableAccessor for the named variable.
-     * 
+     *
      * @param name the name of the variable
      * @return the new VariableAccessor
      */
     synchronized final VariableAccessor allocateVariableAccessor(String name) {
         int id = realClass.id;
-        String[] myVariableNames = variableNames;
 
-        int newIndex = myVariableNames.length;
-        String[] newVariableNames = new String[newIndex + 1];
+        final String[] myVariableNames = variableNames;
+        final int newIndex = myVariableNames.length;
 
         VariableAccessor newVariableAccessor;
-        if (UnsafeHolder.U == null) {
-            newVariableAccessor = new SynchronizedVariableAccessor(realClass, name, newIndex, id);
+        if (Options.VOLATILE_VARIABLES.load()) {
+            if (UnsafeHolder.U == null) {
+                newVariableAccessor = new SynchronizedVariableAccessor(realClass, name, newIndex, id);
+            } else {
+                newVariableAccessor = new StampedVariableAccessor(realClass, name, newIndex, id);
+            }
         } else {
-            newVariableAccessor = new StampedVariableAccessor(realClass, name, newIndex, id);
+            if (UnsafeHolder.U == null) {
+                newVariableAccessor = new NonvolatileVariableAccessor(realClass, name, newIndex, id);
+            } else {
+                // We still need safe updating of the vartable, so we fall back on sync here too.
+                newVariableAccessor = new StampedVariableAccessor(realClass, name, newIndex, id);
+            }
         }
 
-        System.arraycopy(myVariableNames, 0, newVariableNames, 0, newIndex);
-
+        final String[] newVariableNames = new String[newIndex + 1];
+        ArraySupport.copy(myVariableNames, 0, newVariableNames, 0, newIndex);
         newVariableNames[newIndex] = name;
         variableNames = newVariableNames;
 
         return newVariableAccessor;
     }
-    
+
     synchronized final VariableAccessor allocateVariableAccessorForVar(String name, int index) {
         int id = realClass.id;
-        String[] myVariableNames = variableNames;
 
-        int newIndex = myVariableNames.length;
-        String[] newVariableNames = new String[newIndex + 1];
-        
+        final String[] myVariableNames = variableNames;
+        final int newIndex = myVariableNames.length;
+
         fieldVariables += 1;
 
         VariableAccessor newVariableAccessor;
@@ -658,8 +610,8 @@ public class VariableTableManager {
                 throw new RuntimeException("unsupported var index in " + realClass + ": " + index);
         }
 
-        System.arraycopy(myVariableNames, 0, newVariableNames, 0, newIndex);
-
+        final String[] newVariableNames = new String[newIndex + 1];
+        ArraySupport.copy(myVariableNames, 0, newVariableNames, 0, newIndex);
         newVariableNames[newIndex] = name;
         variableNames = newVariableNames;
 
