@@ -43,6 +43,7 @@ import java.util.LinkedList;
 import java.util.List;
 import java.util.Locale;
 
+import jnr.constants.platform.Errno;
 import org.jcodings.Encoding;
 import org.jcodings.specific.ASCIIEncoding;
 import org.joda.time.Chronology;
@@ -51,6 +52,7 @@ import org.joda.time.chrono.GJChronology;
 import org.joda.time.chrono.JulianChronology;
 import org.jruby.RubyEncoding;
 import org.jruby.RubyString;
+import org.jruby.RubyTime;
 import org.jruby.lexer.StrftimeLexer;
 import org.jruby.runtime.ThreadContext;
 import org.jruby.runtime.builtin.IRubyObject;
@@ -67,8 +69,6 @@ public class RubyDateFormatter {
     static enum Format {
         /** encoding to give to output */
         FORMAT_ENCODING,
-        /** taint output */
-        FORMAT_TAINT,
         /** raw string, no formatting */
         FORMAT_STRING,
         /** formatter */
@@ -243,7 +243,10 @@ public class RubyDateFormatter {
     }
 
     public List<Token> compilePattern(RubyString format, boolean dateLibrary) {
-        ByteList pattern = format.getByteList();
+        return compilePattern(format.getByteList(), dateLibrary);
+    }
+
+    public List<Token> compilePattern(ByteList pattern, boolean dateLibrary) {
         List<Token> compiledPattern = new LinkedList<Token>();
 
         Encoding enc = pattern.getEncoding();
@@ -254,10 +257,6 @@ public class RubyDateFormatter {
             compiledPattern.add(new Token(Format.FORMAT_ENCODING, enc));
         }
 
-        if (format.isTaint()) {
-            compiledPattern.add(new Token(Format.FORMAT_TAINT));
-        }
-
         ByteArrayInputStream in = new ByteArrayInputStream(pattern.getUnsafeBytes(), pattern.getBegin(), pattern.getRealSize());
         Reader reader = new InputStreamReader(in, context.runtime.getEncodingService().charsetForEncoding(pattern.getEncoding()));
         lexer.yyreset(reader);
@@ -358,13 +357,20 @@ public class RubyDateFormatter {
 
     /** Convenience method when using no pattern caching */
     public RubyString compileAndFormat(RubyString pattern, boolean dateLibrary, DateTime dt, long nsec, IRubyObject sub_millis) {
-        return format(compilePattern(pattern, dateLibrary), dt, nsec, sub_millis);
+        RubyString out = format(compilePattern(pattern, dateLibrary), dt, nsec, sub_millis);
+        if (pattern.isTaint()) {
+            out.taint(context);
+        }
+        return out;
     }
 
     public RubyString format(List<Token> compiledPattern, DateTime dt, long nsec, IRubyObject sub_millis) {
+        return context.runtime.newString(formatToByteList(compiledPattern, dt, nsec, sub_millis));
+    }
+
+    public ByteList formatToByteList(List<Token> compiledPattern, DateTime dt, long nsec, IRubyObject sub_millis) {
         RubyTimeOutputFormatter formatter = RubyTimeOutputFormatter.DEFAULT_FORMATTER;
         ByteList toAppendTo = new ByteList();
-        boolean taint = false;
 
         for (Token token: compiledPattern) {
             String output = null;
@@ -376,9 +382,6 @@ public class RubyDateFormatter {
                 case FORMAT_ENCODING:
                     toAppendTo.setEncoding((Encoding) token.getData());
                     continue; // go to next token
-                case FORMAT_TAINT:
-                    taint = true;
-                    continue; // go to next token
                 case FORMAT_OUTPUT:
                     formatter = (RubyTimeOutputFormatter) token.getData();
                     continue; // go to next token
@@ -487,7 +490,7 @@ public class RubyDateFormatter {
                     output = formatZone(colons, (int) value, formatter);
                     break;
                 case FORMAT_ZONE_ID:
-                    output = dt.getZone().getShortName(dt.getMillis());
+                    output = RubyTime.getRubyTimeZoneName(context.runtime, dt);
                     break;
                 case FORMAT_CENTURY:
                     type = NUMERIC;
@@ -546,17 +549,19 @@ public class RubyDateFormatter {
                     throw new Error("FORMAT_SPECIAL is a special token only for the lexer.");
             }
 
-            output = formatter.format(output, value, type);
+            try {
+                output = formatter.format(output, value, type);
+            } catch (IndexOutOfBoundsException ioobe) {
+                throw context.runtime.newErrnoFromErrno(Errno.ERANGE, "strftime");
+            }
+
             // reset formatter
             formatter = RubyTimeOutputFormatter.DEFAULT_FORMATTER;
 
             toAppendTo.append(output.getBytes(context.runtime.getEncodingService().charsetForEncoding(toAppendTo.getEncoding())));
         }
 
-        RubyString str = context.runtime.newString(toAppendTo);
-        if (taint)
-            str.taint(context);
-        return str;
+        return toAppendTo;
     }
 
     /**
@@ -619,16 +624,11 @@ public class RubyDateFormatter {
                 after = ":" + mm + ":" + ss;
                 break;
             case 3: // %:::z -> +hh[:mm[:ss]]
-                if (minutes == 0) {
-                    if (seconds == 0) { // +hh
-                        defaultWidth = 3;
-                        after = "";
-                    } else { // +hh:mm
-                        return formatZone(1, value, formatter);
-                    }
-                } else { // +hh:mm:ss
-                    return formatZone(2, value, formatter);
-                }
+                StringBuilder sb = new StringBuilder();
+                if (minutes != 0 || seconds != 0) sb.append(":").append(mm);
+                if (seconds != 0) sb.append(":").append(ss);
+                after = sb.toString();
+                defaultWidth = after.length() + 3;
                 break;
         }
 
