@@ -42,11 +42,10 @@ import java.io.OutputStream;
 import java.io.RandomAccessFile;
 import java.nio.ByteBuffer;
 import java.nio.channels.Channel;
-import java.nio.channels.FileChannel;
 import java.nio.channels.IllegalBlockingModeException;
 import java.nio.channels.ReadableByteChannel;
+import java.nio.channels.SeekableByteChannel;
 import java.nio.channels.SelectableChannel;
-import java.util.LinkedList;
 
 import org.jruby.Finalizable;
 import org.jruby.Ruby;
@@ -60,9 +59,10 @@ import org.jruby.util.log.LoggerFactory;
 /**
  * This file implements a seekable IO file.
  */
-public class ChannelStream implements Stream, Finalizable, NonblockWritingStream {
+@Deprecated
+public class ChannelStream implements Stream, Finalizable {
 
-    private static final Logger LOG = LoggerFactory.getLogger("ChannelStream");
+    private static final Logger LOG = LoggerFactory.getLogger(ChannelStream.class);
 
     private final static boolean DEBUG = false;
 
@@ -127,6 +127,11 @@ public class ChannelStream implements Stream, Finalizable, NonblockWritingStream
         this.modes = modes;
     }
 
+    @Override
+    public ByteBuffer getBuffer() {
+        return buffer;
+    }
+
     public Ruby getRuntime() {
         return runtime;
     }
@@ -199,12 +204,19 @@ public class ChannelStream implements Stream, Finalizable, NonblockWritingStream
         return !reading && buffer.position() > 0;
     }
 
-    private final int refillBuffer() throws IOException {
+    @Override
+    public final int bufferedAvailable() {
+        return buffer.remaining();
+    }
+
+    @Override
+    public final int refillBuffer() throws IOException {
         buffer.clear();
         int n = ((ReadableByteChannel) descriptor.getChannel()).read(buffer);
         buffer.flip();
         return n;
     }
+
     public synchronized ByteList fgets(ByteList separatorString) throws IOException, BadDescriptorException {
         checkReadable();
         ensureRead();
@@ -394,8 +406,8 @@ public class ChannelStream implements Stream, Finalizable, NonblockWritingStream
      */
     @Deprecated
     public synchronized ByteList readall() throws IOException, BadDescriptorException {
-        final long fileSize = descriptor.isSeekable() && descriptor.getChannel() instanceof FileChannel
-                ? ((FileChannel) descriptor.getChannel()).size() : 0;
+        final long fileSize = descriptor.isSeekable() && descriptor.getChannel() instanceof SeekableByteChannel
+                ? ((SeekableByteChannel) descriptor.getChannel()).size() : 0;
         //
         // Check file size - special files in /proc have zero size and need to be
         // handled by the generic read path.
@@ -403,7 +415,7 @@ public class ChannelStream implements Stream, Finalizable, NonblockWritingStream
         if (fileSize > 0) {
             ensureRead();
 
-            FileChannel channel = (FileChannel)descriptor.getChannel();
+            SeekableByteChannel channel = (SeekableByteChannel) descriptor.getChannel();
             final long left = fileSize - channel.position() + bufferedInputBytesRemaining();
             if (left <= 0) {
                 eof = true;
@@ -759,7 +771,7 @@ public class ChannelStream implements Stream, Finalizable, NonblockWritingStream
     public synchronized long fgetpos() throws IOException, PipeException, InvalidValueException, BadDescriptorException {
         // Correct position for read / write buffering (we could invalidate, but expensive)
         if (descriptor.isSeekable()) {
-            FileChannel fileChannel = (FileChannel)descriptor.getChannel();
+            SeekableByteChannel fileChannel = (SeekableByteChannel) descriptor.getChannel();
             long pos = fileChannel.position();
             // Adjust for buffered data
             if (reading) {
@@ -785,7 +797,7 @@ public class ChannelStream implements Stream, Finalizable, NonblockWritingStream
      */
     public synchronized void lseek(long offset, int type) throws IOException, InvalidValueException, PipeException, BadDescriptorException {
         if (descriptor.isSeekable()) {
-            FileChannel fileChannel = (FileChannel)descriptor.getChannel();
+            SeekableByteChannel fileChannel = (SeekableByteChannel) descriptor.getChannel();
             clearUngotChars();
             int adj = 0;
             if (reading) {
@@ -824,7 +836,6 @@ public class ChannelStream implements Stream, Finalizable, NonblockWritingStream
 
     public synchronized void sync() throws IOException, BadDescriptorException {
         flushWrite();
-        descriptor.fsync();
     }
 
     /**
@@ -864,7 +875,7 @@ public class ChannelStream implements Stream, Finalizable, NonblockWritingStream
 
     private void resetForWrite() throws IOException {
         if (descriptor.isSeekable()) {
-            FileChannel fileChannel = (FileChannel)descriptor.getChannel();
+            SeekableByteChannel fileChannel = (SeekableByteChannel) descriptor.getChannel();
             if (buffer.hasRemaining()) { // we have read ahead, and need to back up
                 fileChannel.position(fileChannel.position() - buffer.remaining());
             }
@@ -907,12 +918,12 @@ public class ChannelStream implements Stream, Finalizable, NonblockWritingStream
 
         // 128K seems to be the minimum at which the stat+seek is faster than reallocation
         final int BULK_THRESHOLD = 128 * 1024;
-        if (number >= BULK_THRESHOLD && descriptor.isSeekable() && descriptor.getChannel() instanceof FileChannel) {
+        if (number >= BULK_THRESHOLD && descriptor.isSeekable() && descriptor.getChannel() instanceof SeekableByteChannel) {
             //
             // If it is a file channel, then we can pre-allocate the output buffer
             // to the total size of buffered + remaining bytes in file
             //
-            FileChannel fileChannel = (FileChannel) descriptor.getChannel();
+            SeekableByteChannel fileChannel = (SeekableByteChannel) descriptor.getChannel();
             resultSize = (int) Math.min(fileChannel.size() - fileChannel.position() + bufferedInputBytesRemaining(), number);
         } else {
             //
@@ -1154,11 +1165,11 @@ public class ChannelStream implements Stream, Finalizable, NonblockWritingStream
     public synchronized void ftruncate(long newLength) throws IOException,
             BadDescriptorException, InvalidValueException {
         Channel ch = descriptor.getChannel();
-        if (!(ch instanceof FileChannel)) {
+        if (!(ch instanceof SeekableByteChannel)) {
             throw new InvalidValueException();
         }
         invalidateBuffer();
-        FileChannel fileChannel = (FileChannel)ch;
+        SeekableByteChannel fileChannel = (SeekableByteChannel) ch;
         long position = fileChannel.position();
         if (newLength > fileChannel.size()) {
             // truncate can't lengthen files, so we save position, seek/write, and go back
@@ -1185,7 +1196,7 @@ public class ChannelStream implements Stream, Finalizable, NonblockWritingStream
         if (reading) {
             buffer.flip();
             // if the read buffer is ahead, back up
-            FileChannel fileChannel = (FileChannel)descriptor.getChannel();
+            SeekableByteChannel fileChannel = (SeekableByteChannel) descriptor.getChannel();
             if (posOverrun != 0) fileChannel.position(fileChannel.position() - posOverrun);
         }
     }
@@ -1347,7 +1358,7 @@ public class ChannelStream implements Stream, Finalizable, NonblockWritingStream
                     selectableChannel.configureBlocking(oldBlocking);
                 }
             }
-        } else if (descriptor.getChannel() instanceof FileChannel) {
+        } else if (descriptor.getChannel() instanceof SeekableByteChannel) {
             return fread(number);
         } else {
             return null;
@@ -1360,7 +1371,7 @@ public class ChannelStream implements Stream, Finalizable, NonblockWritingStream
         if (number == 0) {
             return null;
         }
-        if (descriptor.getChannel() instanceof FileChannel) {
+        if (descriptor.getChannel() instanceof SeekableByteChannel) {
             return fread(number);
         }
 
@@ -1374,7 +1385,7 @@ public class ChannelStream implements Stream, Finalizable, NonblockWritingStream
     }
 
     public synchronized int read(ByteBuffer dst) throws IOException, BadDescriptorException, EOFException {
-        return read(dst, !(descriptor.getChannel() instanceof FileChannel));
+        return read(dst, !(descriptor.getChannel() instanceof SeekableByteChannel));
     }
 
     public synchronized int read(ByteBuffer dst, boolean partial) throws IOException, BadDescriptorException, EOFException {
@@ -1496,7 +1507,7 @@ public class ChannelStream implements Stream, Finalizable, NonblockWritingStream
 
     private static Stream maybeWrapWithLineEndingWrapper(Stream stream, ModeFlags modes) {
         if (modes.isText() || // FIXME: Remove this one textmode is part of transcoding.
-                (Platform.IS_WINDOWS && stream.getDescriptor().getChannel() instanceof FileChannel && !modes.isBinary())) {
+                (Platform.IS_WINDOWS && stream.getDescriptor().getChannel() instanceof SeekableByteChannel && !modes.isBinary())) {
             return new CRLFStreamWrapper(stream);
         }
         return stream;
