@@ -1,18 +1,18 @@
-/**
- * **** BEGIN LICENSE BLOCK *****
+/*
+ **** BEGIN LICENSE BLOCK *****
  * Version: EPL 1.0/GPL 2.0/LGPL 2.1
  *
- * The contents of this file are subject to the Common Public
+ * The contents of this file are subject to the Eclipse Public
  * License Version 1.0 (the "License"); you may not use this file
  * except in compliance with the License. You may obtain a copy of
- * the License at http://www.eclipse.org/legal/cpl-v10.html
+ * the License at http://www.eclipse.org/legal/epl-v10.html
  *
  * Software distributed under the License is distributed on an "AS
  * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
  * implied. See the License for the specific language governing
  * rights and limitations under the License.
  *
- * Copyright (C) 2009-2012 Yoko Harada <yokolet@gmail.com>
+ * Copyright (C) 2009-2017 Yoko Harada <yokolet@gmail.com>
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either of the GNU General Public License Version 2 or later (the "GPL"),
@@ -25,21 +25,14 @@
  * and other provisions required by the GPL or the LGPL. If you do not delete
  * the provisions above, a recipient may use your version of this file under
  * the terms of any one of the EPL, the GPL or the LGPL.
- * **** END LICENSE BLOCK *****
- */
+ ***** END LICENSE BLOCK *****/
 package org.jruby.embed.jsr223;
 
 import javax.script.Invocable;
 import javax.script.Compilable;
-import java.io.FileNotFoundException;
-import java.io.FileOutputStream;
-import java.io.FileReader;
-import java.io.FileWriter;
-import java.io.IOException;
-import java.io.OutputStream;
-import java.io.Reader;
-import java.io.StringWriter;
-import java.io.Writer;
+import java.io.*;
+import java.lang.reflect.Field;
+import java.nio.charset.Charset;
 import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.Calendar;
@@ -76,7 +69,7 @@ import static org.junit.Assert.*;
  * @author Yoko Harada
  */
 public class JRubyEngineTest {
-    private String basedir = System.getProperty("user.dir");
+    private String basedir = new File(System.getProperty("user.dir")).getParent();
 
     static Logger logger0 = Logger.getLogger(JRubyEngineTest.class.getName());
     static Logger logger1 = Logger.getLogger(JRubyEngineTest.class.getName());
@@ -97,7 +90,7 @@ public class JRubyEngineTest {
 
     @Before
     public void setUp() throws FileNotFoundException, IOException {
-        outStream = new FileOutputStream(basedir + "/target/run-junit-embed.log", true);
+        outStream = new FileOutputStream(basedir + "/core/target/run-junit-embed.log", true);
         Handler handler = new StreamHandler(outStream, new SimpleFormatter());
         logger0.addHandler(handler);
         logger0.setUseParentHandlers(false);
@@ -106,7 +99,7 @@ public class JRubyEngineTest {
         logger1.addHandler(new ConsoleHandler());
         logger1.setLevel(Level.WARNING);
 
-        writer = new FileWriter(basedir + "/target/run-junit-embed.txt", true);
+        writer = new FileWriter(basedir + "/core/target/run-junit-embed.txt", true);
     }
 
     @After
@@ -117,7 +110,7 @@ public class JRubyEngineTest {
     /**
      * Test of compile method, of class Jsr223JRubyEngine.
      */
-    @Test
+    //@Test
     public void testCompile_String() throws Exception {
         logger1.info("[compile string]");
         ScriptEngine instance;
@@ -150,7 +143,7 @@ public class JRubyEngineTest {
     /**
      * Test of compile method, of class Jsr223JRubyEngine.
      */
-    @Test
+    //@Test
     public void testCompile_Reader() throws Exception {
         logger1.info("[compile reader]");
         ScriptEngine instance;
@@ -160,7 +153,7 @@ public class JRubyEngineTest {
             ScriptEngineManager manager = new ScriptEngineManager();
             instance = manager.getEngineByName("jruby");
         }
-        String filename = basedir + "/src/test/ruby/org/jruby/embed/ruby/proverbs_of_the_day.rb";
+        String filename = basedir + "/core/src/test/ruby/org/jruby/embed/ruby/proverbs_of_the_day.rb";
         Reader reader = new FileReader(filename);
 
         instance.put("$day", -1);
@@ -183,7 +176,7 @@ public class JRubyEngineTest {
     /**
      * Test of eval method, of class Jsr223JRubyEngine.
      */
-    @Test
+    //@Test
     public void testEval_String_ScriptContext() throws Exception {
         logger1.info("[eval String with ScriptContext]");
         ScriptEngine instance;
@@ -230,7 +223,7 @@ public class JRubyEngineTest {
     /**
      * Test of eval method, of class Jsr223JRubyEngine.
      */
-    @Test
+    //@Test
     public void testEval_String_ScriptContext2() throws Exception {
         logger1.info("[eval String with ScriptContext 2]");
         ScriptEngine instance = null;
@@ -260,7 +253,7 @@ public class JRubyEngineTest {
     /**
      * Test of eval method, of class Jsr223JRubyEngine.
      */
-    @Test
+    //@Test
     public void testEval_Reader_ScriptContext() throws Exception {
         logger1.info("[eval Reader with ScriptContext]");
         ScriptEngine instance;
@@ -270,7 +263,7 @@ public class JRubyEngineTest {
             ScriptEngineManager manager = new ScriptEngineManager();
             instance = manager.getEngineByName("jruby");
         }
-        String filename = basedir + "/src/test/ruby/org/jruby/embed/ruby/list_printer.rb";
+        String filename = basedir + "/core/src/test/ruby/org/jruby/embed/ruby/list_printer.rb";
         Reader reader = new FileReader(filename);
         ScriptContext context = new SimpleScriptContext();
 
@@ -289,7 +282,7 @@ public class JRubyEngineTest {
     /**
      * Test of eval method, of class Jsr223JRubyEngine.
      */
-    @Test
+    //@Test
     public void testEval_String() throws Exception {
         logger1.info("eval String");
         ScriptEngine instance;
@@ -314,7 +307,7 @@ public class JRubyEngineTest {
     /**
      * Test of eval method, of class Jsr223JRubyEngine.
      */
-    @Test
+    //@Test
     public void testEval_Reader() throws Exception {
         logger1.info("eval Reader");
         ScriptEngine instance;
@@ -325,7 +318,7 @@ public class JRubyEngineTest {
             ScriptEngineManager manager = new ScriptEngineManager();
             instance = manager.getEngineByName("jruby");
         }
-        String filename = basedir + "/src/test/ruby/org/jruby/embed/ruby/next_year.rb";
+        String filename = basedir + "/core/src/test/ruby/org/jruby/embed/ruby/next_year.rb";
         Reader reader = new FileReader(filename);
 
         long expResult = getNextYear();
@@ -345,7 +338,7 @@ public class JRubyEngineTest {
     /**
      * Test of eval method, of class Jsr223JRubyEngine.
      */
-    @Test
+    //@Test
     public void testEval_String_Bindings() throws Exception {
         logger1.info("eval String with Bindings");
         ScriptEngine instance;
@@ -373,7 +366,7 @@ public class JRubyEngineTest {
     /**
      * Test of eval method, of class Jsr223JRubyEngine.
      */
-    @Test
+    //@Test
     public void testEval_Reader_Bindings() throws Exception {
         logger1.info("eval Reader with Bindings");
         ScriptEngine instance;
@@ -384,7 +377,7 @@ public class JRubyEngineTest {
             ScriptEngineManager manager = new ScriptEngineManager();
             instance = manager.getEngineByName("jruby");
         }
-        String filename = basedir + "/src/test/ruby/org/jruby/embed/ruby/count_down.rb";
+        String filename = basedir + "/core/src/test/ruby/org/jruby/embed/ruby/count_down.rb";
         Reader reader = new FileReader(filename);
         Bindings bindings = new SimpleBindings();
         bindings.put("@month", 6);
@@ -400,7 +393,7 @@ public class JRubyEngineTest {
     /**
      * Test of get method, of class Jsr223JRubyEngine.
      */
-    @Test
+    //@Test
     public void testGet() {
         logger1.info("get");
         ScriptEngine instance;
@@ -436,7 +429,7 @@ public class JRubyEngineTest {
     /**
      * Test of put method, of class Jsr223JRubyEngine.
      */
-    @Test
+    //@Test
     public void testPut() {
         logger1.info("put");
         String key = "";
@@ -457,7 +450,7 @@ public class JRubyEngineTest {
     /**
      * Test of getBindings method, of class Jsr223JRubyEngine.
      */
-    @Test
+    //@Test
     public void testGetBindings() throws ScriptException {
         logger1.info("getBindings");
         ScriptEngine instance;
@@ -495,7 +488,7 @@ public class JRubyEngineTest {
     /**
      * Test of setBindings method, of class Jsr223JRubyEngine.
      */
-    @Test
+    //@Test
     public void testSetBindings() throws ScriptException {
         logger1.info("setBindings");
         ScriptEngine instance;
@@ -525,7 +518,7 @@ public class JRubyEngineTest {
     /**
      * Test of createBindings method, of class Jsr223JRubyEngine.
      */
-    @Test
+    //@Test
     public void testCreateBindings() {
         logger1.info("createBindings");
         ScriptEngineManager manager = new ScriptEngineManager();
@@ -540,7 +533,7 @@ public class JRubyEngineTest {
     /**
      * Test of getContext method, of class Jsr223JRubyEngine.
      */
-    @Test
+    //@Test
     public void testGetContext() {
         logger1.info("getContext");
         ScriptEngineManager manager = new ScriptEngineManager();
@@ -554,7 +547,7 @@ public class JRubyEngineTest {
     /**
      * Test of setContext method, of class Jsr223JRubyEngine.
      */
-    @Test
+    //@Test
     public void testSetContext() {
         logger1.info("setContext");
         ScriptEngineManager manager = new ScriptEngineManager();
@@ -576,7 +569,7 @@ public class JRubyEngineTest {
     /**
      * Test of getFactory method, of class Jsr223JRubyEngine.
      */
-    @Test
+    //@Test
     public void testGetFactory() {
         logger1.info("getFactory");
         ScriptEngineManager manager = new ScriptEngineManager();
@@ -593,11 +586,11 @@ public class JRubyEngineTest {
     /**
      * Test of invokeMethod method, of class Jsr223JRubyEngine.
      */
-    @Test
+    //@Test
     public void testInvokeMethod() throws Exception {
         logger1.info("invokeMethod");
         ScriptEngine instance = newScriptEngine();
-        String filename = basedir + "/src/test/ruby/org/jruby/embed/ruby/tree.rb";
+        String filename = basedir + "/core/src/test/ruby/org/jruby/embed/ruby/tree.rb";
         Reader reader = new FileReader(filename);
         Object receiver = instance.eval(reader);
         String method = "to_s";
@@ -613,7 +606,7 @@ public class JRubyEngineTest {
         bindings.put("color", "nondescript");
         bindings.put("bloomtime", "April - May");
         instance.setBindings(bindings, ScriptContext.ENGINE_SCOPE);
-        filename = basedir + "/src/test/ruby/org/jruby/embed/ruby/tree_given_localvars.rb";
+        filename = basedir + "/core/src/test/ruby/org/jruby/embed/ruby/tree_given_localvars.rb";
         reader = new FileReader(filename);
         receiver = instance.eval(reader);
         expResult = "Cedar is a pyramidal shaped,";
@@ -626,11 +619,11 @@ public class JRubyEngineTest {
     /**
      * Test of invokeFunction method, of class Jsr223JRubyEngine.
      */
-    @Test
+    //@Test
     public void testInvokeFunction() throws Exception {
         logger1.info("invokeFunction");
         ScriptEngine instance = newScriptEngine();
-        String filename = basedir + "/src/test/ruby/org/jruby/embed/ruby/count_down.rb";
+        String filename = basedir + "/core/src/test/ruby/org/jruby/embed/ruby/count_down.rb";
         Reader reader = new FileReader(filename);
         Bindings bindings = new SimpleBindings();
         bindings.put("@month", 6);
@@ -652,12 +645,12 @@ public class JRubyEngineTest {
     /**
      * Test of getInterface method, of class Jsr223JRubyEngine.
      */
-    @Test
+    //@Test
     public void testGetInterface_Class() throws FileNotFoundException, ScriptException {
         logger1.info("getInterface (no receiver)");
         ScriptEngine instance = newScriptEngine();
         Class returnType = RadioActiveDecay.class;
-        String filename = basedir + "/src/test/ruby/org/jruby/embed/ruby/radioactive_decay.rb";
+        String filename = basedir + "/core/src/test/ruby/org/jruby/embed/ruby/radioactive_decay.rb";
         Reader reader = new FileReader(filename);
         Bindings bindings = instance.getBindings(ScriptContext.ENGINE_SCOPE);
         bindings.put("$h", 5715); // half-life of Carbon
@@ -674,11 +667,11 @@ public class JRubyEngineTest {
     /**
      * Test of getInterface method, of class Jsr223JRubyEngine.
      */
-    @Test
+    //@Test
     public void testGetInterface_Object_Class() throws FileNotFoundException, ScriptException {
         logger1.info("getInterface (with receiver)");
         ScriptEngine instance = newScriptEngine();
-        String filename = basedir + "/src/test/ruby/org/jruby/embed/ruby/position_function.rb";
+        String filename = basedir + "/core/src/test/ruby/org/jruby/embed/ruby/position_function.rb";
         Reader reader = new FileReader(filename);
         Bindings bindings = instance.getBindings(ScriptContext.ENGINE_SCOPE);
         bindings.put("initial_velocity", 30.0);
@@ -701,7 +694,7 @@ public class JRubyEngineTest {
     /*
      * Test of ScriptEngine.ARGV, JRUBY-4090
      */
-    @Test
+    //@Test
     public void testARGV() throws ScriptException {
         logger1.info("ScriptEngine.ARGV");
         ScriptEngine instance = newScriptEngine();
@@ -718,7 +711,7 @@ public class JRubyEngineTest {
     /*
      * Test of ScriptEngine.ARGV, JRUBY-4090
      */
-    @Test
+    //@Test
     public void testARGV_2() throws ScriptException {
         logger1.info("ScriptEngine.ARGV before initialization");
         ScriptEngine instance = newScriptEngine();
@@ -763,7 +756,7 @@ public class JRubyEngineTest {
         assertNotSame(obj1, obj2);
     }
 
-    @Test
+    //@Test
     public void testTermination() throws ScriptException {
         logger1.info("Termination Test");
         ScriptEngineManager manager = new ScriptEngineManager();
@@ -783,7 +776,7 @@ public class JRubyEngineTest {
         instance.getContext().setAttribute("org.jruby.embed.termination", false, ScriptContext.ENGINE_SCOPE);
     }
 
-    @Test
+    //@Test
     public void testClearVariables() throws ScriptException {
         logger1.info("Clear Variables Test");
         final ScriptEngine instance = newScriptEngine("singlethread", "global");
@@ -822,4 +815,4 @@ public class JRubyEngineTest {
         }
     }
 
-}
\ No newline at end of file
+}
