@@ -1,17 +1,17 @@
-/***** BEGIN LICENSE BLOCK *****
+/*
+ ***** BEGIN LICENSE BLOCK *****
  * Version: EPL 1.0/GPL 2.0/LGPL 2.1
  *
- * The contents of this file are subject to the Common Public
+ * The contents of this file are subject to the Eclipse Public
  * License Version 1.0 (the "License"); you may not use this file
  * except in compliance with the License. You may obtain a copy of
- * the License at http://www.eclipse.org/legal/cpl-v10.html
+ * the License at http://www.eclipse.org/legal/epl-v10.html
  *
  * Software distributed under the License is distributed on an "AS
  * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
  * implied. See the License for the specific language governing
  * rights and limitations under the License.
  *
- * 
  * Alternatively, the contents of this file may be used under the terms of
  * either of the GNU General Public License Version 2 or later (the "GPL"),
  * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
@@ -24,10 +24,10 @@
  * the provisions above, a recipient may use your version of this file under
  * the terms of any one of the EPL, the GPL or the LGPL.
  ***** END LICENSE BLOCK *****/
-
 package org.jruby.test;
 
 
+import org.jruby.CompatVersion;
 import org.jruby.Ruby;
 import org.jruby.RubyMethod;
 import org.jruby.RubyModule;
@@ -46,7 +46,6 @@ public class TestMethodFactories extends TestRubyBase {
         RubyModule mod = runtime.defineModule("Wombat" + hashCode());
 
         mod.defineAnnotatedMethods(MyBoundClass.class);
-
     }
 
     public void testReflectionMethodFactory() {
@@ -101,4 +100,24 @@ public class TestMethodFactories extends TestRubyBase {
 
         assertEquals(mod.getSingletonClass(), rubyMethod.owner(runtime.getCurrentContext()));
     }
+
+    public static class VersionedMethods {
+        @JRubyMethod(name = "method", compat = CompatVersion.RUBY1_8)
+        public static IRubyObject method18(IRubyObject self) {
+            return self;
+        }
+        @JRubyMethod(name = "method", compat = CompatVersion.RUBY1_9)
+        public static IRubyObject method19(IRubyObject self) {
+            return self;
+        }
+    }
+
+    // #1194: ClassFormatError with Nokogiri 1.6.0
+    public void testVersionedMethods() {
+        RubyModule mod = runtime.defineModule("GH1194");
+
+        mod.defineAnnotatedMethods(VersionedMethods.class);
+
+        assertNotNull(mod.searchMethod("method"));
+    }
 }
