@@ -1,10 +1,11 @@
-/***** BEGIN LICENSE BLOCK *****
+/*
+ ***** BEGIN LICENSE BLOCK *****
  * Version: EPL 1.0/GPL 2.0/LGPL 2.1
  *
- * The contents of this file are subject to the Common Public
+ * The contents of this file are subject to the Eclipse Public
  * License Version 1.0 (the "License"); you may not use this file
  * except in compliance with the License. You may obtain a copy of
- * the License at http://www.eclipse.org/legal/cpl-v10.html
+ * the License at http://www.eclipse.org/legal/epl-v10.html
  *
  * Software distributed under the License is distributed on an "AS
  * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
@@ -29,14 +30,12 @@ package org.jruby.test;
 
 import java.io.ByteArrayOutputStream;
 import java.io.InputStream;
-import java.net.URLClassLoader;
 import java.util.Locale;
 
 import org.jruby.exceptions.MainExitException;
 import org.jruby.Ruby;
 import org.jruby.RubyInstanceConfig;
 import org.jruby.runtime.load.LoadService;
-import org.jruby.util.ClassCache;
 
 /**
  * This should be filled up with more tests for RubyInstanceConfig later
@@ -82,16 +81,6 @@ public class TestRubyInstanceConfig extends TestRubyBase {
         assertEquals(NullLoadService.class, ruby.getLoadService().getClass());
     }
 
-    public void testSettingNewLoaderWillCreateNewClassLoader() throws Exception {
-        ClassLoader beforeCL = config.getLoader();
-        ClassCache beforeCC = config.getClassCache();
-        config.setLoader(beforeCL);
-        assertTrue("setting a new classloader that is the same, should not create a new classcache", beforeCC == config.getClassCache());
-
-        config.setLoader(new URLClassLoader(new java.net.URL[0], beforeCL));
-        assertTrue("setting a new classloader this is different, should create a new classcache", beforeCC != config.getClassCache());
-    }
-
     public void testGetScriptSource() throws Exception {
       config.setCurrentDirectory("uri:classloader:/test_dir");
       config.setScriptFileName("test_script.rb");
