@@ -1,8 +1,8 @@
 # it is war-file
 packaging 'war'
 
-# get jruby dependencies
-properties( 'jruby.version' => '@project.version@',
+# default versions will be overwritten by pom.rb from root directory
+properties( 'jruby.plugins.version' => '1.0.10',
             'project.build.sourceEncoding' => 'utf-8' )
 
 pom( 'org.jruby:jruby', '${jruby.version}' )
@@ -13,19 +13,10 @@ gem 'virtus', '0.5.5'
 repository( :url => 'https://otto.takari.io/content/repositories/rubygems/maven/releases',
             :id => 'rubygems-releases' )
 
-jruby_plugin :gem, :includeRubygemsInResources => true do
+jruby_plugin :gem, :includeRubygemsInResources => true, :jrubyVersion => '9.0.0.0' do
   execute_goal :initialize
-end 
-
-execute 'jrubydir', 'initialize' do |ctx|
-  require 'jruby/commands'
-  JRuby::Commands.generate_dir_info( ctx.project.build.directory.to_pathname + '/rubygems' )
 end
 
-# ruby-maven will dump an equivalent pom.xml
-properties( 'tesla.dump.pom' => 'pom.xml',
-            'jruby.home' => '${basedir}/../../../../../' )
-
 execute 'deploy', :phase => 'pre-integration-test' do |ctx|
   wlp_home = ctx.basedir.to_pathname + '/../../wlp'
   FileUtils.cp( 'target/j2ee_wlp.war', "#{wlp_home}/usr/servers/testing/dropins/packed.war" )
