@@ -8,7 +8,8 @@ def load_build_properties_into_constants
 
     # build const name
     name, value = line.split("=", 2)
-    name.gsub!(".", "_").upcase!
+    name.gsub!(".", "_")
+    name.upcase!
     constant_names << name
     Object.const_set(name.to_sym, value)
   end
@@ -23,9 +24,6 @@ def load_build_properties_into_constants
 end
 load_build_properties_into_constants
 
-# def ant(*args)
-#   raise 'running ant failed!' unless system "ant -logger org.apache.tools.ant.NoBannerLogger #{args.join(' ')}"
-# end
 require 'digest'
 
 class HashTask < Struct.new(:hash, :file)
