@@ -10,23 +10,12 @@ rescue # jarjar renames things, so we try the renamed version
   import "org.jruby.org.objectweb.asm.Opcodes"
 end
 
-class MockMethodVisitor
-  attr_accessor :calls
+describe "SkinnyMethodAdapter" do
+  let(:instance_methods) { SkinnyMethodAdapter.instance_methods.map(&:to_s) }
+  let(:insn_opcodes) do
+    Opcodes.constants.map(&:to_s).select do |c|
+      case c
 
-  def initialize
-    @calls = []
-  end
-
-  def method_missing(name, *args)
-    @calls << [name, args]
-  end
-end
-
-describe "SkinnyMethodAdapter" do  
-  it "supports all JVM opcodes" do
-    insn_opcodes = Opcodes.constants.select do |c|
-      case c.to_s # for 1.9's symbols
-        
       when /ACC_/, # access modifiers
            /V1_/, # version identifiers
            /T_/, # type identifiers
@@ -34,34 +23,29 @@ describe "SkinnyMethodAdapter" do
            /H_/, # method handles
            /ASM/ # ASM version stuff
         false
-        
+
       when "DOUBLE", "FLOAT", "INTEGER", "LONG", "NULL", "TOP", "UNINITIALIZED_THIS"
         false
-        
+
       when "GOTO", "RETURN", "INSTANCEOF", "NEW"
         false
-        
+
       when "INVOKEDYNAMIC_OWNER"
         false
-        
+
       else
         true
       end
-    end
-    
-    # 1.9 makes them symbols, so to_s the lot
-    insn_opcodes = insn_opcodes.map(&:to_s)
-    instance_methods = SkinnyMethodAdapter.instance_methods.map(&:to_s)
-    
-    insn_opcodes.each do |opcode|
-      opcode = opcode.downcase
-      expect(instance_methods).to include(opcode)
-    end
-    
-    expect(instance_methods).to include("go_to")
-    expect(instance_methods).to include("voidreturn")
-    expect(instance_methods).to include("instance_of")
-    expect(instance_methods).to include("newobj")
+    end.map(&:downcase)
+  end
+
+  it "supports all JVM opcodes" do
+    expect(instance_methods).to include(*insn_opcodes)
+    expect(instance_methods).to include(
+      "go_to",
+      "voidreturn",
+      "instance_of",
+      "newobj"
+    )
   end
 end
-
