@@ -15,15 +15,15 @@ describe "A JavaMethod" do
     end
       
     it "should provide a shortcut to invoke the method" do
-      lambda { @method.invoke_static }.should_not raise_error
+      expect { @method.invoke_static }.not_to raise_error
     end
 
     it "should allow invocation with a Ruby nil method" do
-      lambda { @method.invoke nil }.should_not raise_error
+      expect { @method.invoke nil }.not_to raise_error
     end
   
     it "should allow invocation with a Java null method" do
-      lambda { @method.invoke nil.to_java }.should_not raise_error
+      expect { @method.invoke nil.to_java }.not_to raise_error
     end  
   end
   
@@ -34,15 +34,15 @@ describe "A JavaMethod" do
     end
   
     it "should provide a shortcut to invoke protected Java class methods" do
-      lambda { @method.invoke_static }.should_not raise_error
+      expect { @method.invoke_static }.not_to raise_error
     end
 
     it "should allow invocation with a Ruby nil method" do
-      lambda { @method.invoke nil }.should_not raise_error
+      expect { @method.invoke nil }.not_to raise_error
     end
 
     it "should allow invocation with a Java null method" do
-      lambda { @method.invoke nil.to_java }.should_not raise_error
+      expect { @method.invoke nil.to_java }.not_to raise_error
     end
   end
   
@@ -53,15 +53,15 @@ describe "A JavaMethod" do
     end
     
     it "should provide a shortcut to invoke package scope Java class methods" do
-      lambda { @method.invoke_static }.should_not raise_error
+      expect { @method.invoke_static }.not_to raise_error
     end
     
     it "should allow invocation with a Ruby nil method" do
-      lambda { @method.invoke nil }.should_not raise_error
+      expect { @method.invoke nil }.not_to raise_error
     end
 
     it "should allow invocation with a Java null method" do
-      lambda { @method.invoke nil.to_java }.should_not raise_error
+      expect { @method.invoke nil.to_java }.not_to raise_error
     end
   end    
 
@@ -69,41 +69,41 @@ describe "A JavaMethod" do
     o = PrivateInstanceMethod.new
     method = PrivateInstanceMethod.java_class.declared_method_smart :thePrivateMethod
     method.accessible = true
-    lambda { method.invoke(o) }.should_not raise_error
+    expect { method.invoke(o) }.not_to raise_error
   end
   
   it "should provide the ability to invoke protected Java instance methods on a Ruby object" do
     o = ProtectedInstanceMethod.new
     method = ProtectedInstanceMethod.java_class.declared_method_smart :theProtectedMethod
     method.accessible = true
-    lambda { method.invoke(o) }.should_not raise_error
+    expect { method.invoke(o) }.not_to raise_error
   end
   
   it "should provide the ability to invoke package scope Java instance methods on a Ruby object" do
     o = PackageInstanceMethod.new
     method = PackageInstanceMethod.java_class.declared_method_smart :thePackageScopeMethod
     method.accessible = true
-    lambda { method.invoke(o) }.should_not raise_error
+    expect { method.invoke(o) }.not_to raise_error
   end
   
   it "should provide the ability to invoke private Java instance methods on a JavaObject" do
     o = PrivateInstanceMethod.new
     method = PrivateInstanceMethod.java_class.declared_method_smart :thePrivateMethod
     method.accessible = true
-    lambda { method.invoke(o.java_object) }.should_not raise_error
+    expect { method.invoke(o.java_object) }.not_to raise_error
   end
   
   it "should provide the ability to invoke protected Java instance methods on a JavaObject" do
     o = ProtectedInstanceMethod.new
     method = ProtectedInstanceMethod.java_class.declared_method_smart :theProtectedMethod
     method.accessible = true
-    lambda { method.invoke(o.java_object) }.should_not raise_error
+    expect { method.invoke(o.java_object) }.not_to raise_error
   end
   
   it "should provide the ability to invoke package scope Java instance methods on a JavaObject" do
     o = PackageInstanceMethod.new
     method = PackageInstanceMethod.java_class.declared_method_smart :thePackageScopeMethod
     method.accessible = true
-    lambda { method.invoke(o.java_object) }.should_not raise_error
+    expect { method.invoke(o.java_object) }.not_to raise_error
   end
 end
