@@ -12,100 +12,100 @@ describe "A Java primitive Array of type" do
   describe "boolean" do 
     it "should be possible to create empty array" do 
       arr = Java::boolean[0].new
-      arr.java_class.to_s.should == "[Z"
+      expect(arr.java_class.to_s).to eq("[Z")
     end
     
     it "should be possible to create uninitialized single dimensional array" do 
       arr = Java::boolean[10].new
-      arr.java_class.to_s.should == "[Z"
+      expect(arr.java_class.to_s).to eq("[Z")
     end
     
     it "should be possible to create uninitialized multi dimensional array" do 
       arr = Java::boolean[10,10].new
-      arr.java_class.to_s.should == "[[Z"
+      expect(arr.java_class.to_s).to eq("[[Z")
     end
 
     it "should be possible to create primitive array from Ruby array" do 
       # Check with symbol name
       arr = [true, false].to_java :boolean
-      arr.java_class.to_s.should == "[Z"
+      expect(arr.java_class.to_s).to eq("[Z")
 
-      arr.length.should == 2
+      expect(arr.length).to eq(2)
 
-      arr[0].should be_true
-      arr[1].should be_false
+      expect(arr[0]).to be true
+      expect(arr[1]).to be_falsey
 
 
       # Check with type
       arr = [true, false].to_java Java::boolean
-      arr.java_class.to_s.should == "[Z"
+      expect(arr.java_class.to_s).to eq("[Z")
 
-      arr.length.should == 2
+      expect(arr.length).to eq(2)
 
-      arr[0].should be_true
-      arr[1].should be_false
+      expect(arr[0]).to be true
+      expect(arr[1]).to be_falsey
     end
     
     it "should be possible to set values in primitive array" do 
       arr = Java::boolean[5].new
       arr[3] = true
       
-      arr[0].should be_false
-      arr[1].should be_false
-      arr[2].should be_false
-      arr[3].should be_true
-      arr[4].should be_false
+      expect(arr[0]).to be_falsey
+      expect(arr[1]).to be_falsey
+      expect(arr[2]).to be_falsey
+      expect(arr[3]).to be true
+      expect(arr[4]).to be_falsey
     end
 
     it "should be possible to get values from primitive array" do 
       arr = [false, true, false].to_java :boolean
-      arr[0].should be_false
-      arr[1].should be_true
-      arr[2].should be_false
+      expect(arr[0]).to be_falsey
+      expect(arr[1]).to be true
+      expect(arr[2]).to be_falsey
     end
 
     it "should be possible to call methods that take primitive array" do 
       arr = [false, true, false].to_java :boolean
       ret = ArrayReceiver::call_with_boolean(arr)
-      ret.to_a.should == [false, true, false]
+      expect(ret.to_a).to eq([false, true, false])
     end
   end
 
   describe "byte" do 
     it "should be possible to create empty array" do 
       arr = Java::byte[0].new
-      arr.java_class.to_s.should == "[B"
+      expect(arr.java_class.to_s).to eq("[B")
     end
     
     it "should be possible to create uninitialized single dimensional array" do 
       arr = Java::byte[10].new
-      arr.java_class.to_s.should == "[B"
+      expect(arr.java_class.to_s).to eq("[B")
     end
     
     it "should be possible to create uninitialized multi dimensional array" do 
       arr = Java::byte[10,10].new
-      arr.java_class.to_s.should == "[[B"
+      expect(arr.java_class.to_s).to eq("[[B")
     end
 
     it "should be possible to create primitive array from Ruby array" do 
       # Check with symbol name
       arr = [1,2].to_java :byte
-      arr.java_class.to_s.should == "[B"
+      expect(arr.java_class.to_s).to eq("[B")
 
-      arr.length.should == 2
+      expect(arr.length).to eq(2)
 
-      arr[0].should == 1
-      arr[1].should == 2
+      expect(arr[0]).to eq(1)
+      expect(arr[1]).to eq(2)
 
 
       # Check with type
       arr = [1,2].to_java Java::byte
-      arr.java_class.to_s.should == "[B"
+      expect(arr.java_class.to_s).to eq("[B")
 
-      arr.length.should == 2
+      expect(arr.length).to eq(2)
 
-      arr[0].should == 1
-      arr[1].should == 2
+      expect(arr[0]).to eq(1)
+      expect(arr[1]).to eq(2)
     end
     
     it "should be possible to set values in primitive array" do 
@@ -114,62 +114,62 @@ describe "A Java primitive Array of type" do
       arr[1] = 20
       arr[2] = 42
       
-      arr[0].should == 12
-      arr[1].should == 20
-      arr[2].should == 42
-      arr[3].should == 0
-      arr[4].should == 0
+      expect(arr[0]).to eq(12)
+      expect(arr[1]).to eq(20)
+      expect(arr[2]).to eq(42)
+      expect(arr[3]).to eq(0)
+      expect(arr[4]).to eq(0)
     end
 
     it "should be possible to get values from primitive array" do 
       arr = [13, 42, 120].to_java :byte
-      arr[0].should == 13
-      arr[1].should == 42
-      arr[2].should == 120
+      expect(arr[0]).to eq(13)
+      expect(arr[1]).to eq(42)
+      expect(arr[2]).to eq(120)
     end
 
     it "should be possible to call methods that take primitive array" do 
       arr = [13, 42, 120].to_java :byte
       ret = ArrayReceiver::call_with_byte(arr)
-      ret.to_a.should == [13, 42, 120]
+      expect(ret.to_a).to eq([13, 42, 120])
     end
   end
 
   describe "char" do 
     it "should be possible to create empty array" do 
       arr = Java::char[0].new
-      arr.java_class.to_s.should == "[C"
+      expect(arr.java_class.to_s).to eq("[C")
     end
     
     it "should be possible to create uninitialized single dimensional array" do 
       arr = Java::char[10].new
-      arr.java_class.to_s.should == "[C"
+      expect(arr.java_class.to_s).to eq("[C")
     end
     
     it "should be possible to create uninitialized multi dimensional array" do 
       arr = Java::char[10,10].new
-      arr.java_class.to_s.should == "[[C"
+      expect(arr.java_class.to_s).to eq("[[C")
     end
 
     it "should be possible to create primitive array from Ruby array" do 
       # Check with symbol name
       arr = [1,2].to_java :char
-      arr.java_class.to_s.should == "[C"
+      expect(arr.java_class.to_s).to eq("[C")
 
-      arr.length.should == 2
+      expect(arr.length).to eq(2)
 
-      arr[0].should == 1
-      arr[1].should == 2
+      expect(arr[0]).to eq(1)
+      expect(arr[1]).to eq(2)
 
 
       # Check with type
       arr = [1,2].to_java Java::char
-      arr.java_class.to_s.should == "[C"
+      expect(arr.java_class.to_s).to eq("[C")
 
-      arr.length.should == 2
+      expect(arr.length).to eq(2)
 
-      arr[0].should == 1
-      arr[1].should == 2
+      expect(arr[0]).to eq(1)
+      expect(arr[1]).to eq(2)
     end
     
     it "should be possible to set values in primitive array" do 
@@ -178,62 +178,62 @@ describe "A Java primitive Array of type" do
       arr[1] = 20
       arr[2] = 42
       
-      arr[0].should == 12
-      arr[1].should == 20
-      arr[2].should == 42
-      arr[3].should == 0
-      arr[4].should == 0
+      expect(arr[0]).to eq(12)
+      expect(arr[1]).to eq(20)
+      expect(arr[2]).to eq(42)
+      expect(arr[3]).to eq(0)
+      expect(arr[4]).to eq(0)
     end
 
     it "should be possible to get values from primitive array" do 
       arr = [13, 42, 120].to_java :char
-      arr[0].should == 13
-      arr[1].should == 42
-      arr[2].should == 120
+      expect(arr[0]).to eq(13)
+      expect(arr[1]).to eq(42)
+      expect(arr[2]).to eq(120)
     end
 
     it "should be possible to call methods that take primitive array" do 
       arr = [13, 42, 120].to_java :char
       ret = ArrayReceiver::call_with_char(arr)
-      ret.to_a.should == [13, 42, 120]
+      expect(ret.to_a).to eq([13, 42, 120])
     end
   end
 
   describe "double" do 
     it "should be possible to create empty array" do 
       arr = Java::double[0].new
-      arr.java_class.to_s.should == "[D"
+      expect(arr.java_class.to_s).to eq("[D")
     end
     
     it "should be possible to create uninitialized single dimensional array" do 
       arr = Java::double[10].new
-      arr.java_class.to_s.should == "[D"
+      expect(arr.java_class.to_s).to eq("[D")
     end
     
     it "should be possible to create uninitialized multi dimensional array" do 
       arr = Java::double[10,10].new
-      arr.java_class.to_s.should == "[[D"
+      expect(arr.java_class.to_s).to eq("[[D")
     end
 
     it "should be possible to create primitive array from Ruby array" do 
       # Check with symbol name
       arr = [1.2,2.3].to_java :double
-      arr.java_class.to_s.should == "[D"
+      expect(arr.java_class.to_s).to eq("[D")
 
-      arr.length.should == 2
+      expect(arr.length).to eq(2)
 
-      arr[0].should == 1.2
-      arr[1].should == 2.3
+      expect(arr[0]).to eq(1.2)
+      expect(arr[1]).to eq(2.3)
 
 
       # Check with type
       arr = [1.2,2.3].to_java Java::double
-      arr.java_class.to_s.should == "[D"
+      expect(arr.java_class.to_s).to eq("[D")
 
-      arr.length.should == 2
+      expect(arr.length).to eq(2)
 
-      arr[0].should == 1.2
-      arr[1].should == 2.3
+      expect(arr[0]).to eq(1.2)
+      expect(arr[1]).to eq(2.3)
     end
     
     it "should be possible to set values in primitive array" do 
@@ -242,62 +242,62 @@ describe "A Java primitive Array of type" do
       arr[1] = 20.3
       arr[2] = 42.4
       
-      arr[0].should == 12.2
-      arr[1].should == 20.3
-      arr[2].should == 42.4
-      arr[3].should == 0.0
-      arr[4].should == 0.0
+      expect(arr[0]).to eq(12.2)
+      expect(arr[1]).to eq(20.3)
+      expect(arr[2]).to eq(42.4)
+      expect(arr[3]).to eq(0.0)
+      expect(arr[4]).to eq(0.0)
     end
 
     it "should be possible to get values from primitive array" do 
       arr = [13.2, 42.3, 120.4].to_java :double
-      arr[0].should == 13.2
-      arr[1].should == 42.3
-      arr[2].should == 120.4
+      expect(arr[0]).to eq(13.2)
+      expect(arr[1]).to eq(42.3)
+      expect(arr[2]).to eq(120.4)
     end
 
     it "should be possible to call methods that take primitive array" do 
       arr = [13.2, 42.3, 120.4].to_java :double
       ret = ArrayReceiver::call_with_double(arr)
-      ret.to_a.should == [13.2, 42.3, 120.4]
+      expect(ret.to_a).to eq([13.2, 42.3, 120.4])
     end
   end
 
   describe "float" do 
     it "should be possible to create empty array" do 
       arr = Java::float[0].new
-      arr.java_class.to_s.should == "[F"
+      expect(arr.java_class.to_s).to eq("[F")
     end
     
     it "should be possible to create uninitialized single dimensional array" do 
       arr = Java::float[10].new
-      arr.java_class.to_s.should == "[F"
+      expect(arr.java_class.to_s).to eq("[F")
     end
     
     it "should be possible to create uninitialized multi dimensional array" do 
       arr = Java::float[10,10].new
-      arr.java_class.to_s.should == "[[F"
+      expect(arr.java_class.to_s).to eq("[[F")
     end
 
     it "should be possible to create primitive array from Ruby array" do 
       # Check with symbol name
       arr = [1.2,2.3].to_java :float
-      arr.java_class.to_s.should == "[F"
+      expect(arr.java_class.to_s).to eq("[F")
 
-      arr.length.should == 2
+      expect(arr.length).to eq(2)
 
-      arr[0].should be_within(0.00001).of(1.2)
-      arr[1].should be_within(0.00001).of(2.3)
+      expect(arr[0]).to be_within(0.00001).of(1.2)
+      expect(arr[1]).to be_within(0.00001).of(2.3)
 
 
       # Check with type
       arr = [1.2,2.3].to_java Java::float
-      arr.java_class.to_s.should == "[F"
+      expect(arr.java_class.to_s).to eq("[F")
 
-      arr.length.should == 2
+      expect(arr.length).to eq(2)
 
-      arr[0].should be_within(0.00001).of(1.2)
-      arr[1].should be_within(0.00001).of(2.3)
+      expect(arr[0]).to be_within(0.00001).of(1.2)
+      expect(arr[1]).to be_within(0.00001).of(2.3)
     end
     
     it "should be possible to set values in primitive array" do 
@@ -306,66 +306,66 @@ describe "A Java primitive Array of type" do
       arr[1] = 20.3
       arr[2] = 42.4
       
-      arr[0].should be_within(0.00001).of(12.2)
-      arr[1].should be_within(0.00001).of(20.3)
-      arr[2].should be_within(0.00001).of(42.4)
-      arr[3].should == 0.0
-      arr[4].should == 0.0
+      expect(arr[0]).to be_within(0.00001).of(12.2)
+      expect(arr[1]).to be_within(0.00001).of(20.3)
+      expect(arr[2]).to be_within(0.00001).of(42.4)
+      expect(arr[3]).to eq(0.0)
+      expect(arr[4]).to eq(0.0)
     end
 
     it "should be possible to get values from primitive array" do 
       arr = [13.2, 42.3, 120.4].to_java :float
 
-      arr[0].should be_within(0.00001).of(13.2)
-      arr[1].should be_within(0.00001).of(42.3)
-      arr[2].should be_within(0.00001).of(120.4)
+      expect(arr[0]).to be_within(0.00001).of(13.2)
+      expect(arr[1]).to be_within(0.00001).of(42.3)
+      expect(arr[2]).to be_within(0.00001).of(120.4)
     end
 
     it "should be possible to call methods that take primitive array" do 
       arr = [13.2, 42.3, 120.4].to_java :float
       ret = ArrayReceiver::call_with_float(arr)
-      ret.length.should == 3
-      ret[0].should be_within(0.00001).of(13.2)
-      ret[1].should be_within(0.00001).of(42.3)
-      ret[2].should be_within(0.00001).of(120.4)
+      expect(ret.length).to eq(3)
+      expect(ret[0]).to be_within(0.00001).of(13.2)
+      expect(ret[1]).to be_within(0.00001).of(42.3)
+      expect(ret[2]).to be_within(0.00001).of(120.4)
     end
   end
 
   describe "int" do 
     it "should be possible to create empty array" do 
       arr = Java::int[0].new
-      arr.java_class.to_s.should == "[I"
+      expect(arr.java_class.to_s).to eq("[I")
     end
     
     it "should be possible to create uninitialized single dimensional array" do 
       arr = Java::int[10].new
-      arr.java_class.to_s.should == "[I"
+      expect(arr.java_class.to_s).to eq("[I")
     end
     
     it "should be possible to create uninitialized multi dimensional array" do 
       arr = Java::int[10,10].new
-      arr.java_class.to_s.should == "[[I"
+      expect(arr.java_class.to_s).to eq("[[I")
     end
 
     it "should be possible to create primitive array from Ruby array" do 
       # Check with symbol name
       arr = [1,2].to_java :int
-      arr.java_class.to_s.should == "[I"
+      expect(arr.java_class.to_s).to eq("[I")
 
-      arr.length.should == 2
+      expect(arr.length).to eq(2)
 
-      arr[0].should == 1
-      arr[1].should == 2
+      expect(arr[0]).to eq(1)
+      expect(arr[1]).to eq(2)
 
 
       # Check with type
       arr = [1,2].to_java Java::int
-      arr.java_class.to_s.should == "[I"
+      expect(arr.java_class.to_s).to eq("[I")
 
-      arr.length.should == 2
+      expect(arr.length).to eq(2)
 
-      arr[0].should == 1
-      arr[1].should == 2
+      expect(arr[0]).to eq(1)
+      expect(arr[1]).to eq(2)
     end
     
     it "should be possible to set values in primitive array" do 
@@ -374,62 +374,62 @@ describe "A Java primitive Array of type" do
       arr[1] = 20
       arr[2] = 42
       
-      arr[0].should == 12
-      arr[1].should == 20
-      arr[2].should == 42
-      arr[3].should == 0
-      arr[4].should == 0
+      expect(arr[0]).to eq(12)
+      expect(arr[1]).to eq(20)
+      expect(arr[2]).to eq(42)
+      expect(arr[3]).to eq(0)
+      expect(arr[4]).to eq(0)
     end
 
     it "should be possible to get values from primitive array" do 
       arr = [13, 42, 120].to_java :int
-      arr[0].should == 13
-      arr[1].should == 42
-      arr[2].should == 120
+      expect(arr[0]).to eq(13)
+      expect(arr[1]).to eq(42)
+      expect(arr[2]).to eq(120)
     end
 
     it "should be possible to call methods that take primitive array" do 
       arr = [13, 42, 120].to_java :int
       ret = ArrayReceiver::call_with_int(arr)
-      ret.to_a.should == [13, 42, 120]
+      expect(ret.to_a).to eq([13, 42, 120])
     end
   end
 
   describe "long" do 
     it "should be possible to create empty array" do 
       arr = Java::long[0].new
-      arr.java_class.to_s.should == "[J"
+      expect(arr.java_class.to_s).to eq("[J")
     end
     
     it "should be possible to create uninitialized single dimensional array" do 
       arr = Java::long[10].new
-      arr.java_class.to_s.should == "[J"
+      expect(arr.java_class.to_s).to eq("[J")
     end
     
     it "should be possible to create uninitialized multi dimensional array" do 
       arr = Java::long[10,10].new
-      arr.java_class.to_s.should == "[[J"
+      expect(arr.java_class.to_s).to eq("[[J")
     end
 
     it "should be possible to create primitive array from Ruby array" do 
       # Check with symbol name
       arr = [1,2].to_java :long
-      arr.java_class.to_s.should == "[J"
+      expect(arr.java_class.to_s).to eq("[J")
 
-      arr.length.should == 2
+      expect(arr.length).to eq(2)
 
-      arr[0].should == 1
-      arr[1].should == 2
+      expect(arr[0]).to eq(1)
+      expect(arr[1]).to eq(2)
 
 
       # Check with type
       arr = [1,2].to_java Java::long
-      arr.java_class.to_s.should == "[J"
+      expect(arr.java_class.to_s).to eq("[J")
 
-      arr.length.should == 2
+      expect(arr.length).to eq(2)
 
-      arr[0].should == 1
-      arr[1].should == 2
+      expect(arr[0]).to eq(1)
+      expect(arr[1]).to eq(2)
     end
     
     it "should be possible to set values in primitive array" do 
@@ -438,62 +438,62 @@ describe "A Java primitive Array of type" do
       arr[1] = 20
       arr[2] = 42
       
-      arr[0].should == 12
-      arr[1].should == 20
-      arr[2].should == 42
-      arr[3].should == 0
-      arr[4].should == 0
+      expect(arr[0]).to eq(12)
+      expect(arr[1]).to eq(20)
+      expect(arr[2]).to eq(42)
+      expect(arr[3]).to eq(0)
+      expect(arr[4]).to eq(0)
     end
 
     it "should be possible to get values from primitive array" do 
       arr = [13, 42, 120].to_java :long
-      arr[0].should == 13
-      arr[1].should == 42
-      arr[2].should == 120
+      expect(arr[0]).to eq(13)
+      expect(arr[1]).to eq(42)
+      expect(arr[2]).to eq(120)
     end
 
     it "should be possible to call methods that take primitive array" do 
       arr = [13, 42, 120].to_java :long
       ret = ArrayReceiver::call_with_long(arr)
-      ret.to_a.should == [13, 42, 120]
+      expect(ret.to_a).to eq([13, 42, 120])
     end
   end
 
   describe "short" do 
     it "should be possible to create empty array" do 
       arr = Java::short[0].new
-      arr.java_class.to_s.should == "[S"
+      expect(arr.java_class.to_s).to eq("[S")
     end
     
     it "should be possible to create uninitialized single dimensional array" do 
       arr = Java::short[10].new
-      arr.java_class.to_s.should == "[S"
+      expect(arr.java_class.to_s).to eq("[S")
     end
     
     it "should be possible to create uninitialized multi dimensional array" do 
       arr = Java::short[10,10].new
-      arr.java_class.to_s.should == "[[S"
+      expect(arr.java_class.to_s).to eq("[[S")
     end
 
     it "should be possible to create primitive array from Ruby array" do 
       # Check with symbol name
       arr = [1,2].to_java :short
-      arr.java_class.to_s.should == "[S"
+      expect(arr.java_class.to_s).to eq("[S")
 
-      arr.length.should == 2
+      expect(arr.length).to eq(2)
 
-      arr[0].should == 1
-      arr[1].should == 2
+      expect(arr[0]).to eq(1)
+      expect(arr[1]).to eq(2)
 
 
       # Check with type
       arr = [1,2].to_java Java::short
-      arr.java_class.to_s.should == "[S"
+      expect(arr.java_class.to_s).to eq("[S")
 
-      arr.length.should == 2
+      expect(arr.length).to eq(2)
 
-      arr[0].should == 1
-      arr[1].should == 2
+      expect(arr[0]).to eq(1)
+      expect(arr[1]).to eq(2)
     end
     
     it "should be possible to set values in primitive array" do 
@@ -502,62 +502,62 @@ describe "A Java primitive Array of type" do
       arr[1] = 20
       arr[2] = 42
       
-      arr[0].should == 12
-      arr[1].should == 20
-      arr[2].should == 42
-      arr[3].should == 0
-      arr[4].should == 0
+      expect(arr[0]).to eq(12)
+      expect(arr[1]).to eq(20)
+      expect(arr[2]).to eq(42)
+      expect(arr[3]).to eq(0)
+      expect(arr[4]).to eq(0)
     end
 
     it "should be possible to get values from primitive array" do 
       arr = [13, 42, 120].to_java :short
-      arr[0].should == 13
-      arr[1].should == 42
-      arr[2].should == 120
+      expect(arr[0]).to eq(13)
+      expect(arr[1]).to eq(42)
+      expect(arr[2]).to eq(120)
     end
 
     it "should be possible to call methods that take primitive array" do 
       arr = [13, 42, 120].to_java :short
       ret = ArrayReceiver::call_with_short(arr)
-      ret.to_a.should == [13, 42, 120]
+      expect(ret.to_a).to eq([13, 42, 120])
     end
   end
 
   describe "string" do 
     it "should be possible to create empty array" do 
       arr = java.lang.String[0].new
-      arr.java_class.to_s.should == "[Ljava.lang.String;"
+      expect(arr.java_class.to_s).to eq("[Ljava.lang.String;")
     end
     
     it "should be possible to create uninitialized single dimensional array" do 
       arr = java.lang.String[10].new
-      arr.java_class.to_s.should == "[Ljava.lang.String;"
+      expect(arr.java_class.to_s).to eq("[Ljava.lang.String;")
     end
     
     it "should be possible to create uninitialized multi dimensional array" do 
       arr = java.lang.String[10,10].new
-      arr.java_class.to_s.should == "[[Ljava.lang.String;"
+      expect(arr.java_class.to_s).to eq("[[Ljava.lang.String;")
     end
 
     it "should be possible to create primitive array from Ruby array" do 
       # Check with symbol name
       arr = ["foo","bar"].to_java :string
-      arr.java_class.to_s.should == "[Ljava.lang.String;"
+      expect(arr.java_class.to_s).to eq("[Ljava.lang.String;")
 
-      arr.length.should == 2
+      expect(arr.length).to eq(2)
 
-      arr[0].should == "foo"
-      arr[1].should == "bar"
+      expect(arr[0]).to eq("foo")
+      expect(arr[1]).to eq("bar")
 
 
       # Check with type
       arr = ["foo","bar"].to_java java.lang.String
-      arr.java_class.to_s.should == "[Ljava.lang.String;"
+      expect(arr.java_class.to_s).to eq("[Ljava.lang.String;")
 
-      arr.length.should == 2
+      expect(arr.length).to eq(2)
 
-      arr[0].should == "foo"
-      arr[1].should == "bar"
+      expect(arr[0]).to eq("foo")
+      expect(arr[1]).to eq("bar")
     end
     
     it "should be possible to set values in primitive array" do 
@@ -566,41 +566,41 @@ describe "A Java primitive Array of type" do
       arr[1] = "20"
       arr[2] = "42"
       
-      arr[0].should == "12"
-      arr[1].should == "20"
-      arr[2].should == "42"
-      arr[3].should be_nil
-      arr[4].should be_nil
+      expect(arr[0]).to eq("12")
+      expect(arr[1]).to eq("20")
+      expect(arr[2]).to eq("42")
+      expect(arr[3]).to be_nil
+      expect(arr[4]).to be_nil
     end
 
     it "should be possible to get values from primitive array" do 
       arr = ["flurg", "glax", "morg"].to_java :string
-      arr[0].should == "flurg"
-      arr[1].should == "glax"
-      arr[2].should == "morg"
+      expect(arr[0]).to eq("flurg")
+      expect(arr[1]).to eq("glax")
+      expect(arr[2]).to eq("morg")
     end
 
     it "should be possible to call methods that take primitive array" do 
       arr = ["flurg", "glax", "morg"].to_java :string
       ret = ArrayReceiver::call_with_string(arr)
-      ret.to_a.should == ["flurg", "glax", "morg"]
+      expect(ret.to_a).to eq(["flurg", "glax", "morg"])
     end
   end
 
   describe "Object ref" do 
     it "should be possible to create empty array" do 
       arr = java.util.HashMap[0].new
-      arr.java_class.to_s.should == "[Ljava.util.HashMap;"
+      expect(arr.java_class.to_s).to eq("[Ljava.util.HashMap;")
     end
     
     it "should be possible to create uninitialized single dimensional array" do 
       arr = java.util.HashMap[10].new
-      arr.java_class.to_s.should == "[Ljava.util.HashMap;"
+      expect(arr.java_class.to_s).to eq("[Ljava.util.HashMap;")
     end
     
     it "should be possible to create uninitialized multi dimensional array" do 
       arr = java.util.HashMap[10,10].new
-      arr.java_class.to_s.should == "[[Ljava.util.HashMap;"
+      expect(arr.java_class.to_s).to eq("[[Ljava.util.HashMap;")
     end
 
     it "should be possible to create primitive array from Ruby array" do
@@ -611,12 +611,12 @@ describe "A Java primitive Array of type" do
       h2["max"] = "foo"
 
       arr = [h1, h2].to_java java.util.HashMap
-      arr.java_class.to_s.should == "[Ljava.util.HashMap;"
+      expect(arr.java_class.to_s).to eq("[Ljava.util.HashMap;")
 
-      arr.length.should == 2
+      expect(arr.length).to eq(2)
 
-      arr[0].should == h1
-      arr[1].should == h2
+      expect(arr[0]).to eq(h1)
+      expect(arr[1]).to eq(h2)
     end
     
     it "should be possible to set values in primitive array" do 
@@ -634,11 +634,11 @@ describe "A Java primitive Array of type" do
       arr[1] = h2
       arr[2] = h3
       
-      arr[0].should == h1
-      arr[1].should == h2
-      arr[2].should == h3
-      arr[3].should be_nil
-      arr[4].should be_nil
+      expect(arr[0]).to eq(h1)
+      expect(arr[1]).to eq(h2)
+      expect(arr[2]).to eq(h3)
+      expect(arr[3]).to be_nil
+      expect(arr[4]).to be_nil
     end
 
     it "should be possible to get values from primitive array" do
@@ -652,9 +652,9 @@ describe "A Java primitive Array of type" do
       h3["flix"] = "mux"
 
       arr = [h1, h2, h3].to_java java.util.HashMap
-      arr[0].should == h1
-      arr[1].should == h2
-      arr[2].should == h3
+      expect(arr[0]).to eq(h1)
+      expect(arr[1]).to eq(h2)
+      expect(arr[2]).to eq(h3)
     end
 
     it "should be possible to call methods that take primitive array" do
@@ -669,24 +669,24 @@ describe "A Java primitive Array of type" do
 
       arr = [h1, h2, h3].to_java java.util.HashMap
       ret = ArrayReceiver::call_with_object(arr)
-      ret.to_a.should == [h1, h2, h3]
+      expect(ret.to_a).to eq([h1, h2, h3])
     end
   end
 
   describe "Class ref" do 
     it "should be possible to create empty array" do 
       arr = java.lang.Class[0].new
-      arr.java_class.to_s.should == "[Ljava.lang.Class;"
+      expect(arr.java_class.to_s).to eq("[Ljava.lang.Class;")
     end
     
     it "should be possible to create uninitialized single dimensional array" do 
       arr = java.lang.Class[10].new
-      arr.java_class.to_s.should == "[Ljava.lang.Class;"
+      expect(arr.java_class.to_s).to eq("[Ljava.lang.Class;")
     end
     
     it "should be possible to create uninitialized multi dimensional array" do 
       arr = java.lang.Class[10,10].new
-      arr.java_class.to_s.should == "[[Ljava.lang.Class;"
+      expect(arr.java_class.to_s).to eq("[[Ljava.lang.Class;")
     end
 
     it "should be possible to create primitive array from Ruby array" do
@@ -694,12 +694,12 @@ describe "A Java primitive Array of type" do
         h2 = java.util.HashMap.java_class
 
         arr = [h1, h2].to_java java.lang.Class
-        arr.java_class.to_s.should == "[Ljava.lang.Class;"
+        expect(arr.java_class.to_s).to eq("[Ljava.lang.Class;")
 
-        arr.length.should == 2
+        expect(arr.length).to eq(2)
 
-        arr[0].should == h1
-        arr[1].should == h2
+        expect(arr[0]).to eq(h1)
+        expect(arr[1]).to eq(h2)
     end
     
     it "should be possible to set values in primitive array" do 
@@ -712,11 +712,11 @@ describe "A Java primitive Array of type" do
         arr[1] = h2
         arr[2] = h3
         
-        arr[0].should == h1
-        arr[1].should == h2
-        arr[2].should == h3
-        arr[3].should be_nil
-        arr[4].should be_nil
+        expect(arr[0]).to eq(h1)
+        expect(arr[1]).to eq(h2)
+        expect(arr[2]).to eq(h3)
+        expect(arr[3]).to be_nil
+        expect(arr[4]).to be_nil
     end
 
     it "should be possible to get values from primitive array" do
@@ -725,9 +725,9 @@ describe "A Java primitive Array of type" do
         h3 = java.lang.ref.SoftReference.java_class
 
         arr = [h1, h2, h3].to_java java.lang.Class
-        arr[0].should == h1
-        arr[1].should == h2
-        arr[2].should == h3
+        expect(arr[0]).to eq(h1)
+        expect(arr[1]).to eq(h2)
+        expect(arr[2]).to eq(h3)
     end
 
     it "should be possible to call methods that take primitive array" do
@@ -737,7 +737,7 @@ describe "A Java primitive Array of type" do
 
         arr = [h1, h2, h3].to_java java.lang.Class
         ret = ArrayReceiver::call_with_object(arr)
-        ret.to_a.should == [h1, h2, h3]
+        expect(ret.to_a).to eq([h1, h2, h3])
     end
   end
 end
@@ -746,13 +746,13 @@ describe "A Ruby array with a nil element" do
   it "can be coerced to an array of objects" do
     ary = [nil]
     result = ary.to_java java.lang.Runnable
-    result[0].should be_nil
+    expect(result[0]).to be_nil
   end
 
   it "can be coerced to an array of classes" do
     ary = [nil]
     result = ary.to_java java.lang.Class
-    result[0].should be_nil
+    expect(result[0]).to be_nil
   end
 end
 
@@ -760,18 +760,18 @@ describe "A multi-dimensional Ruby array" do
   it "can be coerced to a multi-dimensional Java array" do
     ary = [[1,2],[3,4],[5,6],[7,8],[9,0]]
     java_ary = ary.to_java(Java::long[])
-    java_ary.class.should == Java::long[][]
-    java_ary[0].class.should == Java::long[]
+    expect(java_ary.class).to eq(Java::long[][])
+    expect(java_ary[0].class).to eq(Java::long[])
 
     java_ary = ary.to_java(Java::double[])
-    java_ary.class.should == Java::double[][]
-    java_ary[0].class.should == Java::double[]
+    expect(java_ary.class).to eq(Java::double[][])
+    expect(java_ary[0].class).to eq(Java::double[])
 
     ary = [[[1]]]
     java_ary = ary.to_java(Java::long[][])
-    java_ary.class.should == Java::long[][][]
-    java_ary[0].class.should == Java::long[][]
-    java_ary[0][0].class.should == Java::long[]
+    expect(java_ary.class).to eq(Java::long[][][])
+    expect(java_ary[0].class).to eq(Java::long[][])
+    expect(java_ary[0][0].class).to eq(Java::long[])
   end
 end
 
@@ -788,9 +788,9 @@ describe "A Ruby class implementing an interface returning a Java Object[]" do
         return a.to_java
       end
     end
-    ArrayReturningInterfaceConsumer.new.eat(Bar.new).should_not == nil
-    ArrayReturningInterfaceConsumer.new.eat(Bar.new).java_object.class.name.should == 'Java::JavaArray'
-    ArrayReturningInterfaceConsumer.new.eat(Bar.new).java_object.class.should == Java::JavaArray
+    expect(ArrayReturningInterfaceConsumer.new.eat(Bar.new)).not_to eq(nil)
+    expect(ArrayReturningInterfaceConsumer.new.eat(Bar.new).java_object.class.name).to eq('Java::JavaArray')
+    expect(ArrayReturningInterfaceConsumer.new.eat(Bar.new).java_object.class).to eq(Java::JavaArray)
   end
 end
 
@@ -802,7 +802,11 @@ describe "A Ruby class extending a Java class" do
       end
     end
 
-    lambda {cls.new}.should raise_error
+    begin
+      cls.new
+    rescue TypeError => e
+      expect( e.message ).to match /class .*PrivateConstructor doesn't have .* constructor/
+    end
   end
 
   it "should fail when constructing through package superclass constructor" do
@@ -812,7 +816,7 @@ describe "A Ruby class extending a Java class" do
       end
     end
 
-    lambda {cls.new}.should raise_error
+    expect {cls.new}.to raise_error(TypeError)
   end
 
   it "should succeed when constructing through public superclass constructor" do
@@ -822,7 +826,7 @@ describe "A Ruby class extending a Java class" do
       end
     end
 
-    lambda {cls.new}.should_not raise_error
+    expect {cls.new}.not_to raise_error
   end
 
   it "should succeed when constructing through protected superclass constructor" do
@@ -832,7 +836,39 @@ describe "A Ruby class extending a Java class" do
       end
     end
 
-    lambda {cls.new}.should_not raise_error
+    expect {cls.new}.not_to raise_error
+  end
+
+  it 'resolves expected constructor' do
+    cls = Class.new(java.lang.Thread) # plenty of constructors
+
+    expect { cls.new }.not_to raise_error # Thread()
+    expect { cls.new('test') }.not_to raise_error # Thread(String)
+
+    cls = Class.new(java.io.CharArrayWriter) {} # has only 2 constructors
+    expect { cls.new }.not_to raise_error # ()
+    expect { cls.new(42) }.not_to raise_error # (int initialSize)
+    expect { cls.new('xxx') }.to raise_error(ArgumentError) # wrong number of arguments for constructor
+  end
+
+  class RubyFilterWriter < java.io.FilterWriter
+    # protected FilterWriter(Writer out)
+    def flush; end
+  end
+
+  it 'resolves expected constructor (1)' do # optimized _case_
+    writer = java.io.CharArrayWriter.new
+    # protected FilterWriter(Writer out)
+    expect { RubyFilterWriter.new(writer) }.not_to raise_error
+    expect { RubyFilterWriter.new }.to raise_error(ArgumentError)
+  end
+
+  it 'resolves expected constructor (0)' do # optimized _case_
+    klass = Class.new(java.util.concurrent.RecursiveTask) do
+      # public RecursiveTask()
+      def compute; return 42 end
+    end
+    expect { klass.new }.not_to raise_error
   end
 
 end
