@@ -1,21 +1,19 @@
-require 'zlib'
+describe 'Zlib::GzipWriter' do
 
-class IoLikeObject
-  attr_reader :body
+  before(:all) { require 'zlib' }
 
-  def initialize
-    @body = []
+  class IoLikeObject
+    attr_reader :body
+
+    def initialize
+      @body = []
+    end
+
+    def write(str)
+      @body << str
+    end
   end
 
-  def write(str)
-    @body << str
-  end
-end
-
-describe Zlib::GzipWriter do
-  GZIP_MAGIC_1 = 0x1F
-  GZIP_MAGIC_2 = 0X8B
-
   it "doesn't corrupt the output" do
     io_like_object = IoLikeObject.new
     gzip = Zlib::GzipWriter.new(io_like_object)
@@ -31,11 +29,15 @@ describe Zlib::GzipWriter do
     gzip.flush
 
     # first chunk should still be intact (specifically: it should not be overwritten with the second)
-    first_chunk.should == io_like_object.body[0]
+    expect( first_chunk ).to eql io_like_object.body[0]
 
     gzip.close
 
+    gzip_magic_1 = 0x1F
+    gzip_magic_2 = 0X8B
+
     # extra sanity check of the gzip "magic bytes"
-    io_like_object.body[0].bytes.to_a[0..1].should == [GZIP_MAGIC_1, GZIP_MAGIC_2]
+    expect( io_like_object.body[0].bytes.to_a[0..1] ).to eql [gzip_magic_1, gzip_magic_2]
   end
+
 end
\ No newline at end of file
