@@ -1,7 +1,7 @@
 require File.expand_path('../../../spec_helper', __FILE__)
 
 describe "Bignum#**" do
-  before(:each) do
+  before :each do
     @bignum = bignum_value(47)
   end
 
@@ -22,11 +22,8 @@ describe "Bignum#**" do
     flt.infinite?.should == 1
   end
 
-  ruby_version_is '1.9' do
-    it "returns a complex number when negative and raised to a fractional power" do
-      ((-@bignum) ** (1.0/3))      .should be_close(Complex(1048576,1816186.907597341), TOLERANCE)
-      ((-@bignum) ** Rational(1,3)).should be_close(Complex(1048576,1816186.907597341), TOLERANCE)
-    end
+  it "returns a complex number when negative and raised to a fractional power" do
+    ((-@bignum) ** (1.0/3))      .should be_close(Complex(1048576,1816186.907597341), TOLERANCE)
+    ((-@bignum) ** Rational(1,3)).should be_close(Complex(1048576,1816186.907597341), TOLERANCE)
   end
-
 end
