@@ -30,9 +30,7 @@ describe "File.utime" do
     File.mtime(@file2).to_i.should be_close(Time.now.to_i, 2)
   end
 
-  ruby_version_is "1.9" do
-    it "accepts an object that has a #to_path method" do
-      File.utime(@atime, @mtime, mock_to_path(@file1), mock_to_path(@file2))
-    end
+  it "accepts an object that has a #to_path method" do
+    File.utime(@atime, @mtime, mock_to_path(@file1), mock_to_path(@file2))
   end
 end
