@@ -23,7 +23,7 @@ describe "Kernel.at_exit" do
   it "gives access to the last raised exception" do
     code = <<-EOC
       at_exit do
-        puts "The exception matches: #{$! == $exception}"
+        puts "The exception matches: \#{$! == $exception}"
       end
 
       begin
@@ -33,7 +33,7 @@ describe "Kernel.at_exit" do
       end
     EOC
 
-    result = ruby_exe(code, :args => "2>&1", :escape => true)
+    result = ruby_exe(code, args: "2>&1", escape: true)
     result.should =~ /The exception matches: true/
   end
 
