@@ -1,4 +1,4 @@
-describe :string_to_s, :shared => true do
+describe :string_to_s, shared: true do
   it "returns self when self.class == String" do
     a = "a string"
     a.should equal(a.send(@method))
@@ -8,7 +8,7 @@ describe :string_to_s, :shared => true do
     a = StringSpecs::MyString.new("a string")
     s = a.send(@method)
     s.should == "a string"
-    s.should be_kind_of(String)
+    s.should be_an_instance_of(String)
   end
 
   it "taints the result when self is tainted" do
