@@ -20,18 +20,16 @@ describe "Matrix#column" do
     @m.column(-1).should == Vector[3, 4]
   end
 
-  ruby_bug "redmine:1532", "1.8.7" do
-    it "returns self when called with a block" do
-      @m.column(0) { |x| x }.should equal(@m)
-    end
+  it "returns self when called with a block" do
+    @m.column(0) { |x| x }.should equal(@m)
+  end
 
-    it "returns nil when out of bounds" do
-      @m.column(3).should == nil
-    end
+  it "returns nil when out of bounds" do
+    @m.column(3).should == nil
+  end
 
-    it "never yields when out of bounds" do
-      lambda { @m.column(3){ raise } }.should_not raise_error
-      lambda { @m.column(-4){ raise } }.should_not raise_error
-    end
+  it "never yields when out of bounds" do
+    lambda { @m.column(3){ raise } }.should_not raise_error
+    lambda { @m.column(-4){ raise } }.should_not raise_error
   end
 end
