@@ -1,7 +1,7 @@
 require File.expand_path('../../fixtures/classes', __FILE__)
 require 'matrix'
 
-describe :matrix_rectangular, :shared => true do
+describe :matrix_rectangular, shared: true do
   it "returns [receiver.real, receiver.imag]" do
     m = Matrix[ [1.2, Complex(1,2)], [Complex(-2,0.42), 4] ]
     m.send(@method).should == [m.real, m.imag]
@@ -10,11 +10,9 @@ describe :matrix_rectangular, :shared => true do
     m.send(@method).should == [m.real, m.imag]
   end
 
-  ruby_bug "redmine #5307", "1.9.3" do
-    describe "for a subclass of Matrix" do
-      it "returns instances of that subclass" do
-        MatrixSub.ins.send(@method).each{|m| m.should be_an_instance_of(MatrixSub) }
-      end
+  describe "for a subclass of Matrix" do
+    it "returns instances of that subclass" do
+      MatrixSub.ins.send(@method).each{|m| m.should be_an_instance_of(MatrixSub) }
     end
   end
 end
