@@ -1,9 +1,9 @@
 require File.expand_path('../../../../spec_helper', __FILE__)
-require 'net/ftp'
+require File.expand_path('../spec_helper', __FILE__)
 require File.expand_path('../fixtures/server', __FILE__)
 
 describe "Net::FTP#storbinary" do
-  before(:each) do
+  before :each do
     @server = NetFTPSpecs::DummyFTP.new
     @server.serve_once
 
@@ -11,10 +11,10 @@ describe "Net::FTP#storbinary" do
     @tmp_file = tmp("binaryfile", false)
 
     @ftp = Net::FTP.new
-    @ftp.connect("localhost", 9921)
+    @ftp.connect(@server.hostname, @server.server_port)
   end
 
-  after(:each) do
+  after :each do
     @ftp.quit rescue nil
     @ftp.close
     @server.stop
@@ -41,7 +41,7 @@ describe "Net::FTP#storbinary" do
         "This is an", " example f",
         "ile\nwhich ", "is going t",
         "o be trans", "mitted\nusi",
-        "ng #putbin", "aryfile."
+        "ng #putbin", "aryfile.\n"
       ]
     end
   end
