@@ -2,7 +2,7 @@ require File.expand_path('../../../spec_helper', __FILE__)
 require File.expand_path('../fixtures/classes', __FILE__)
 
 describe "Observer#delete_observer" do
-  before(:each) do
+  before :each do
     @observable = ObservableSpecs.new
     @observer = ObserverCallbackSpecs.new
   end
