@@ -1,10 +1,10 @@
 require File.expand_path('../../../../../spec_helper', __FILE__)
 require 'openssl'
 
-describe :openssl_random_bytes, :shared => true do |cmd|
+describe :openssl_random_bytes, shared: true do |cmd|
   it "generates a random binary string of specified length" do
     (1..64).each do |idx|
-      bytes = OpenSSL::Random.pseudo_bytes(idx)
+      bytes = OpenSSL::Random.send(@method, idx)
       bytes.should be_kind_of(String)
       bytes.length.should == idx
     end
@@ -14,7 +14,7 @@ describe :openssl_random_bytes, :shared => true do |cmd|
     # quick and dirty check, but good enough
     values = []
     256.times do
-      val = OpenSSL::Random.pseudo_bytes(16)
+      val = OpenSSL::Random.send(@method, 16)
       # make sure the random bytes are not repeating
       values.include?(val).should == false
       values << val
@@ -23,7 +23,7 @@ describe :openssl_random_bytes, :shared => true do |cmd|
 
   it "raises ArgumentError on negative arguments" do
     lambda {
-      OpenSSL::Random.pseudo_bytes(-1)
+      OpenSSL::Random.send(@method, -1)
     }.should raise_error(ArgumentError)
   end
 end
