@@ -2,7 +2,7 @@ require File.expand_path('../../../spec_helper', __FILE__)
 require File.expand_path('../fixtures/classes', __FILE__)
 
 describe "StringIO#rewind" do
-  before(:each) do
+  before :each do
     @io = StringIO.new("hello\nworld")
     @io.pos = 3
     @io.lineno = 1
