@@ -15,11 +15,9 @@ describe "URI#route_from" do
     URI("mailto:foo@example.com#bar").route_from('mailto:foo@example.com').to_s.should == "#bar"
   end
 
-  ruby_bug "redmine:3506", "1.9.2" do
-    it "accepts a string-like argument" do
-      str = mock('string-like')
-      str.should_receive(:to_str).and_return("http://example.com/b.html")
-      URI("http://example.com/a.html").route_from(str).to_s.should == "a.html"
-    end
+  it "accepts a string-like argument" do
+    str = mock('string-like')
+    str.should_receive(:to_str).and_return("http://example.com/b.html")
+    URI("http://example.com/a.html").route_from(str).to_s.should == "a.html"
   end
 end
