@@ -1,7 +1,7 @@
 platform_is :windows do
   require 'win32ole'
 
-  describe :win32ole_param_name, :shared => true do
+  describe :win32ole_param_name, shared: true do
     before :each do
       ole_type_detail = WIN32OLE_TYPE.new("Microsoft Scripting Runtime", "FileSystemObject")
       m_copyfile = WIN32OLE_METHOD.new(ole_type_detail, "CopyFile")
