@@ -14,19 +14,17 @@ describe "Zlib::GzipWriter#mtime=" do
       gzio.mtime.should == Time.at(1)
     end
 
-    @io.string[4, 4].should == "\001\0\0\0"
+    @io.string[4, 4].should == [1,0,0,0].pack('C*')
   end
 
-  ruby_bug '253', '1.9.0' do
-  it "sets mtime using Time" do
-    Zlib::GzipWriter.wrap @io do |gzio|
-      gzio.mtime = Time.at 1
+it "sets mtime using Time" do
+  Zlib::GzipWriter.wrap @io do |gzio|
+    gzio.mtime = Time.at 1
 
-      gzio.mtime.should == Time.at(1)
-    end
-
-    @io.string[4, 4].should == "\001\0\0\0"
+    gzio.mtime.should == Time.at(1)
   end
+
+  @io.string[4, 4].should == [1,0,0,0].pack('C*')
   end
 
   it "raises if the header was written" do
