@@ -5,9 +5,6 @@
 #undef HAVE_RB_DEFINE_HOOKED_VARIABLE
 #undef HAVE_RB_DEFINE_VARIABLE
 
-#undef HAVE_RB_PROTECT_INSPECT
-#undef HAVE_RB_INSPECTING_P
-
 #undef HAVE_RB_EXEC_RECURSIVE
 
 #endif
