@@ -1,8 +1,9 @@
-#include <string.h>
-
 #include "ruby.h"
 #include "rubyspec.h"
 
+#include <string.h>
+#include <stdarg.h>
+
 #ifdef HAVE_RUBY_ENCODING_H
 #include "ruby/encoding.h"
 #endif
@@ -25,27 +26,6 @@ static VALUE string_spec_rb_cstr_to_inum(VALUE self, VALUE str, VALUE inum, VALU
 }
 #endif
 
-#ifdef HAVE_RB_STR2CSTR
-VALUE string_spec_rb_str2cstr(VALUE self, VALUE str, VALUE return_length) {
-  if(return_length == Qtrue) {
-    long len = 0;
-    char* ptr = rb_str2cstr(str, &len);
-    VALUE ary = rb_ary_new();
-    rb_ary_push(ary, rb_str_new2(ptr));
-    rb_ary_push(ary, INT2FIX(len));
-    return ary;
-  } else {
-    return rb_str_new2(rb_str2cstr(str, NULL));
-  }
-}
-
-VALUE string_spec_rb_str2cstr_replace(VALUE self, VALUE str) {
-  char* ptr = rb_str2cstr(str, NULL);
-  ptr[0] = 'f'; ptr[1] = 'o'; ptr[2] = 'o'; ptr[3] = 0;
-  return Qnil;
-}
-#endif
-
 #ifdef HAVE_RB_STR2INUM
 VALUE string_spec_rb_str2inum(VALUE self, VALUE str, VALUE inum) {
   int num = FIX2INT(inum);
@@ -191,12 +171,6 @@ VALUE string_spec_rb_str_intern(VALUE self, VALUE str) {
 }
 #endif
 
-#ifdef HAVE_RB_STR_LEN
-VALUE string_spec_rb_str_len(VALUE self, VALUE str) {
-  return INT2FIX(rb_str_len(str));
-}
-#endif
-
 #ifdef HAVE_RB_STR_LENGTH
 VALUE string_spec_rb_str_length(VALUE self, VALUE str) {
   return rb_str_length(str);
@@ -210,7 +184,6 @@ VALUE string_spec_rb_str_new(VALUE self, VALUE str, VALUE len) {
 #endif
 
 #ifdef HAVE_RB_STR_NEW2
-#ifdef RUBY_VERSION_IS_1_9
 VALUE string_spec_rb_str_new2(VALUE self, VALUE str) {
   if(NIL_P(str)) {
     return rb_str_new2("");
@@ -218,15 +191,6 @@ VALUE string_spec_rb_str_new2(VALUE self, VALUE str) {
     return rb_str_new2(RSTRING_PTR(str));
   }
 }
-#else
-VALUE string_spec_rb_str_new2(VALUE self, VALUE str) {
-  if(NIL_P(str)) {
-    return rb_str_new2(NULL);
-  } else {
-    return rb_str_new2(RSTRING_PTR(str));
-  }
-}
-#endif
 #endif
 
 #ifdef HAVE_RB_STR_ENCODE
@@ -236,7 +200,6 @@ VALUE string_spec_rb_str_encode(VALUE self, VALUE str, VALUE enc, VALUE flags, V
 #endif
 
 #ifdef HAVE_RB_STR_NEW_CSTR
-#ifdef RUBY_VERSION_IS_1_9
 VALUE string_spec_rb_str_new_cstr(VALUE self, VALUE str) {
   if(NIL_P(str)) {
     return rb_str_new_cstr("");
@@ -244,15 +207,6 @@ VALUE string_spec_rb_str_new_cstr(VALUE self, VALUE str) {
     return rb_str_new_cstr(RSTRING_PTR(str));
   }
 }
-#else
-VALUE string_spec_rb_str_new_cstr(VALUE self, VALUE str) {
-  if(NIL_P(str)) {
-    return rb_str_new_cstr(NULL);
-  } else {
-    return rb_str_new_cstr(RSTRING_PTR(str));
-  }
-}
-#endif
 #endif
 
 #ifdef HAVE_RB_EXTERNAL_STR_NEW
@@ -297,88 +251,21 @@ VALUE string_spec_rb_str_new4(VALUE self, VALUE str) {
 }
 #endif
 
+#ifdef HAVE_RB_STR_NEW5
+VALUE string_spec_rb_str_new5(VALUE self, VALUE str, VALUE ptr, VALUE len) {
+  return rb_str_new5(str, RSTRING_PTR(ptr), FIX2INT(len));
+}
+#endif
+
 #ifdef HAVE_RB_STR_PLUS
 VALUE string_spec_rb_str_plus(VALUE self, VALUE str1, VALUE str2) {
   return rb_str_plus(str1, str2);
 }
 #endif
 
-#ifdef HAVE_RB_STR_PTR
-VALUE string_spec_rb_str_ptr_iterate(VALUE self, VALUE str) {
-  int i;
-  char* ptr;
-
-  ptr = rb_str_ptr(str);
-  for(i = 0; i < RSTRING_LEN(str); i++) {
-    rb_yield(INT2FIX(ptr[i]));
-  }
-  return Qnil;
-}
-
-VALUE string_spec_rb_str_ptr_assign(VALUE self, VALUE str, VALUE chr) {
-  int i;
-  char c;
-  char* ptr;
-
-  ptr = rb_str_ptr(str);
-  c = FIX2INT(chr);
-
-  for(i = 0; i < RSTRING_LEN(str); i++) {
-    ptr[i] = c;
-  }
-  return Qnil;
-}
-
-VALUE string_spec_rb_str_ptr_assign_call(VALUE self, VALUE str) {
-  char *ptr = rb_str_ptr(str);
-
-  ptr[1] = 'x';
-  rb_str_concat(str, rb_str_new2("d"));
-  return str;
-}
-
-VALUE string_spec_rb_str_ptr_assign_funcall(VALUE self, VALUE str) {
-  char *ptr = rb_str_ptr(str);
-
-  ptr[1] = 'x';
-  rb_str_flush(str);
-  rb_funcall(str, rb_intern("<<"), 1, rb_str_new2("e"));
-  return str;
-}
-#endif
-
-#ifdef HAVE_RB_STR_PTR_READONLY
-VALUE string_spec_rb_str_ptr_readonly_iterate(VALUE self, VALUE str) {
-  int i;
-  char* ptr;
-
-  ptr = rb_str_ptr_readonly(str);
-  for(i = 0; i < RSTRING_LEN(str); i++) {
-    rb_yield(INT2FIX(ptr[i]));
-  }
-  return Qnil;
-}
-
-VALUE string_spec_rb_str_ptr_readonly_assign(VALUE self, VALUE str, VALUE chr) {
-  int i;
-  char c;
-  char* ptr;
-
-  ptr = rb_str_ptr_readonly(str);
-  c = FIX2INT(chr);
-
-  for(i = 0; i < RSTRING_LEN(str); i++) {
-    ptr[i] = c;
-  }
-  return Qnil;
-}
-
-VALUE string_spec_rb_str_ptr_readonly_append(VALUE self, VALUE str, VALUE more) {
-  char *ptr = rb_str_ptr_readonly(str);
-
-  rb_str_concat(str, more);
-
-  return rb_str_new2(ptr);
+#ifdef HAVE_RB_STR_TIMES
+VALUE string_spec_rb_str_times(VALUE self, VALUE str, VALUE times) {
+  return rb_str_times(str, times);
 }
 #endif
 
@@ -417,60 +304,6 @@ VALUE string_spec_rb_str_to_str(VALUE self, VALUE arg) {
 }
 #endif
 
-#ifdef HAVE_RSTRING
-VALUE string_spec_RSTRING_len(VALUE self, VALUE str) {
-  return INT2FIX(RSTRING(str)->len);
-}
-
-VALUE string_spec_RSTRING_ptr_iterate(VALUE self, VALUE str) {
-  int i;
-  char* ptr;
-
-  ptr = RSTRING(str)->ptr;
-  for(i = 0; i < RSTRING_LEN(str); i++) {
-    rb_yield(INT2FIX(ptr[i]));
-  }
-  return Qnil;
-}
-
-VALUE string_spec_RSTRING_ptr_assign(VALUE self, VALUE str, VALUE chr) {
-  int i;
-  char c;
-  char* ptr;
-
-  ptr = RSTRING(str)->ptr;
-  c = FIX2INT(chr);
-
-  for(i = 0; i < RSTRING_LEN(str); i++) {
-    ptr[i] = c;
-  }
-  return Qnil;
-}
-
-VALUE string_spec_RSTRING_ptr_assign_call(VALUE self, VALUE str) {
-  char *ptr = RSTRING(str)->ptr;
-
-  ptr[1] = 'x';
-  RSTRING(str)->len = 2;
-  rb_str_concat(str, rb_str_new2("d"));
-  return str;
-}
-
-VALUE string_spec_RSTRING_ptr_assign_funcall(VALUE self, VALUE str) {
-  char *ptr = RSTRING(str)->ptr;
-
-  ptr[1] = 'x';
-  rb_funcall(str, rb_intern("<<"), 1, rb_str_new2("e"));
-  return str;
-}
-
-VALUE string_spec_RSTRING_ptr_write(VALUE self, VALUE str, VALUE text) {
-  strcpy(RSTRING(str)->ptr, RSTRING_PTR(text));
-  RSTRING(str)->len = RSTRING_LEN(text);
-  return Qnil;
-}
-#endif
-
 #ifdef HAVE_RSTRING_LEN
 VALUE string_spec_RSTRING_LEN(VALUE self, VALUE str) {
   return INT2FIX(RSTRING_LEN(str));
@@ -490,7 +323,7 @@ VALUE string_spec_RSTRING_PTR_iterate(VALUE self, VALUE str) {
 
   ptr = RSTRING_PTR(str);
   for(i = 0; i < RSTRING_LEN(str); i++) {
-    rb_yield(INT2FIX(ptr[i]));
+    rb_yield(CHR2FIX(ptr[i]));
   }
   return Qnil;
 }
@@ -519,18 +352,6 @@ VALUE string_spec_RSTRING_PTR_after_funcall(VALUE self, VALUE str, VALUE cb) {
 }
 #endif
 
-#ifdef HAVE_STR2CSTR
-VALUE string_spec_STR2CSTR(VALUE self, VALUE str) {
-  return rb_str_new2(STR2CSTR(str));
-}
-
-VALUE string_spec_STR2CSTR_replace(VALUE self, VALUE str) {
-  char* ptr = STR2CSTR(str);
-  ptr[0] = 'f'; ptr[1] = 'o'; ptr[2] = 'o'; ptr[3] = 0;
-  return Qnil;
-}
-#endif
-
 #ifdef HAVE_STRINGVALUE
 VALUE string_spec_StringValue(VALUE self, VALUE str) {
   return StringValue(str);
@@ -539,7 +360,27 @@ VALUE string_spec_StringValue(VALUE self, VALUE str) {
 
 #ifdef HAVE_RB_STR_HASH
 static VALUE string_spec_rb_str_hash(VALUE self, VALUE str) {
-  return INT2FIX(rb_str_hash(str));
+  st_index_t val = rb_str_hash(str);
+
+#if SIZEOF_LONG == SIZEOF_VOIDP || SIZEOF_LONG_LONG == SIZEOF_VOIDP
+  return LONG2FIX((long)val);
+#else
+# error unsupported platform
+#endif
+}
+#endif
+
+#ifdef HAVE_RB_STR_UPDATE
+static VALUE string_spec_rb_str_update(VALUE self, VALUE str, VALUE beg, VALUE end, VALUE replacement) {
+  rb_str_update(str, FIX2LONG(beg), FIX2LONG(end), replacement);
+  return str;
+}
+#endif
+
+#ifdef HAVE_RB_STR_FREE
+static VALUE string_spec_rb_str_free(VALUE self, VALUE str) {
+  rb_str_free(str);
+  return Qnil;
 }
 #endif
 
@@ -552,6 +393,24 @@ static VALUE string_spec_rb_sprintf2(VALUE self, VALUE str, VALUE repl1, VALUE r
 }
 #endif
 
+#ifdef HAVE_RB_VSPRINTF
+static VALUE string_spec_rb_vsprintf_worker(char* fmt, ...) {
+  va_list varargs;
+  VALUE str;
+
+  va_start(varargs, fmt);
+  str = rb_vsprintf(fmt, varargs);
+  va_end(varargs);
+
+  return str;
+}
+
+static VALUE string_spec_rb_vsprintf(VALUE self, VALUE fmt, VALUE str, VALUE i, VALUE f) {
+  return string_spec_rb_vsprintf_worker(RSTRING_PTR(fmt), RSTRING_PTR(str),
+      FIX2INT(i), RFLOAT_VALUE(f));
+}
+#endif
+
 #ifdef HAVE_RB_STR_EQUAL
 VALUE string_spec_rb_str_equal(VALUE self, VALUE str1, VALUE str2) {
   return rb_str_equal(str1, str2);
@@ -570,7 +429,13 @@ static VALUE string_spec_rb_usascii_str_new_cstr(VALUE self, VALUE str) {
 }
 #endif
 
-void Init_string_spec() {
+#ifdef HAVE_RB_STRING
+static VALUE string_spec_rb_String(VALUE self, VALUE val) {
+  return rb_String(val);
+}
+#endif
+
+void Init_string_spec(void) {
   VALUE cls;
   cls = rb_define_class("CApiStringSpecs", rb_cObject);
 
@@ -582,11 +447,6 @@ void Init_string_spec() {
   rb_define_method(cls, "rb_cstr_to_inum", string_spec_rb_cstr_to_inum, 3);
 #endif
 
-#ifdef HAVE_RB_STR2CSTR
-  rb_define_method(cls, "rb_str2cstr", string_spec_rb_str2cstr, 2);
-  rb_define_method(cls, "rb_str2cstr_replace", string_spec_rb_str2cstr_replace, 1);
-#endif
-
 #ifdef HAVE_RB_STR2INUM
   rb_define_method(cls, "rb_str2inum", string_spec_rb_str2inum, 2);
 #endif
@@ -639,9 +499,6 @@ void Init_string_spec() {
   rb_define_method(cls, "rb_str_dup", string_spec_rb_str_dup, 1);
 #endif
 
-#ifdef HAVE_RB_STR_FLUSH
-#endif
-
 #ifdef HAVE_RB_STR_FREEZE
   rb_define_method(cls, "rb_str_freeze", string_spec_rb_str_freeze, 1);
 #endif
@@ -654,10 +511,6 @@ void Init_string_spec() {
   rb_define_method(cls, "rb_str_intern", string_spec_rb_str_intern, 1);
 #endif
 
-#ifdef HAVE_RB_STR_LEN
-  rb_define_method(cls, "rb_str_len", string_spec_rb_str_len, 1);
-#endif
-
 #ifdef HAVE_RB_STR_LENGTH
   rb_define_method(cls, "rb_str_length", string_spec_rb_str_length, 1);
 #endif
@@ -707,25 +560,16 @@ void Init_string_spec() {
   rb_define_method(cls, "rb_str_new4", string_spec_rb_str_new4, 1);
 #endif
 
+#ifdef HAVE_RB_STR_NEW5
+  rb_define_method(cls, "rb_str_new5", string_spec_rb_str_new5, 3);
+#endif
+
 #ifdef HAVE_RB_STR_PLUS
   rb_define_method(cls, "rb_str_plus", string_spec_rb_str_plus, 2);
 #endif
 
-#ifdef HAVE_RB_STR_PTR
-  rb_define_method(cls, "rb_str_ptr_iterate", string_spec_rb_str_ptr_iterate, 1);
-  rb_define_method(cls, "rb_str_ptr_assign", string_spec_rb_str_ptr_assign, 2);
-  rb_define_method(cls, "rb_str_ptr_assign_call", string_spec_rb_str_ptr_assign_call, 1);
-  rb_define_method(cls, "rb_str_ptr_assign_funcall",
-      string_spec_rb_str_ptr_assign_funcall, 1);
-#endif
-
-#ifdef HAVE_RB_STR_PTR_READONLY
-  rb_define_method(cls, "rb_str_ptr_readonly_iterate",
-      string_spec_rb_str_ptr_readonly_iterate, 1);
-  rb_define_method(cls, "rb_str_ptr_readonly_assign",
-      string_spec_rb_str_ptr_readonly_assign, 2);
-  rb_define_method(cls, "rb_str_ptr_readonly_append",
-      string_spec_rb_str_ptr_readonly_append, 2);
+#ifdef HAVE_RB_STR_TIMES
+  rb_define_method(cls, "rb_str_times", string_spec_rb_str_times, 2);
 #endif
 
 #ifdef HAVE_RB_STR_RESIZE
@@ -756,16 +600,6 @@ void Init_string_spec() {
   rb_define_method(cls, "rb_str_to_str", string_spec_rb_str_to_str, 1);
 #endif
 
-#ifdef HAVE_RSTRING
-  rb_define_method(cls, "RSTRING_ptr_iterate", string_spec_RSTRING_ptr_iterate, 1);
-  rb_define_method(cls, "RSTRING_ptr_assign", string_spec_RSTRING_ptr_assign, 2);
-  rb_define_method(cls, "RSTRING_ptr_assign_call", string_spec_RSTRING_ptr_assign_call, 1);
-  rb_define_method(cls, "RSTRING_ptr_write", string_spec_RSTRING_ptr_write, 2);
-  rb_define_method(cls, "RSTRING_ptr_assign_funcall",
-      string_spec_RSTRING_ptr_assign_funcall, 1);
-  rb_define_method(cls, "RSTRING_len", string_spec_RSTRING_len, 1);
-#endif
-
 #ifdef HAVE_RSTRING_LEN
   rb_define_method(cls, "RSTRING_LEN", string_spec_RSTRING_LEN, 1);
 #endif
@@ -781,11 +615,6 @@ void Init_string_spec() {
       string_spec_RSTRING_PTR_after_funcall, 2);
 #endif
 
-#ifdef HAVE_STR2CSTR
-  rb_define_method(cls, "STR2CSTR", string_spec_STR2CSTR, 1);
-  rb_define_method(cls, "STR2CSTR_replace", string_spec_STR2CSTR_replace, 1);
-#endif
-
 #ifdef HAVE_STRINGVALUE
   rb_define_method(cls, "StringValue", string_spec_StringValue, 1);
 #endif
@@ -794,11 +623,23 @@ void Init_string_spec() {
   rb_define_method(cls, "rb_str_hash", string_spec_rb_str_hash, 1);
 #endif
 
+#ifdef HAVE_RB_STR_UPDATE
+  rb_define_method(cls, "rb_str_update", string_spec_rb_str_update, 4);
+#endif
+
+#ifdef HAVE_RB_STR_FREE
+  rb_define_method(cls, "rb_str_free", string_spec_rb_str_free, 1);
+#endif
+
 #ifdef HAVE_RB_SPRINTF
   rb_define_method(cls, "rb_sprintf1", string_spec_rb_sprintf1, 2);
   rb_define_method(cls, "rb_sprintf2", string_spec_rb_sprintf2, 3);
 #endif
 
+#ifdef HAVE_RB_VSPRINTF
+  rb_define_method(cls, "rb_vsprintf", string_spec_rb_vsprintf, 4);
+#endif
+
 #ifdef HAVE_RB_STR_EQUAL
   rb_define_method(cls, "rb_str_equal", string_spec_rb_str_equal, 2);
 #endif
@@ -810,6 +651,10 @@ void Init_string_spec() {
 #ifdef HAVE_RB_USASCII_STR_NEW_CSTR
   rb_define_method(cls, "rb_usascii_str_new_cstr", string_spec_rb_usascii_str_new_cstr, 1);
 #endif
+
+#ifdef HAVE_RB_STRING
+  rb_define_method(cls, "rb_String", string_spec_rb_String, 1);
+#endif
 }
 
 #ifdef __cplusplus
