@@ -1,18 +1,14 @@
 require File.expand_path('../../../../spec_helper', __FILE__)
 
-describe :float_arg, :shared => true do
-  ruby_bug "#1715", "1.8.6.369" do
-    it "returns NaN if NaN" do
-      f = nan_value
-      f.send(@method).nan?.should be_true
-    end
+describe :float_arg, shared: true do
+  it "returns NaN if NaN" do
+    f = nan_value
+    f.send(@method).nan?.should be_true
   end
 
-  ruby_version_is "1.9" do
-    it "returns self if NaN" do
-      f = nan_value
-      f.send(@method).should equal(f)
-    end
+  it "returns self if NaN" do
+    f = nan_value
+    f.send(@method).should equal(f)
   end
 
   it "returns 0 if positive" do
