@@ -1,4 +1,4 @@
-describe :complex_real, :shared => true do
+describe :complex_real, shared: true do
   it "returns the real part of self" do
     Complex(1, 0).real.should == 1
     Complex(2, 1).real.should == 2
