@@ -1,26 +1,26 @@
 require File.expand_path('../../../spec_helper', __FILE__)
 
-describe :enum_new, :shared => true do
+describe :enum_new, shared: true do
   it "creates a new custom enumerator with the given object, iterator and arguments" do
-    enum = enumerator_class.new(1, :upto, 3)
-    enum.should be_an_instance_of(enumerator_class)
+    enum = Enumerator.new(1, :upto, 3)
+    enum.should be_an_instance_of(Enumerator)
   end
 
   it "creates a new custom enumerator that responds to #each" do
-    enum = enumerator_class.new(1, :upto, 3)
+    enum = Enumerator.new(1, :upto, 3)
     enum.respond_to?(:each).should == true
   end
 
   it "creates a new custom enumerator that runs correctly" do
-    enumerator_class.new(1, :upto, 3).map{|x|x}.should == [1,2,3]
+    Enumerator.new(1, :upto, 3).map{|x|x}.should == [1,2,3]
   end
 
   it "aliases the second argument to :each" do
-    enumerator_class.new(1..2).to_a.should == enumerator_class.new(1..2, :each).to_a
+    Enumerator.new(1..2).to_a.should == Enumerator.new(1..2, :each).to_a
   end
 
   it "doesn't check for the presence of the iterator method" do
-    enumerator_class.new(nil).should be_an_instance_of(enumerator_class)
+    Enumerator.new(nil).should be_an_instance_of(Enumerator)
   end
 
   it "uses the latest define iterator method" do
@@ -29,7 +29,7 @@ describe :enum_new, :shared => true do
         yield :foo
       end
     end
-    enum = enumerator_class.new(StrangeEach.new)
+    enum = Enumerator.new(StrangeEach.new)
     enum.to_a.should == [:foo]
     class StrangeEach
       def each
