@@ -1,6 +1,6 @@
 require File.expand_path('../../../spec_helper', __FILE__)
 
-describe :enum_with_object, :shared => true do
+describe :enum_with_object, shared: true do
   before :each do
     @enum = [:a, :b].to_enum
     @memo = ''
@@ -35,8 +35,8 @@ describe :enum_with_object, :shared => true do
   context "without block" do
     it "returns new Enumerator" do
       ret = @enum.send(@method, @memo)
-      ret.should be_an_instance_of(enumerator_class)
+      ret.should be_an_instance_of(Enumerator)
       ret.should_not equal(@enum)
     end
   end
-end
\ No newline at end of file
+end
