@@ -1,8 +1,8 @@
-describe :file_readable, :shared => true do
+describe :file_readable, shared: true do
   before :each do
     @file = tmp('i_exist')
     platform_is :windows do
-      @file2 = "C:\\windows\\notepad.exe"
+      @file2 = File.join(ENV["WINDIR"], "system32/drivers/etc/services").tr(File::SEPARATOR, File::ALT_SEPARATOR)
     end
     platform_is_not :windows do
       @file2 = "/etc/passwd"
@@ -18,14 +18,12 @@ describe :file_readable, :shared => true do
     File.open(@file,'w') { @object.send(@method, @file).should == true }
   end
 
-  ruby_version_is "1.9" do
-    it "accepts an object that has a #to_path method" do
-      @object.send(@method, mock_to_path(@file2)).should == true
-    end
+  it "accepts an object that has a #to_path method" do
+    @object.send(@method, mock_to_path(@file2)).should == true
   end
 end
 
-describe :file_readable_missing, :shared => true do
+describe :file_readable_missing, shared: true do
   it "returns false if the file does not exist" do
     @object.send(@method, 'fake_file').should == false
   end
